@@ -1,9 +1,12 @@
 """Core app functionality."""
 
 from nautobot.core.signals import nautobot_database_ready
+from nautobot.core.settings_funcs import ConstanceConfigItem
 from nautobot.extras.plugins import NautobotAppConfig
 
+
 __all__ = (
+    "ConstanceConfigItem",
     "nautobot_database_ready",
     "NautobotAppConfig",
 )
