@@ -1,31 +1,46 @@
 """Utilities for apps to integrate with and extend the existing Nautobot UI."""
 
 from nautobot.core.apps import (
+    HomePageBase,
+    HomePageGroup,
     HomePageItem,
     HomePagePanel,
+    NavContext,
+    NavGrouping,
+    NavItem,
     NavMenuAddButton,
+    NavMenuBase,
     NavMenuButton,
     NavMenuGroup,
     NavMenuImportButton,
     NavMenuItem,
     NavMenuTab,
+    PermissionsMixin,
 )
+from nautobot.core.choices import ButtonColorChoices
 from nautobot.extras.choices import BannerClassChoices
 from nautobot.extras.plugins import Banner
 from nautobot.extras.plugins import TemplateExtension
-from nautobot.utilities.choices import ButtonColorChoices
+
 
 __all__ = (
     "Banner",
     "BannerClassChoices",
     "ButtonColorChoices",
+    "HomePageBase",
+    "HomePageGroup",
     "HomePageItem",
     "HomePagePanel",
+    "NavContext",
+    "NavGrouping",
+    "NavItem",
     "NavMenuAddButton",
+    "NavMenuBase",
     "NavMenuButton",
     "NavMenuGroup",
     "NavMenuImportButton",
     "NavMenuItem",
     "NavMenuTab",
+    "PermissionsMixin",
     "TemplateExtension",
 )
