@@ -1,4 +1,4 @@
-from nautobot.utilities.choices import ChoiceSet
+from nautobot.core.choices import ChoiceSet
 
 
 #
@@ -7,7 +7,6 @@ from nautobot.utilities.choices import ChoiceSet
 
 
 class CircuitStatusChoices(ChoiceSet):
-
     STATUS_DEPROVISIONING = "deprovisioning"
     STATUS_ACTIVE = "active"
     STATUS_PLANNED = "planned"
@@ -24,16 +23,6 @@ class CircuitStatusChoices(ChoiceSet):
         (STATUS_DECOMMISSIONED, "Decommissioned"),
     )
 
-    # 2.0 TODO: No longer used (see extras.management.COLOR_MAP instead), remove in 2.0
-    CSS_CLASSES = {
-        STATUS_DEPROVISIONING: "warning",
-        STATUS_ACTIVE: "success",
-        STATUS_PLANNED: "info",
-        STATUS_PROVISIONING: "primary",
-        STATUS_OFFLINE: "danger",
-        STATUS_DECOMMISSIONED: "default",
-    }
-
 
 #
 # CircuitTerminations
@@ -41,7 +30,6 @@ class CircuitStatusChoices(ChoiceSet):
 
 
 class CircuitTerminationSideChoices(ChoiceSet):
-
     SIDE_A = "A"
     SIDE_Z = "Z"
 
