@@ -1,4 +1,13 @@
-from nautobot.core.apps import NavMenuAddButton, NavMenuGroup, NavMenuItem, NavMenuImportButton, NavMenuTab
+from nautobot.core.apps import (
+    NavContext,
+    NavGrouping,
+    NavItem,
+    NavMenuAddButton,
+    NavMenuGroup,
+    NavMenuItem,
+    NavMenuImportButton,
+    NavMenuTab,
+)
 
 
 menu_items = (
@@ -109,3 +118,57 @@ menu_items = (
         ),
     ),
 )
+
+navigation = (
+    NavContext(
+        name="Inventory",
+        groups=(
+            NavGrouping(
+                name="Circuits",
+                weight=400,
+                items=(
+                    NavItem(
+                        link="circuits:circuit_list",
+                        name="Circuits",
+                        weight=100,
+                        permissions=[
+                            "circuits.view_circuit",
+                        ],
+                    ),
+                    NavItem(
+                        link="circuits:circuittermination_list",
+                        name="Circuit Terminations",
+                        weight=200,
+                        permissions=[
+                            "circuits.view_circuittermination",
+                        ],
+                    ),
+                    NavItem(
+                        link="circuits:circuittype_list",
+                        name="Circuit Types",
+                        weight=300,
+                        permissions=[
+                            "circuits.view_circuittype",
+                        ],
+                    ),
+                    NavItem(
+                        link="circuits:provider_list",
+                        name="Providers",
+                        weight=400,
+                        permissions=[
+                            "circuits.view_provider",
+                        ],
+                    ),
+                    NavItem(
+                        link="circuits:providernetwork_list",
+                        name="Provider Networks",
+                        weight=500,
+                        permissions=[
+                            "circuits.view_providernetwork",
+                        ],
+                    ),
+                ),
+            ),
+        ),
+    ),
+)
