@@ -3,10 +3,10 @@ import uuid
 from django.contrib.contenttypes.models import ContentType
 
 from nautobot.circuits.models import Circuit, CircuitTermination, CircuitType, Provider
-from nautobot.dcim.models import PowerPanel, Site
+from nautobot.core.testing.integration import SeleniumTestCase
+from nautobot.dcim.models import PowerPanel, Location, LocationType
 from nautobot.extras.choices import RelationshipTypeChoices
 from nautobot.extras.models import Relationship, RelationshipAssociation, Status
-from nautobot.utilities.testing.integration import SeleniumTestCase
 
 
 class CircuitRelationshipsTestCase(SeleniumTestCase):
@@ -19,47 +19,45 @@ class CircuitRelationshipsTestCase(SeleniumTestCase):
         self.user.is_superuser = True
         self.user.save()
         self.login(self.user.username, self.password)
-        site_ct = ContentType.objects.get_for_model(Site)
+        location_type = LocationType.objects.get(name="Campus")
+        location_ct = ContentType.objects.get_for_model(Location)
         circuit_termination_ct = ContentType.objects.get_for_model(CircuitTermination)
         provider_ct = ContentType.objects.get_for_model(Provider)
         power_panel_ct = ContentType.objects.get_for_model(PowerPanel)
-        active_circuit_status = Status.objects.get_for_model(Circuit).get(slug="active")
-        active_site_status = Status.objects.get_for_model(Site).get(slug="active")
+        circuit_status = Status.objects.get_for_model(Circuit).first()
+        location_status = Status.objects.get_for_model(Location).first()
         provider1 = Provider.objects.create(
             name="Test Provider 1",
-            slug="test-provider-1",
         )
         provider2 = Provider.objects.create(
             name="Test Provider 2",
-            slug="test-provider-2",
         )
         circuit_type = CircuitType.objects.create(
             name="Test Circuit Type",
-            slug="test-circuit-type",
         )
         circuit = Circuit.objects.create(
             provider=provider1,
             cid="1234",
-            type=circuit_type,
-            status=active_circuit_status,
+            circuit_type=circuit_type,
+            status=circuit_status,
         )
-        site = Site.objects.create(
-            name="Test Site",
-            slug="test-site",
-            status=active_site_status,
+        location = Location.objects.create(
+            name="Test Location",
+            status=location_status,
+            location_type=location_type,
         )
         circuit_termination = CircuitTermination.objects.create(
             circuit=circuit,
             term_side="A",
-            site=site,
+            location=location,
         )
         power_panel = PowerPanel.objects.create(
-            site=site,
+            location=location,
             name="Test Power Panel",
         )
         m2m = Relationship.objects.create(
-            name="Termination 2 Provider m2m",
-            slug="termination-2-provider-m2m",
+            label="Termination 2 Provider m2m",
+            key="termination_2_provider_m2m",
             source_type=circuit_termination_ct,
             destination_type=provider_ct,
             type=RelationshipTypeChoices.TYPE_MANY_TO_MANY,
@@ -75,20 +73,20 @@ class CircuitRelationshipsTestCase(SeleniumTestCase):
             destination=provider2,
         )
         o2m = Relationship.objects.create(
-            name="Termination 2 Site o2m",
-            slug="termination-2-provider-o2m",
+            label="Termination 2 Location o2m",
+            key="termination_2_provider_o2m",
             source_type=circuit_termination_ct,
-            destination_type=site_ct,
+            destination_type=location_ct,
             type=RelationshipTypeChoices.TYPE_ONE_TO_MANY,
         )
         RelationshipAssociation.objects.create(
             relationship=o2m,
             source=circuit_termination,
-            destination=site,
+            destination=location,
         )
         o2o = Relationship.objects.create(
-            name="Termination 2 Power Panel o2o",
-            slug="termination-2-power-panel-o2o",
+            label="Termination 2 Power Panel o2o",
+            key="termination_2_power_panel_o2o",
             source_type=circuit_termination_ct,
             destination_type=power_panel_ct,
             type=RelationshipTypeChoices.TYPE_ONE_TO_ONE,
@@ -101,7 +99,7 @@ class CircuitRelationshipsTestCase(SeleniumTestCase):
         # https://github.com/nautobot/nautobot/issues/2077
         fake_ct = ContentType.objects.create(app_label="nonexistent", model="nonexistentmodel")
         bad_relation = Relationship.objects.create(
-            name="Termination 2 Nonexistent",
+            label="Termination 2 Nonexistent",
             source_type=circuit_termination_ct,
             destination_type=fake_ct,
             type=RelationshipTypeChoices.TYPE_MANY_TO_MANY,
@@ -124,7 +122,7 @@ class CircuitRelationshipsTestCase(SeleniumTestCase):
         self.browser.visit(self.live_server_url)
 
         # Click Circuits dropdown button
-        self.browser.links.find_by_partial_text("Circuits")[0].click()
+        self.browser.links.find_by_partial_text("Circuits").click()
 
         # Click Circuits link
         self.browser.links.find_by_partial_text("Circuits")[1].click()
@@ -135,5 +133,5 @@ class CircuitRelationshipsTestCase(SeleniumTestCase):
         # Verify custom relationships are visible
         self.assertTrue(self.browser.is_text_present("Power Panel"))
         self.assertTrue(self.browser.is_text_present("2 providers"))
-        self.assertTrue(self.browser.is_text_present("1 site"))
+        self.assertTrue(self.browser.is_text_present("1 location"))
         self.assertTrue(self.browser.is_text_present("1 nonexistentmodel(s)"))
