@@ -10,16 +10,18 @@ from nautobot.core.api.views import (
     APIRootView,
     GetFilterSetFieldDOMElementAPIView,
     GetFilterSetFieldLookupExpressionChoicesAPIView,
+    GetMenuAPIView,
+    GetObjectCountsView,
+    GetSettingsView,
     GraphQLDRFAPIView,
-    StatusView,
     NautobotSpectacularSwaggerView,
     NautobotSpectacularRedocView,
+    NewUIReadyRoutesAPIView,
+    StatusView,
 )
 from nautobot.extras.plugins.urls import plugin_api_patterns
 
-
 core_api_patterns = [
-    # Lookup Expr
     path(
         "filterset-fields/lookup-choices/",
         GetFilterSetFieldLookupExpressionChoicesAPIView.as_view(),
@@ -31,6 +33,14 @@ core_api_patterns = [
         name="filtersetfield-retrieve-lookupvaluedomelement",
     ),
 ]
+ui_api_patterns = [
+    # Lookup Expr
+    path("core/", include((core_api_patterns, "core-api"))),
+    path("new-ui-ready-routes/", NewUIReadyRoutesAPIView.as_view(), name="new-ui-ready-routes"),
+    path("get-menu/", GetMenuAPIView.as_view(), name="get-menu"),
+    path("get-object-counts/", GetObjectCountsView.as_view(), name="get-object-counts"),
+    path("settings/", GetSettingsView.as_view(), name="settings"),
+]
 
 urlpatterns = [
     # Base views
@@ -52,6 +62,8 @@ urlpatterns = [
     path("graphql/", GraphQLDRFAPIView.as_view(), name="graphql-api"),
     # Plugins
     path("plugins/", include((plugin_api_patterns, "plugins-api"))),
-    # Core
+    # Core, keeping for backwards compatibility of the legacy UI (Dynamic Filter Form)
     path("core/", include((core_api_patterns, "core-api"))),
+    # UI
+    path("ui/", include((ui_api_patterns, "ui-api"))),
 ]
