@@ -5,23 +5,28 @@ from abc import ABC, abstractmethod
 from collections import OrderedDict
 
 from django.apps import AppConfig, apps as global_apps
+from django.contrib.admin.sites import NotRegistered
 from django.db.models import JSONField, BigIntegerField, BinaryField
 from django.db.models.signals import post_migrate
 from django.urls import reverse
 from django.urls.exceptions import NoReverseMatch
+from django.utils.module_loading import import_string
 
 from constance.apps import ConstanceConfig
 from graphene.types import generic, String
 
+from nautobot.core.choices import ButtonActionColorChoices, ButtonActionIconChoices
 from nautobot.core.signals import nautobot_database_ready
-from nautobot.extras.plugins.utils import import_object
+from nautobot.core.utils.navigation import get_all_new_ui_ready_routes
 from nautobot.extras.registry import registry
-from nautobot.utilities.choices import ButtonActionColorChoices, ButtonActionIconChoices
 
 
-logger = logging.getLogger("nautobot.core.apps")
+logger = logging.getLogger(__name__)
 registry["nav_menu"] = {"tabs": {}}
+registry["new_ui_nav_menu"] = {}
 registry["homepage_layout"] = {"panels": {}}
+registry["new_ui_ready_routes"] = set()
+NAV_CONTEXT_NAMES = ("Inventory", "Networks", "Security", "Automation", "Platform")
 
 
 class NautobotConfig(AppConfig):
@@ -36,18 +41,37 @@ class NautobotConfig(AppConfig):
 
     homepage_layout = "homepage.layout"
     menu_tabs = "navigation.menu_items"
+    # New UI Navigation
+    navigation = "navigation.navigation"
+    searchable_models = []  # models included in global search; list of ["modelname", "modelname", "modelname"...]
 
     def ready(self):
         """
         Ready function initiates the import application.
         """
-        homepage_layout = import_object(f"{self.name}.{self.homepage_layout}")
-        if homepage_layout is not None:
+        try:
+            homepage_layout = import_string(f"{self.name}.{self.homepage_layout}")
             register_homepage_panels(self.path, self.label, homepage_layout)
+        except ModuleNotFoundError:
+            pass
 
-        menu_items = import_object(f"{self.name}.{self.menu_tabs}")
-        if menu_items is not None:
+        try:
+            menu_items = import_string(f"{self.name}.{self.menu_tabs}")
             register_menu_items(menu_items)
+        except ModuleNotFoundError:
+            pass
+
+        try:
+            navigation = import_string(f"{self.name}.{self.navigation}")
+            register_new_ui_menu_items(navigation)
+        except ModuleNotFoundError:
+            pass
+
+        try:
+            registry["new_ui_ready_routes"].update(get_all_new_ui_ready_routes())
+        except NotRegistered:
+            # NOTE: Catch this error: The "Tag" model is not registered, which may be related to the admin not registering Tags. Further research is needed on this.
+            pass
 
 
 def create_or_check_entry(grouping, record, key, path):
@@ -55,9 +79,15 @@ def create_or_check_entry(grouping, record, key, path):
         grouping[key] = record.initial_dict
     else:
         for attr, value in record.fixed_fields:
-            if grouping[key][attr]:
+            if grouping[key][attr] != value:
                 logger.error("Unable to redefine %s on %s from %s to %s", attr, path, grouping[key][attr], value)
 
+    if isinstance(record, (NavContext, NavGrouping)):
+        groups = record.groups if isinstance(record, NavContext) else record.items
+        for item in groups:
+            create_or_check_entry(grouping[key]["data"], item, item.name, f"{path} -> {item.name}")
+            grouping[key]["data"] = dict(sorted(grouping[key]["data"].items(), key=lambda x: x[1]["weight"]))
+
 
 def register_menu_items(tab_list):
     """
@@ -78,7 +108,6 @@ def register_menu_items(tab_list):
 
                 group_perms = set()
                 for item in group.items:
-
                     # Instead of passing the reverse url strings, we pass in the url itself initialized with args and kwargs.
                     try:
                         item.link = reverse(item.link, args=item.args, kwargs=item.kwargs)
@@ -134,6 +163,11 @@ def register_menu_items(tab_list):
         )
 
 
+def register_new_ui_menu_items(context_list):
+    for nav_context in context_list:
+        create_or_check_entry(registry["new_ui_nav_menu"], nav_context, nav_context.name, nav_context.name)
+
+
 def register_homepage_panels(path, label, homepage_layout):
     """
     Register homepage panels using `homepage.py`.
@@ -230,12 +264,14 @@ class NavMenuBase(ABC):  # replaces PermissionsMixin
 
     @property
     @abstractmethod
-    def initial_dict(self):  # to be implemented by each subclass
+    def initial_dict(self) -> dict:  # to be implemented by each subclass
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
         return {}
 
     @property
     @abstractmethod
-    def fixed_fields(self):  # to be implemented by subclass
+    def fixed_fields(self) -> tuple:  # to be implemented by subclass
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
         return ()
 
 
@@ -246,14 +282,13 @@ class PermissionsMixin:
         """Ensure permissions."""
         if permissions is not None and not isinstance(permissions, (list, tuple)):
             raise TypeError("Permissions must be passed as a tuple or list.")
-        self.permissions = permissions
+        self.permissions = set(permissions) if permissions else set()
 
 
 class HomePagePanel(HomePageBase, PermissionsMixin):
     """Defines properties that can be used for a panel."""
 
-    permissions = []
-    items = []
+    items = None
     template_path = None
 
     @property
@@ -283,13 +318,10 @@ class HomePagePanel(HomePageBase, PermissionsMixin):
             items (list): List of items to be rendered in this panel.
             weight (int): The weight of this panel.
         """
-        if permissions is None:
-            permissions = []
         super().__init__(permissions)
         self.custom_data = custom_data
         self.custom_template = custom_template
         self.name = name
-        self.permissions = permissions
         self.weight = weight
 
         if items is not None and custom_template is not None:
@@ -300,19 +332,20 @@ class HomePagePanel(HomePageBase, PermissionsMixin):
             elif not all(isinstance(item, (HomePageGroup, HomePageItem)) for item in items):
                 raise TypeError("All items defined in a panel must be an instance of HomePageGroup or HomePageItem")
             self.items = items
+        else:
+            self.items = []
 
 
 class HomePageGroup(HomePageBase, PermissionsMixin):
     """Defines properties that can be used for a panel group."""
 
-    permissions = []
     items = []
 
     @property
     def initial_dict(self):
         return {
             "items": {},
-            "permissions": set(),
+            "permissions": self.permissions,
             "weight": self.weight,
         }
 
@@ -330,8 +363,6 @@ class HomePageGroup(HomePageBase, PermissionsMixin):
             items (list): List of items to be rendered in this group.
             weight (int): The weight of this group.
         """
-        if permissions is None:
-            permissions = []
         super().__init__(permissions)
         self.name = name
         self.weight = weight
@@ -347,7 +378,6 @@ class HomePageGroup(HomePageBase, PermissionsMixin):
 class HomePageItem(HomePageBase, PermissionsMixin):
     """Defines properties that can be used for a panel item."""
 
-    permissions = []
     items = []
     template_path = None
 
@@ -415,7 +445,8 @@ class NavMenuTab(NavMenuBase, PermissionsMixin):
     groups = []
 
     @property
-    def initial_dict(self):
+    def initial_dict(self) -> dict:
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
         return {
             "weight": self.weight,
             "groups": {},
@@ -423,7 +454,8 @@ class NavMenuTab(NavMenuBase, PermissionsMixin):
         }
 
     @property
-    def fixed_fields(self):
+    def fixed_fields(self) -> tuple:
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
         return ()
 
     def __init__(self, name, permissions=None, groups=None, weight=1000):
@@ -459,14 +491,16 @@ class NavMenuGroup(NavMenuBase, PermissionsMixin):
     items = []
 
     @property
-    def initial_dict(self):
+    def initial_dict(self) -> dict:
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
         return {
             "weight": self.weight,
             "items": {},
         }
 
     @property
-    def fixed_fields(self):
+    def fixed_fields(self) -> tuple:
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
         return ()
 
     def __init__(self, name, items=None, weight=1000):
@@ -498,7 +532,8 @@ class NavMenuItem(NavMenuBase, PermissionsMixin):
     """
 
     @property
-    def initial_dict(self):
+    def initial_dict(self) -> dict:
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
         return {
             "name": self.name,
             "weight": self.weight,
@@ -509,7 +544,8 @@ class NavMenuItem(NavMenuBase, PermissionsMixin):
         }
 
     @property
-    def fixed_fields(self):
+    def fixed_fields(self) -> tuple:
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
         return (
             ("name", self.name),
             ("permissions", self.permissions),
@@ -554,7 +590,8 @@ class NavMenuButton(NavMenuBase, PermissionsMixin):
     """
 
     @property
-    def initial_dict(self):
+    def initial_dict(self) -> dict:
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
         return {
             "link": self.link,
             "icon_class": self.icon_class,
@@ -565,7 +602,8 @@ class NavMenuButton(NavMenuBase, PermissionsMixin):
         }
 
     @property
-    def fixed_fields(self):
+    def fixed_fields(self) -> tuple:
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
         return (
             ("button_class", self.button_class),
             ("icon_class", self.icon_class),
@@ -633,6 +671,122 @@ class NavMenuImportButton(NavMenuButton):
         super().__init__(*args, **kwargs)
 
 
+class NavContext(NavMenuBase):
+    """Ths class represents a navigation menu tab for new ui.
+
+    Groups are each specified as a list of NavGrouping instances.
+    """
+
+    def __init__(self, name, groups, weight=1000):
+        self.name = name
+        self.groups = groups
+        self.weight = weight
+        self.validate()
+
+    def validate(self):
+        # NavContext name must belong in this group ("Inventory", "Networks", "Security", "Automation", "Platform")
+        if self.name not in NAV_CONTEXT_NAMES:
+            raise TypeError(f"`{self.name}` is an invalid context name, valid choices are: {NAV_CONTEXT_NAMES}")
+
+        if self.groups:
+            groups = self.groups
+            if not isinstance(groups, (list, tuple)):
+                raise TypeError("Groups must be passed as a tuple or list.")
+            elif not all(isinstance(group, NavGrouping) for group in groups):
+                raise TypeError("All groups defined in a NavContext must be an instance of NavGrouping")
+
+    @property
+    def initial_dict(self) -> dict:
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
+        return {
+            "weight": self.weight,
+            "data": {},
+        }
+
+    @property
+    def fixed_fields(self) -> tuple:
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
+        return ()
+
+
+class NavGrouping(NavMenuBase, PermissionsMixin):
+    """
+    Ths class represents a navigation menu group for the new ui. This is built up from a name and a weight value. The name is
+    the display text and the weight defines its position in the navigation sidebar.
+
+    Items are each specified as a list of NavItem or NavGrouping instances.
+    """
+
+    def __init__(self, name, items, weight=1000):
+        self.name = name
+        self.items = items
+        self.weight = weight
+        self.validate()
+
+    def validate(self):
+        if self.items:
+            items = self.items
+            if items is not None and not isinstance(items, (list, tuple)):
+                raise TypeError("Items must be passed as a tuple or list.")
+
+            if not all(isinstance(item, (NavItem, self.__class__)) for item in items):
+                raise TypeError("All items defined in a NavGrouping must be an instance of NavItem or NavGrouping")
+
+    @property
+    def initial_dict(self) -> dict:
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
+        return {
+            "weight": self.weight,
+            "data": {},
+        }
+
+    @property
+    def fixed_fields(self) -> tuple:
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
+        return ()
+
+
+class NavItem(NavMenuBase, PermissionsMixin):
+    """
+    This class represents a navigation menu item for the new ui. This constitutes link and its text.
+
+    Links are specified as Django reverse URL strings.
+    """
+
+    def __init__(self, name, link, *args, permissions=None, weight=1000, **kwargs):
+        self.name = name
+        self.link = link
+        self.permissions = permissions or []
+        self.weight = weight
+        self.args = args
+        self.kwargs = kwargs
+
+    @property
+    def initial_dict(self) -> dict:
+        """Attributes to be stored when adding this item to the nav menu data for the first time."""
+        return {
+            "name": self.name,
+            "weight": self.weight,
+            "permissions": self.permissions,
+            "data": self.url(),
+        }
+
+    @property
+    def fixed_fields(self):
+        """Tuple of (name, attribute) entries describing fields that may not be altered after declaration."""
+        return (
+            ("name", self.name),
+            ("permissions", self.permissions),
+        )
+
+    def url(self):
+        try:
+            return reverse(self.link, args=self.args, kwargs=self.kwargs)
+        except NoReverseMatch as e:
+            logger.error("Error in link construction for %s: %s", self.name, e)
+            return ""
+
+
 def post_migrate_send_nautobot_database_ready(sender, app_config, signal, **kwargs):
     """
     Send the `nautobot_database_ready` signal to all installed apps and plugins.
@@ -653,6 +807,14 @@ class CoreConfig(NautobotConfig):
     verbose_name = "Nautobot Core"
 
     def ready(self):
+        # Register netutils jinja2 filters in django_jinja
+        from django_jinja import library
+        from netutils.utils import jinja2_convenience_function
+
+        for name, func in jinja2_convenience_function().items():
+            # Register in django_jinja
+            library.filter(name=name, fn=func)
+
         from graphene_django.converter import convert_django_field
         from nautobot.core.graphql import BigInteger
 
@@ -685,12 +847,10 @@ class CoreConfig(NautobotConfig):
 
         super().ready()
 
-        # Magical monkey-patch TaggableManager to replace the `formfield()` method from our mixin.
-        from cacheops.utils import monkey_mix
-        from nautobot.extras.models import mixins
-        from taggit.managers import TaggableManager
+        # Register jobs last after everything else has been done.
+        from nautobot.core.celery import app, import_jobs_as_celery_tasks
 
-        monkey_mix(TaggableManager, mixins.TaggableManagerMonkeyMixin)
+        import_jobs_as_celery_tasks(app, database_ready=False)
 
 
 class NautobotConstanceConfig(ConstanceConfig):
