@@ -1,7 +1,7 @@
 from django.conf import settings as django_settings
 
 from nautobot.core.settings_funcs import sso_auth_enabled
-from nautobot.extras.registry import registry
+from nautobot.core.utils.navigation import is_route_new_ui_ready
 
 
 def get_saml_idp():
@@ -30,14 +30,17 @@ def get_saml_idp():
     return value
 
 
-def settings_and_registry(request):
+def settings(request):
     """
-    Expose Django settings and Nautobot registry stores in the template context. Example: {{ settings.DEBUG }}
+    Expose Django settings in the template context. Example: {{ settings.DEBUG }}
     """
-
+    root_template = "base_django.html"
+    if django_settings.ENABLE_ALPHA_UI:
+        use_new_ui = request.COOKIES.get("newui", False) and is_route_new_ui_ready(request.path)
+        root_template = "base_react.html" if use_new_ui else "base_django.html"
     return {
         "settings": django_settings,
-        "registry": registry,
+        "root_template": root_template,
     }
 
 
