@@ -12,13 +12,13 @@ def execute_query(query, variables=None, request=None, user=None):
     """Execute a query from the ORM.
 
     Args:
-        - query (str): String with GraphQL query.
-        - variables (dict, optional): If the query has variables they need to be passed in as a dictionary.
-        - request (django.test.client.RequestFactory, optional): Used to authenticate.
-        - user (django.contrib.auth.models.User, optional): Used to authenticate.
+        query (str): String with GraphQL query.
+        variables (dict, optional): If the query has variables they need to be passed in as a dictionary.
+        request (django.test.client.RequestFactory, optional): Used to authenticate.
+        user (django.contrib.auth.models.User, optional): Used to authenticate.
 
     Returns:
-        GraphQL Object: Result for query
+        (GraphQLDocument): Result for query
     """
     if not request and not user:
         raise ValueError("Either request or username should be provided")
@@ -34,19 +34,21 @@ def execute_query(query, variables=None, request=None, user=None):
         return document.execute(context_value=request)
 
 
-def execute_saved_query(saved_query_slug, **kwargs):
+def execute_saved_query(saved_query_name, **kwargs):
     """Execute saved query from the ORM.
 
     Args:
-        - saved_query_slug (str): Slug of a saved GraphQL query.
-        - variables (dict, optional): If the query has variables they need to be passed in as a dictionary.
-        - request (django.test.client.RequestFactory, optional): Used to authenticate.
-        - user (django.contrib.auth.models.User, optional): Used to authenticate.
+        saved_query_name (str): Name of a saved GraphQL query.
+
+    Keyword Args:
+        variables (Optional[dict]): If the query has variables they need to be passed in as a dictionary.
+        request (Optional[django.test.client.RequestFactory]): Used to authenticate.
+        user (Optional[django.contrib.auth.models.User]): Used to authenticate.
 
     Returns:
-        GraphQL Object: Result for query
+        (GraphQLDocument): Result for query
     """
-    query = GraphQLQuery.objects.get(slug=saved_query_slug)
+    query = GraphQLQuery.objects.get(name=saved_query_name)
     return execute_query(query=query.query, **kwargs)
 
 
@@ -67,3 +69,4 @@ class BigInteger(Scalar):
     def parse_literal(node):
         if isinstance(node, ast.IntValue):
             return int(node.value)
+        return None
