@@ -5,14 +5,14 @@ import logging
 import graphene
 import graphene_django_optimizer as gql_optimizer
 from graphql import GraphQLError
-from graphene_django import DjangoObjectType
 
+from nautobot.core.graphql.types import OptimizedNautobotObjectType
 from nautobot.core.graphql.utils import str_to_var_name, get_filtering_args_from_filterset
+from nautobot.core.utils.lookup import get_filterset_for_model
 from nautobot.extras.choices import RelationshipSideChoices
 from nautobot.extras.models import RelationshipAssociation
-from nautobot.utilities.utils import get_filterset_for_model
 
-logger = logging.getLogger("nautobot.graphql.generators")
+logger = logging.getLogger(__name__)
 RESOLVER_PREFIX = "resolve_"
 
 
@@ -68,6 +68,10 @@ def generate_filter_resolver(schema_type, resolver_name, field_name):
         if not filterset_class:
             return getattr(self, field_name).all()
 
+        # Inverse of substitution logic from get_filtering_args_from_filterset() - transform "_type" back to "type"
+        if "_type" in kwargs:
+            kwargs["type"] = kwargs.pop("_type")
+
         resolved_obj = filterset_class(kwargs, getattr(self, field_name).all())
 
         # Check result filter for errors.
@@ -88,16 +92,16 @@ def generate_filter_resolver(schema_type, resolver_name, field_name):
     return resolve_filter
 
 
-def generate_custom_field_resolver(name, resolver_name):
+def generate_custom_field_resolver(key, resolver_name):
     """Generate function to resolve each custom field within each DjangoObjectType.
 
     Args:
-        name (str): name of the custom field to resolve
+        key (str): unique key of the custom field to resolve
         resolver_name (str): name of the resolver as declare in DjangoObjectType
     """
 
     def resolve_custom_field(self, info, **kwargs):
-        return self.cf.get(name, None)
+        return self.cf.get(key, None)
 
     resolve_custom_field.__name__ = resolver_name
     return resolve_custom_field
@@ -201,7 +205,7 @@ def generate_relationship_resolver(name, resolver_name, relationship, side, peer
     return resolve_relationship
 
 
-def generate_schema_type(app_name: str, model: object) -> DjangoObjectType:
+def generate_schema_type(app_name: str, model: object) -> OptimizedNautobotObjectType:
     """
     Take a Django model and generate a Graphene Type class definition.
 
@@ -212,7 +216,7 @@ def generate_schema_type(app_name: str, model: object) -> DjangoObjectType:
     Example:
         For a model with a name of "Device", the following class definition is generated:
 
-        class DeviceType(DjangoObjectType):
+        class DeviceType(OptimizedNautobotObjectType):
             Meta:
                 model = Device
                 fields = ["__all__"]
@@ -221,7 +225,7 @@ def generate_schema_type(app_name: str, model: object) -> DjangoObjectType:
         '<app_name>.filters.<ModelName>FilterSet' the filterset will be stored in
         filterset_class as follows:
 
-        class DeviceType(DjangoObjectType):
+        class DeviceType(OptimizedNautobotObjectType):
             Meta:
                 model = Device
                 fields = ["__all__"]
@@ -237,17 +241,22 @@ def generate_schema_type(app_name: str, model: object) -> DjangoObjectType:
 
     main_attrs["Meta"] = type("Meta", (object,), meta_attrs)
 
-    schema_type = type(f"{model.__name__}Type", (DjangoObjectType,), main_attrs)
+    schema_type = type(f"{model.__name__}Type", (OptimizedNautobotObjectType,), main_attrs)
     return schema_type
 
 
 def generate_list_search_parameters(schema_type):
     """Generate list of query parameters for the list resolver based on a filterset."""
 
-    search_params = {}
+    search_params = {
+        "limit": graphene.Int(),
+        "offset": graphene.Int(),
+    }
     if schema_type._meta.filterset_class is not None:
-        search_params = get_filtering_args_from_filterset(
-            schema_type._meta.filterset_class,
+        search_params.update(
+            get_filtering_args_from_filterset(
+                schema_type._meta.filterset_class,
+            )
         )
 
     return search_params
@@ -261,12 +270,11 @@ def generate_single_item_resolver(schema_type, resolver_name):
         resolver_name (str): name of the resolver
 
     Returns:
-        callable: Resolver function for a single element
+        (func): Resolver function for a single element
     """
     model = schema_type._meta.model
 
     def single_resolver(self, info, **kwargs):
-
         obj_id = kwargs.get("id", None)
         if obj_id:
             return gql_optimizer.query(
@@ -291,11 +299,11 @@ def generate_list_resolver(schema_type, resolver_name):
         resolver_name (str): name of the resolver
 
     Returns:
-        callable: Resolver function for list of element
+        (func): Resolver function for list of element
     """
     model = schema_type._meta.model
 
-    def list_resolver(self, info, **kwargs):
+    def list_resolver(self, info, limit=None, offset=None, **kwargs):
         filterset_class = schema_type._meta.filterset_class
         if filterset_class is not None:
             resolved_obj = filterset_class(kwargs, model.objects.restrict(info.context.user, "view").all())
@@ -311,10 +319,18 @@ def generate_list_resolver(schema_type, resolver_name):
 
                 # Raising this exception will send the error message in the response of the GraphQL request
                 raise GraphQLError(errors)
+            qs = resolved_obj.qs.all()
 
-            return gql_optimizer.query(resolved_obj.qs.all(), info)
+        else:
+            qs = model.objects.restrict(info.context.user, "view").all()
 
-        return gql_optimizer.query(model.objects.restrict(info.context.user, "view").all(), info)
+        if offset:
+            qs = qs[offset:]
+
+        if limit:
+            qs = qs[:limit]
+
+        return gql_optimizer.query(qs, info)
 
     list_resolver.__name__ = resolver_name
     return list_resolver
@@ -327,7 +343,7 @@ def generate_attrs_for_schema_type(schema_type):
         schema_type (DjangoObjectType): DjangoObjectType for a given model
 
     Returns:
-        dict: Dict of attributes ready to merge into the QueryMixin class
+        (dict): Dict of attributes ready to merge into the QueryMixin class
     """
     attrs = {}
     model = schema_type._meta.model
