@@ -14,6 +14,7 @@ import sys
 from django.core import management
 
 from nautobot import __version__
+from nautobot.core.settings_funcs import is_truthy
 from . import importer
 from .settings import create_default_settings
 
@@ -82,7 +83,7 @@ def configure_app(
     project_filename = sanitize_name(project)
 
     if default_config_path is None:
-        default_config_path = "~/%s/%s.conf.py" % (project_filename, project_filename)
+        default_config_path = f"~/{project_filename}/{project_filename}.conf.py"
 
     if settings_envvar is None:
         settings_envvar = project_filename.upper() + "_CONF"
@@ -103,10 +104,8 @@ def configure_app(
 
     if not os.path.exists(config_path):
         if runner_name:
-            raise ValueError(
-                "Configuration file does not exist. Use '%s init' to initialize the file." % (runner_name,)
-            )
-        raise ValueError("Configuration file does not exist at %r" % (config_path,))
+            raise ValueError(f"Configuration file does not exist. Use '{runner_name} init' to initialize the file.")
+        raise ValueError(f"Configuration file does not exist at {config_path}")
 
     os.environ["DJANGO_SETTINGS_MODULE"] = config_module_name
 
@@ -124,7 +123,6 @@ def configure_app(
             )
         except Exception:
             # XXX: Django doesn't like various errors in this path
-            import sys
             import traceback
 
             traceback.print_exc()
@@ -200,6 +198,11 @@ def run_app(**kwargs):
         "init",
         help="Initialize a new configuration",
     )
+    init_parser.add_argument(
+        "--disable-installation-metrics",
+        action="store_true",
+        help="Disable sending of anonymized installation metrics for this configuration.",
+    )
     init_parser.add_argument(
         "config_path",
         default=default_config_path,
@@ -241,13 +244,41 @@ def run_app(**kwargs):
             )
             return
 
+        if not sys.__stdin__.isatty():
+            # Non-interactive invocation, maybe during a build?
+            installation_metrics_enabled = not args.disable_installation_metrics
+        elif args.disable_installation_metrics is True:
+            # already explicitly specified
+            installation_metrics_enabled = False
+        else:
+            # Prompt user to enable installation metrics
+            installation_metrics_enabled = None
+            while installation_metrics_enabled is None:
+                installation_metrics_prompt = input(
+                    "Nautobot would like to send anonymized installation metrics to the project's maintainers.\n"
+                    "These metrics include the installed Nautobot version, the Python version in use, an anonymous "
+                    '"deployment ID", and a list of one-way-hashed names of enabled Nautobot Apps and their versions.\n'
+                    "Allow Nautobot to send these metrics? [y/n]: "
+                )
+                try:
+                    installation_metrics_enabled = is_truthy(installation_metrics_prompt)
+                except ValueError:
+                    print("Please enter 'y' or 'n'.")
+
+        if installation_metrics_enabled:
+            print("Installation metrics will be sent when running 'nautobot-server post_upgrade'. Thank you!")
+        else:
+            print("Installation metrics will not be sent by default.")
+
         # Create the config
         try:
-            create_default_settings(config_path, settings_initializer)
+            create_default_settings(
+                config_path, settings_initializer, installation_metrics_enabled=installation_metrics_enabled
+            )
         except OSError as e:
-            raise e.__class__("Unable to write default settings file to %r" % config_path)
+            raise e.__class__(f"Unable to write default settings file to {config_path}")
 
-        print("Configuration file created at %r" % config_path)
+        print(f"Configuration file created at {config_path}")
 
         return
 
@@ -256,7 +287,7 @@ def run_app(**kwargs):
     # process default path or environment variable.
     config_path = args.config
 
-    # Overlay our config w/ defautls
+    # Overlay our config w/ defaults
     try:
         configure_app(config_path=config_path, **kwargs)
     except ValueError as err:
