@@ -4,6 +4,8 @@ from django.core.management import call_command
 from django.conf import settings
 from django.test.runner import DiscoverRunner
 
+from nautobot.core.celery import app, setup_nautobot_job_logging
+
 
 class NautobotTestRunner(DiscoverRunner):
     """
@@ -13,7 +15,7 @@ class NautobotTestRunner(DiscoverRunner):
     explicitly passed in with `nautobot-server test --tag integration`.
 
     By Nautobot convention, integration tests must be tagged with "integration". The base
-    `nautobot.utilities.testing.integration.SeleniumTestCase` has this tag, therefore any test cases
+    `nautobot.core.testing.integration.SeleniumTestCase` has this tag, therefore any test cases
     inheriting from that class do not need to be explicitly tagged.
 
     Only integration tests that DO NOT inherit from `SeleniumTestCase` will need to be explicitly tagged.
@@ -22,7 +24,6 @@ class NautobotTestRunner(DiscoverRunner):
     exclude_tags = ["integration"]
 
     def __init__(self, cache_test_fixtures=False, **kwargs):
-
         self.cache_test_fixtures = cache_test_fixtures
 
         # Assert "integration" hasn't been provided w/ --tag
@@ -50,26 +51,34 @@ class NautobotTestRunner(DiscoverRunner):
         super().setup_test_environment(**kwargs)
         # Remove 'testserver' that Django "helpfully" adds automatically to ALLOWED_HOSTS, masking issues like #3065
         settings.ALLOWED_HOSTS.remove("testserver")
+        if getattr(settings, "CELERY_TASK_ALWAYS_EAGER", False):
+            # Make sure logs get captured when running Celery tasks, even though we don't have/need a Celery worker
+            setup_nautobot_job_logging(None, None, app.conf)
 
     def setup_databases(self, **kwargs):
         result = super().setup_databases(**kwargs)
 
-        if settings.TEST_USE_FACTORIES:
-            print("Pre-populating test database with factory data...")
+        if settings.TEST_USE_FACTORIES and result:
             command = ["generate_test_data", "--flush", "--no-input"]
             if settings.TEST_FACTORY_SEED is not None:
                 command += ["--seed", settings.TEST_FACTORY_SEED]
             if self.cache_test_fixtures:
                 command += ["--cache-test-fixtures"]
-            call_command(*command)
+            for connection in result:
+                db_name = connection[0].alias
+                print(f'Pre-populating test database "{db_name}" with factory data...')
+                db_command = command + ["--database", db_name]
+                call_command(*db_command)
 
         return result
 
     def teardown_databases(self, old_config, **kwargs):
-        if settings.TEST_USE_FACTORIES:
-            print("Emptying test database...")
-            call_command("flush", "--no-input")
-            print("Database emptied!")
+        if settings.TEST_USE_FACTORIES and old_config:
+            for connection in old_config:
+                db_name = connection[0].alias
+                print(f'Emptying test database "{db_name}"...')
+                call_command("flush", "--no-input", "--database", db_name)
+                print(f"Database {db_name} emptied!")
 
         super().teardown_databases(old_config, **kwargs)
 
