@@ -1,21 +1,16 @@
-from nautobot.core.api import OrderedDefaultRouter
+from nautobot.core.api.routers import OrderedDefaultRouter
 from . import views
 
 
 router = OrderedDefaultRouter()
 router.APIRootView = views.DCIMRootView
 
-# Sites
-router.register("regions", views.RegionViewSet)
-router.register("sites", views.SiteViewSet)
-
 # Locations
 router.register("location-types", views.LocationTypeViewSet)
 router.register("locations", views.LocationViewSet)
 
 # Racks
 router.register("rack-groups", views.RackGroupViewSet)
-router.register("rack-roles", views.RackRoleViewSet)
 router.register("racks", views.RackViewSet)
 router.register("rack-reservations", views.RackReservationViewSet)
 
@@ -34,7 +29,6 @@ router.register("rear-port-templates", views.RearPortTemplateViewSet)
 router.register("device-bay-templates", views.DeviceBayTemplateViewSet)
 
 # Devices
-router.register("device-roles", views.DeviceRoleViewSet)
 router.register("platforms", views.PlatformViewSet)
 router.register("devices", views.DeviceViewSet)
 
@@ -61,6 +55,10 @@ router.register(
 # Cables
 router.register("cables", views.CableViewSet)
 
+# Interface Redundancy Group
+router.register("interface-redundancy-groups", views.InterfaceRedundancyGroupViewSet)
+router.register("interface-redundancy-group-associations", views.InterfaceRedundancyGroupAssociationViewSet)
+
 # Virtual chassis
 router.register("virtual-chassis", views.VirtualChassisViewSet)
 
