@@ -1,35 +1,4 @@
-from nautobot.utilities.choices import ChoiceSet
-
-
-#
-# Sites
-#
-
-
-class SiteStatusChoices(ChoiceSet):
-
-    STATUS_PLANNED = "planned"
-    STATUS_STAGING = "staging"
-    STATUS_ACTIVE = "active"
-    STATUS_DECOMMISSIONING = "decommissioning"
-    STATUS_RETIRED = "retired"
-
-    CHOICES = (
-        (STATUS_PLANNED, "Planned"),
-        (STATUS_STAGING, "Staging"),
-        (STATUS_ACTIVE, "Active"),
-        (STATUS_DECOMMISSIONING, "Decommissioning"),
-        (STATUS_RETIRED, "Retired"),
-    )
-
-    # 2.0 TODO: No longer used (see extras.management.COLOR_MAP instead), remove in 2.0
-    CSS_CLASSES = {
-        STATUS_PLANNED: "info",
-        STATUS_STAGING: "primary",
-        STATUS_ACTIVE: "success",
-        STATUS_DECOMMISSIONING: "warning",
-        STATUS_RETIRED: "danger",
-    }
+from nautobot.core.choices import ChoiceSet
 
 
 #
@@ -38,7 +7,6 @@ class SiteStatusChoices(ChoiceSet):
 
 
 class LocationStatusChoices(ChoiceSet):
-
     STATUS_PLANNED = "planned"
     STATUS_STAGING = "staging"
     STATUS_ACTIVE = "active"
@@ -60,7 +28,6 @@ class LocationStatusChoices(ChoiceSet):
 
 
 class RackTypeChoices(ChoiceSet):
-
     TYPE_2POST = "2-post-frame"
     TYPE_4POST = "4-post-frame"
     TYPE_CABINET = "4-post-cabinet"
@@ -83,7 +50,6 @@ class RackTypeChoices(ChoiceSet):
 
 
 class RackWidthChoices(ChoiceSet):
-
     WIDTH_10IN = 10
     WIDTH_19IN = 19
     WIDTH_21IN = 21
@@ -98,7 +64,6 @@ class RackWidthChoices(ChoiceSet):
 
 
 class RackStatusChoices(ChoiceSet):
-
     STATUS_RESERVED = "reserved"
     STATUS_AVAILABLE = "available"
     STATUS_PLANNED = "planned"
@@ -113,18 +78,8 @@ class RackStatusChoices(ChoiceSet):
         (STATUS_DEPRECATED, "Deprecated"),
     )
 
-    # 2.0 TODO: No longer used (see extras.management.COLOR_MAP instead), remove in 2.0
-    CSS_CLASSES = {
-        STATUS_RESERVED: "warning",
-        STATUS_AVAILABLE: "success",
-        STATUS_PLANNED: "info",
-        STATUS_ACTIVE: "primary",
-        STATUS_DEPRECATED: "danger",
-    }
-
 
 class RackDimensionUnitChoices(ChoiceSet):
-
     UNIT_MILLIMETER = "mm"
     UNIT_INCH = "in"
 
@@ -135,7 +90,6 @@ class RackDimensionUnitChoices(ChoiceSet):
 
 
 class RackElevationDetailRenderChoices(ChoiceSet):
-
     RENDER_JSON = "json"
     RENDER_SVG = "svg"
 
@@ -148,7 +102,6 @@ class RackElevationDetailRenderChoices(ChoiceSet):
 
 
 class SubdeviceRoleChoices(ChoiceSet):
-
     ROLE_PARENT = "parent"
     ROLE_CHILD = "child"
 
@@ -164,7 +117,6 @@ class SubdeviceRoleChoices(ChoiceSet):
 
 
 class DeviceFaceChoices(ChoiceSet):
-
     FACE_FRONT = "front"
     FACE_REAR = "rear"
 
@@ -175,7 +127,6 @@ class DeviceFaceChoices(ChoiceSet):
 
 
 class DeviceStatusChoices(ChoiceSet):
-
     STATUS_OFFLINE = "offline"
     STATUS_ACTIVE = "active"
     STATUS_PLANNED = "planned"
@@ -194,17 +145,6 @@ class DeviceStatusChoices(ChoiceSet):
         (STATUS_DECOMMISSIONING, "Decommissioning"),
     )
 
-    # 2.0 TODO: No longer used (see extras.management.COLOR_MAP instead), remove in 2.0
-    CSS_CLASSES = {
-        STATUS_OFFLINE: "warning",
-        STATUS_ACTIVE: "success",
-        STATUS_PLANNED: "info",
-        STATUS_STAGED: "primary",
-        STATUS_FAILED: "danger",
-        STATUS_INVENTORY: "default",
-        STATUS_DECOMMISSIONING: "warning",
-    }
-
 
 #
 # ConsolePorts
@@ -212,7 +152,6 @@ class DeviceStatusChoices(ChoiceSet):
 
 
 class ConsolePortTypeChoices(ChoiceSet):
-
     TYPE_DE9 = "de-9"
     TYPE_DB25 = "db-25"
     TYPE_RJ11 = "rj-11"
@@ -264,7 +203,6 @@ class ConsolePortTypeChoices(ChoiceSet):
 
 
 class PowerPortTypeChoices(ChoiceSet):
-
     # IEC 60320
     TYPE_IEC_C6 = "iec-60320-c6"
     TYPE_IEC_C8 = "iec-60320-c8"
@@ -525,7 +463,6 @@ class PowerPortTypeChoices(ChoiceSet):
 
 
 class PowerOutletTypeChoices(ChoiceSet):
-
     # IEC 60320
     TYPE_IEC_C5 = "iec-60320-c5"
     TYPE_IEC_C7 = "iec-60320-c7"
@@ -767,7 +704,6 @@ class PowerOutletTypeChoices(ChoiceSet):
 
 
 class PowerOutletFeedLegChoices(ChoiceSet):
-
     FEED_LEG_A = "A"
     FEED_LEG_B = "B"
     FEED_LEG_C = "C"
@@ -785,7 +721,6 @@ class PowerOutletFeedLegChoices(ChoiceSet):
 
 
 class InterfaceTypeChoices(ChoiceSet):
-
     # Virtual
     TYPE_VIRTUAL = "virtual"
     TYPE_BRIDGE = "bridge"
@@ -1089,7 +1024,6 @@ class InterfaceTypeChoices(ChoiceSet):
 
 
 class InterfaceModeChoices(ChoiceSet):
-
     MODE_ACCESS = "access"
     MODE_TAGGED = "tagged"
     MODE_TAGGED_ALL = "tagged-all"
@@ -1102,7 +1036,6 @@ class InterfaceModeChoices(ChoiceSet):
 
 
 class InterfaceStatusChoices(ChoiceSet):
-
     STATUS_PLANNED = "planned"
     STATUS_ACTIVE = "active"
     STATUS_DECOMMISSIONING = "decommissioning"
@@ -1124,7 +1057,6 @@ class InterfaceStatusChoices(ChoiceSet):
 
 
 class PortTypeChoices(ChoiceSet):
-
     TYPE_8P8C = "8p8c"
     TYPE_8P6C = "8p6c"
     TYPE_8P4C = "8p4c"
@@ -1232,7 +1164,6 @@ class PortTypeChoices(ChoiceSet):
 
 
 class CableTypeChoices(ChoiceSet):
-
     TYPE_CAT3 = "cat3"
     TYPE_CAT5 = "cat5"
     TYPE_CAT5E = "cat5e"
@@ -1295,7 +1226,6 @@ class CableTypeChoices(ChoiceSet):
 
 
 class CableStatusChoices(ChoiceSet):
-
     STATUS_CONNECTED = "connected"
     STATUS_PLANNED = "planned"
     STATUS_DECOMMISSIONING = "decommissioning"
@@ -1306,16 +1236,8 @@ class CableStatusChoices(ChoiceSet):
         (STATUS_DECOMMISSIONING, "Decommissioning"),
     )
 
-    # 2.0 TODO: No longer used (see extras.management.COLOR_MAP instead), remove in 2.0
-    CSS_CLASSES = {
-        STATUS_CONNECTED: "success",
-        STATUS_PLANNED: "info",
-        STATUS_DECOMMISSIONING: "warning",
-    }
-
 
 class CableLengthUnitChoices(ChoiceSet):
-
     # Metric
     UNIT_KILOMETER = "km"
     UNIT_METER = "m"
@@ -1342,7 +1264,6 @@ class CableLengthUnitChoices(ChoiceSet):
 
 
 class PowerFeedStatusChoices(ChoiceSet):
-
     STATUS_OFFLINE = "offline"
     STATUS_ACTIVE = "active"
     STATUS_PLANNED = "planned"
@@ -1355,17 +1276,8 @@ class PowerFeedStatusChoices(ChoiceSet):
         (STATUS_FAILED, "Failed"),
     )
 
-    # 2.0 TODO: No longer used (see extras.management.COLOR_MAP instead), remove in 2.0
-    CSS_CLASSES = {
-        STATUS_OFFLINE: "warning",
-        STATUS_ACTIVE: "success",
-        STATUS_PLANNED: "info",
-        STATUS_FAILED: "danger",
-    }
-
 
 class PowerFeedTypeChoices(ChoiceSet):
-
     TYPE_PRIMARY = "primary"
     TYPE_REDUNDANT = "redundant"
 
@@ -1381,7 +1293,6 @@ class PowerFeedTypeChoices(ChoiceSet):
 
 
 class PowerFeedSupplyChoices(ChoiceSet):
-
     SUPPLY_AC = "ac"
     SUPPLY_DC = "dc"
 
@@ -1392,7 +1303,6 @@ class PowerFeedSupplyChoices(ChoiceSet):
 
 
 class PowerFeedPhaseChoices(ChoiceSet):
-
     PHASE_SINGLE = "single-phase"
     PHASE_3PHASE = "three-phase"
 
@@ -1408,7 +1318,6 @@ class PowerFeedPhaseChoices(ChoiceSet):
 
 
 class DeviceRedundancyGroupStatusChoices(ChoiceSet):
-
     STATUS_PLANNED = "planned"
     STATUS_STAGING = "staging"
     STATUS_ACTIVE = "active"
@@ -1425,10 +1334,47 @@ class DeviceRedundancyGroupStatusChoices(ChoiceSet):
 
 
 class DeviceRedundancyGroupFailoverStrategyChoices(ChoiceSet):
+    FAILOVER_UNSPECIFIED = ""
     FAILOVER_ACTIVE_ACTIVE = "active-active"
     FAILOVER_ACTIVE_PASSIVE = "active-passive"
 
     CHOICES = (
+        (FAILOVER_UNSPECIFIED, "(unspecified)"),
         (FAILOVER_ACTIVE_ACTIVE, "Active/Active"),
         (FAILOVER_ACTIVE_PASSIVE, "Active/Passive"),
     )
+
+
+#
+# Interface Redundancy Groups
+#
+
+
+class InterfaceRedundancyGroupStatusChoices(ChoiceSet):
+    STATUS_PLANNED = "planned"
+    STATUS_STAGING = "staging"
+    STATUS_ACTIVE = "active"
+    STATUS_DECOMMISSIONING = "decommissioning"
+    STATUS_RETIRED = "retired"
+
+    CHOICES = (
+        (STATUS_PLANNED, "Planned"),
+        (STATUS_STAGING, "Staging"),
+        (STATUS_ACTIVE, "Active"),
+        (STATUS_DECOMMISSIONING, "Decommissioning"),
+        (STATUS_RETIRED, "Retired"),
+    )
+
+
+class InterfaceRedundancyGroupProtocolChoices(ChoiceSet):
+    HSRP = "hsrp"
+    VRRP = "vrrp"
+    GLBP = "glbp"
+    CARP = "carp"
+
+    CHOICES = (
+        (HSRP, "HSRP"),
+        (VRRP, "VRRP"),
+        (GLBP, "GLBP"),
+        (CARP, "CARP"),
+    )
