@@ -17,17 +17,18 @@ from .device_components import (
     DeviceBay,
     FrontPort,
     Interface,
+    InterfaceRedundancyGroup,
+    InterfaceRedundancyGroupAssociation,
     InventoryItem,
     PathEndpoint,
     PowerOutlet,
     PowerPort,
     RearPort,
 )
-from .devices import Device, DeviceRedundancyGroup, DeviceRole, DeviceType, Manufacturer, Platform, VirtualChassis
+from .devices import Device, DeviceRedundancyGroup, DeviceType, Manufacturer, Platform, VirtualChassis
 from .locations import Location, LocationType
 from .power import PowerFeed, PowerPanel
-from .racks import Rack, RackGroup, RackReservation, RackRole
-from .sites import Region, Site
+from .racks import Rack, RackGroup, RackReservation
 
 __all__ = (
     "BaseInterface",
@@ -42,11 +43,12 @@ __all__ = (
     "DeviceBay",
     "DeviceBayTemplate",
     "DeviceRedundancyGroup",
-    "DeviceRole",
     "DeviceType",
     "FrontPort",
     "FrontPortTemplate",
     "Interface",
+    "InterfaceRedundancyGroup",
+    "InterfaceRedundancyGroupAssociation",
     "InterfaceTemplate",
     "InventoryItem",
     "Location",
@@ -63,10 +65,7 @@ __all__ = (
     "Rack",
     "RackGroup",
     "RackReservation",
-    "RackRole",
     "RearPort",
     "RearPortTemplate",
-    "Region",
-    "Site",
     "VirtualChassis",
 )
