@@ -3,6 +3,9 @@ from django.core.validators import MaxValueValidator, MinValueValidator
 from django.db import models
 from django.contrib.contenttypes.models import ContentType
 
+from nautobot.core.models import BaseModel
+from nautobot.core.models.fields import ForeignKeyWithAutoRelatedName, NaturalOrderingField
+from nautobot.core.models.ordering import naturalize_interface
 from nautobot.dcim.choices import (
     SubdeviceRoleChoices,
     ConsolePortTypeChoices,
@@ -10,16 +13,12 @@ from nautobot.dcim.choices import (
     PowerOutletTypeChoices,
     PowerOutletFeedLegChoices,
     InterfaceTypeChoices,
-    InterfaceStatusChoices,
     PortTypeChoices,
 )
 
-from nautobot.core.models import BaseModel
 from nautobot.dcim.constants import REARPORT_POSITIONS_MAX, REARPORT_POSITIONS_MIN
 from nautobot.extras.models import ChangeLoggedModel, CustomField, CustomFieldModel, RelationshipModel, Status
 from nautobot.extras.utils import extras_features
-from nautobot.utilities.fields import NaturalOrderingField
-from nautobot.utilities.ordering import naturalize_interface
 from .device_components import (
     ConsolePort,
     ConsoleServerPort,
@@ -44,7 +43,7 @@ __all__ = (
 
 
 class ComponentTemplateModel(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel):
-    device_type = models.ForeignKey(to="dcim.DeviceType", on_delete=models.CASCADE, related_name="%(class)ss")
+    device_type = ForeignKeyWithAutoRelatedName(to="dcim.DeviceType", on_delete=models.CASCADE)
     name = models.CharField(max_length=64)
     _name = NaturalOrderingField(target_field="name", max_length=100, blank=True)
     label = models.CharField(max_length=64, blank=True, help_text="Physical label")
@@ -76,6 +75,12 @@ class ComponentTemplateModel(BaseModel, ChangeLoggedModel, CustomFieldModel, Rel
 
         return super().to_objectchange(action, related_object=device_type, **kwargs)
 
+    def get_absolute_url(self, api=False):
+        # TODO: in the new UI, this should be able to link directly to the object, instead of the device-type.
+        if not api:
+            return self.device_type.get_absolute_url(api=api)
+        return super().get_absolute_url(api=api)
+
     def instantiate_model(self, model, device, **kwargs):
         """
         Helper method to self.instantiate().
@@ -84,8 +89,7 @@ class ComponentTemplateModel(BaseModel, ChangeLoggedModel, CustomFieldModel, Rel
         content_type = ContentType.objects.get_for_model(model)
         fields = CustomField.objects.filter(content_types=content_type)
         for field in fields:
-            # 2.0 TODO: #824 use field.slug
-            custom_field_data[field.name] = field.default
+            custom_field_data[field.key] = field.default
 
         return model(
             device=device,
@@ -98,9 +102,7 @@ class ComponentTemplateModel(BaseModel, ChangeLoggedModel, CustomFieldModel, Rel
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class ConsolePortTemplate(ComponentTemplateModel):
     """
@@ -118,9 +120,7 @@ class ConsolePortTemplate(ComponentTemplateModel):
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class ConsoleServerPortTemplate(ComponentTemplateModel):
     """
@@ -138,9 +138,7 @@ class ConsoleServerPortTemplate(ComponentTemplateModel):
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class PowerPortTemplate(ComponentTemplateModel):
     """
@@ -185,9 +183,7 @@ class PowerPortTemplate(ComponentTemplateModel):
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class PowerOutletTemplate(ComponentTemplateModel):
     """
@@ -195,12 +191,12 @@ class PowerOutletTemplate(ComponentTemplateModel):
     """
 
     type = models.CharField(max_length=50, choices=PowerOutletTypeChoices, blank=True)
-    power_port = models.ForeignKey(
+    power_port_template = models.ForeignKey(
         to="dcim.PowerPortTemplate",
         on_delete=models.SET_NULL,
         blank=True,
         null=True,
-        related_name="poweroutlet_templates",
+        related_name="power_outlet_templates",
     )
     feed_leg = models.CharField(
         max_length=50,
@@ -217,12 +213,12 @@ class PowerOutletTemplate(ComponentTemplateModel):
         super().clean()
 
         # Validate power port assignment
-        if self.power_port and self.power_port.device_type != self.device_type:
-            raise ValidationError(f"Parent power port ({self.power_port}) must belong to the same device type")
+        if self.power_port_template and self.power_port_template.device_type != self.device_type:
+            raise ValidationError(f"Parent power port ({self.power_port_template}) must belong to the same device type")
 
     def instantiate(self, device):
-        if self.power_port:
-            power_port = PowerPort.objects.get(device=device, name=self.power_port.name)
+        if self.power_port_template:
+            power_port = PowerPort.objects.get(device=device, name=self.power_port_template.name)
         else:
             power_port = None
         return self.instantiate_model(
@@ -235,9 +231,7 @@ class PowerOutletTemplate(ComponentTemplateModel):
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class InterfaceTemplate(ComponentTemplateModel):
     """
@@ -260,7 +254,7 @@ class InterfaceTemplate(ComponentTemplateModel):
 
     def instantiate(self, device):
         try:
-            status = Status.objects.get_for_model(Interface).get(slug=InterfaceStatusChoices.STATUS_ACTIVE)
+            status = Status.objects.get_for_model(Interface).get(name="Active")
         except Status.DoesNotExist:
             status = Status.objects.get_for_model(Interface).first()
         return self.instantiate_model(
@@ -273,9 +267,7 @@ class InterfaceTemplate(ComponentTemplateModel):
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class FrontPortTemplate(ComponentTemplateModel):
     """
@@ -283,10 +275,10 @@ class FrontPortTemplate(ComponentTemplateModel):
     """
 
     type = models.CharField(max_length=50, choices=PortTypeChoices)
-    rear_port = models.ForeignKey(
+    rear_port_template = models.ForeignKey(
         to="dcim.RearPortTemplate",
         on_delete=models.CASCADE,
-        related_name="frontport_templates",
+        related_name="front_port_templates",
     )
     rear_port_position = models.PositiveSmallIntegerField(
         default=1,
@@ -300,28 +292,28 @@ class FrontPortTemplate(ComponentTemplateModel):
         ordering = ("device_type", "_name")
         unique_together = (
             ("device_type", "name"),
-            ("rear_port", "rear_port_position"),
+            ("rear_port_template", "rear_port_position"),
         )
 
     def clean(self):
         super().clean()
 
         # Validate rear port assignment
-        if self.rear_port.device_type != self.device_type:
-            raise ValidationError(f"Rear port ({self.rear_port}) must belong to the same device type")
+        if self.rear_port_template.device_type != self.device_type:
+            raise ValidationError(f"Rear port ({self.rear_port_template}) must belong to the same device type")
 
         # Validate rear port position assignment
-        if self.rear_port_position > self.rear_port.positions:
+        if self.rear_port_position > self.rear_port_template.positions:
             raise ValidationError(
                 (
                     f"Invalid rear port position ({self.rear_port_position}); "
-                    f"rear port {self.rear_port.name} has only {self.rear_port.positions} positions"
+                    f"rear port {self.rear_port_template.name} has only {self.rear_port_template.positions} positions"
                 )
             )
 
     def instantiate(self, device):
-        if self.rear_port:
-            rear_port = RearPort.objects.get(device=device, name=self.rear_port.name)
+        if self.rear_port_template:
+            rear_port = RearPort.objects.get(device=device, name=self.rear_port_template.name)
         else:
             rear_port = None
         return self.instantiate_model(
@@ -334,9 +326,7 @@ class FrontPortTemplate(ComponentTemplateModel):
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class RearPortTemplate(ComponentTemplateModel):
     """
@@ -366,9 +356,7 @@ class RearPortTemplate(ComponentTemplateModel):
 
 
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
 class DeviceBayTemplate(ComponentTemplateModel):
     """
