@@ -2,8 +2,9 @@ from django.contrib.contenttypes.models import ContentType
 from django.core.exceptions import ValidationError
 from django.core.validators import MaxValueValidator, MinValueValidator
 from django.db import models
-from django.urls import reverse
 
+from nautobot.core.models.generics import PrimaryModel
+from nautobot.core.models.validators import ExclusionValidator
 from nautobot.dcim.choices import PowerFeedPhaseChoices, PowerFeedSupplyChoices, PowerFeedTypeChoices
 from nautobot.dcim.constants import (
     POWERFEED_AMPERAGE_DEFAULT,
@@ -11,10 +12,8 @@ from nautobot.dcim.constants import (
     POWERFEED_VOLTAGE_DEFAULT,
 )
 
-from nautobot.extras.models import StatusModel
+from nautobot.extras.models import StatusField
 from nautobot.extras.utils import extras_features
-from nautobot.core.models.generics import PrimaryModel
-from nautobot.utilities.validators import ExclusionValidator
 from .device_components import CableTermination, PathEndpoint
 
 __all__ = (
@@ -29,13 +28,11 @@ __all__ = (
 
 
 @extras_features(
-    "custom_fields",
     "custom_links",
     "custom_validators",
     "export_templates",
     "graphql",
     "locations",
-    "relationships",
     "webhooks",
 )
 class PowerPanel(PrimaryModel):
@@ -43,43 +40,26 @@ class PowerPanel(PrimaryModel):
     A distribution point for electrical power; e.g. a data center RPP.
     """
 
-    site = models.ForeignKey(to="Site", on_delete=models.PROTECT)
-    location = models.ForeignKey(
-        to="dcim.Location", on_delete=models.PROTECT, related_name="powerpanels", blank=True, null=True
+    location = models.ForeignKey(to="dcim.Location", on_delete=models.PROTECT, related_name="power_panels")
+    rack_group = models.ForeignKey(
+        to="RackGroup", on_delete=models.PROTECT, blank=True, null=True, related_name="power_panels"
     )
-    rack_group = models.ForeignKey(to="RackGroup", on_delete=models.PROTECT, blank=True, null=True)
     name = models.CharField(max_length=100, db_index=True)
 
-    csv_headers = ["site", "location", "rack_group", "name"]
+    natural_key_field_names = ["name", "location"]
 
     class Meta:
-        ordering = ["site", "name"]
-        unique_together = ["site", "name"]
+        ordering = ["location", "name"]
+        unique_together = ["location", "name"]
 
     def __str__(self):
         return self.name
 
-    def get_absolute_url(self):
-        return reverse("dcim:powerpanel", args=[self.pk])
-
-    def to_csv(self):
-        return (
-            self.site.name,
-            self.location.name if self.location else None,
-            self.rack_group.name if self.rack_group else None,
-            self.name,
-        )
-
     def clean(self):
         super().clean()
 
         # Validate location
         if self.location is not None:
-            if self.location.base_site != self.site:
-                raise ValidationError(
-                    {"location": f'Location "{self.location}" does not belong to site "{self.site}".'}
-                )
-
             if ContentType.objects.get_for_model(self) not in self.location.location_type.content_types.all():
                 raise ValidationError(
                     {
@@ -88,12 +68,8 @@ class PowerPanel(PrimaryModel):
                     }
                 )
 
-        # RackGroup must belong to assigned Site and Location
+        # RackGroup must belong to assigned Location
         if self.rack_group:
-            if self.rack_group.site != self.site:
-                raise ValidationError(
-                    f"Rack group {self.rack_group} ({self.rack_group.site}) is in a different site than {self.site}"
-                )
             if (
                 self.location is not None
                 and self.rack_group.location is not None
@@ -109,23 +85,22 @@ class PowerPanel(PrimaryModel):
 
 @extras_features(
     "cable_terminations",
-    "custom_fields",
     "custom_links",
     "custom_validators",
     "export_templates",
     "graphql",
-    "relationships",
     "statuses",
     "webhooks",
 )
-class PowerFeed(PrimaryModel, PathEndpoint, CableTermination, StatusModel):
+class PowerFeed(PrimaryModel, PathEndpoint, CableTermination):
     """
     An electrical circuit delivered from a PowerPanel.
     """
 
-    power_panel = models.ForeignKey(to="PowerPanel", on_delete=models.PROTECT, related_name="powerfeeds")
-    rack = models.ForeignKey(to="Rack", on_delete=models.PROTECT, blank=True, null=True)
+    power_panel = models.ForeignKey(to="PowerPanel", on_delete=models.PROTECT, related_name="power_feeds")
+    rack = models.ForeignKey(to="Rack", on_delete=models.PROTECT, blank=True, null=True, related_name="power_feeds")
     name = models.CharField(max_length=100)
+    status = StatusField(blank=False, null=False)
     type = models.CharField(
         max_length=50,
         choices=PowerFeedTypeChoices,
@@ -151,21 +126,6 @@ class PowerFeed(PrimaryModel, PathEndpoint, CableTermination, StatusModel):
     available_power = models.PositiveIntegerField(default=0, editable=False)
     comments = models.TextField(blank=True)
 
-    csv_headers = [
-        "site",
-        "power_panel",
-        "rack_group",
-        "rack",
-        "name",
-        "status",
-        "type",
-        "supply",
-        "phase",
-        "voltage",
-        "amperage",
-        "max_utilization",
-        "comments",
-    ]
     clone_fields = [
         "power_panel",
         "rack",
@@ -186,33 +146,13 @@ class PowerFeed(PrimaryModel, PathEndpoint, CableTermination, StatusModel):
     def __str__(self):
         return self.name
 
-    def get_absolute_url(self):
-        return reverse("dcim:powerfeed", args=[self.pk])
-
-    def to_csv(self):
-        return (
-            self.power_panel.site.name,
-            self.power_panel.name,
-            self.rack.group.name if self.rack and self.rack.group else None,
-            self.rack.name if self.rack else None,
-            self.name,
-            self.get_status_display(),
-            self.get_type_display(),
-            self.get_supply_display(),
-            self.get_phase_display(),
-            self.voltage,
-            self.amperage,
-            self.max_utilization,
-            self.comments,
-        )
-
     def clean(self):
         super().clean()
 
-        # Rack must belong to same Site as PowerPanel
-        if self.rack and self.rack.site != self.power_panel.site:
+        # Rack must belong to same Location as PowerPanel
+        if self.rack and self.rack.location != self.power_panel.location:
             raise ValidationError(
-                f"Rack {self.rack} ({self.rack.site}) and power panel {self.power_panel} ({self.power_panel.site}) are in different sites"
+                f"Rack {self.rack} ({self.rack.location}) and power panel {self.power_panel} ({self.power_panel.location}) are in different locations"
             )
 
         # AC voltage cannot be negative
@@ -220,7 +160,6 @@ class PowerFeed(PrimaryModel, PathEndpoint, CableTermination, StatusModel):
             raise ValidationError({"voltage": "Voltage cannot be negative for AC supply"})
 
     def save(self, *args, **kwargs):
-
         # Cache the available_power property on the instance
         kva = abs(self.voltage) * self.amperage * (self.max_utilization / 100)
         if self.phase == PowerFeedPhaseChoices.PHASE_3PHASE:
