@@ -1,4 +1,13 @@
-from nautobot.core.apps import NavMenuAddButton, NavMenuGroup, NavMenuItem, NavMenuImportButton, NavMenuTab
+from nautobot.core.apps import (
+    NavContext,
+    NavGrouping,
+    NavItem,
+    NavMenuAddButton,
+    NavMenuGroup,
+    NavMenuItem,
+    NavMenuImportButton,
+    NavMenuTab,
+)
 
 
 menu_items = (
@@ -6,56 +15,6 @@ menu_items = (
         name="Organization",
         weight=100,
         groups=(
-            NavMenuGroup(
-                name="Sites",
-                weight=100,
-                items=(
-                    NavMenuItem(
-                        link="dcim:site_list",
-                        name="Sites",
-                        weight=100,
-                        permissions=[
-                            "dcim.view_site",
-                        ],
-                        buttons=(
-                            NavMenuAddButton(
-                                link="dcim:site_add",
-                                permissions=[
-                                    "dcim.add_site",
-                                ],
-                            ),
-                            NavMenuImportButton(
-                                link="dcim:site_import",
-                                permissions=[
-                                    "dcim.add_site",
-                                ],
-                            ),
-                        ),
-                    ),
-                    NavMenuItem(
-                        link="dcim:region_list",
-                        name="Regions",
-                        weight=200,
-                        permissions=[
-                            "dcim.view_region",
-                        ],
-                        buttons=(
-                            NavMenuAddButton(
-                                link="dcim:region_add",
-                                permissions=[
-                                    "dcim.add_region",
-                                ],
-                            ),
-                            NavMenuImportButton(
-                                link="dcim:region_import",
-                                permissions=[
-                                    "dcim.add_region",
-                                ],
-                            ),
-                        ),
-                    ),
-                ),
-            ),
             NavMenuGroup(
                 name="Locations",
                 weight=150,
@@ -154,28 +113,6 @@ menu_items = (
                             ),
                         ),
                     ),
-                    NavMenuItem(
-                        link="dcim:rackrole_list",
-                        name="Rack Roles",
-                        weight=300,
-                        permissions=[
-                            "dcim.view_rackrole",
-                        ],
-                        buttons=(
-                            NavMenuAddButton(
-                                link="dcim:rackrole_add",
-                                permissions=[
-                                    "dcim.add_rackrole",
-                                ],
-                            ),
-                            NavMenuImportButton(
-                                link="dcim:rackrole_import",
-                                permissions=[
-                                    "dcim.add_rackrole",
-                                ],
-                            ),
-                        ),
-                    ),
                     NavMenuItem(
                         link="dcim:rackreservation_list",
                         name="Reservations",
@@ -241,28 +178,6 @@ menu_items = (
                             ),
                         ),
                     ),
-                    NavMenuItem(
-                        link="dcim:devicerole_list",
-                        name="Device Roles",
-                        weight=200,
-                        permissions=[
-                            "dcim.view_devicerole",
-                        ],
-                        buttons=(
-                            NavMenuAddButton(
-                                link="dcim:devicerole_add",
-                                permissions=[
-                                    "dcim.add_devicerole",
-                                ],
-                            ),
-                            NavMenuImportButton(
-                                link="dcim:devicerole_import",
-                                permissions=[
-                                    "dcim.add_devicerole",
-                                ],
-                            ),
-                        ),
-                    ),
                     NavMenuItem(
                         link="dcim:platform_list",
                         name="Platforms",
@@ -329,6 +244,28 @@ menu_items = (
                             ),
                         ),
                     ),
+                    NavMenuItem(
+                        link="dcim:interfaceredundancygroup_list",
+                        name="Interface Redundancy Groups",
+                        weight=600,
+                        permissions=[
+                            "dcim.view_interfaceredundancygroup",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="dcim:interfaceredundancygroup_add",
+                                permissions=[
+                                    "dcim.add_interfaceredundancygroup",
+                                ],
+                            ),
+                            NavMenuImportButton(
+                                link="dcim:interfaceredundancygroup_import",
+                                permissions=[
+                                    "dcim.add_interfaceredundancygroup",
+                                ],
+                            ),
+                        ),
+                    ),
                 ),
             ),
             NavMenuGroup(
@@ -639,3 +576,223 @@ menu_items = (
         ),
     ),
 )
+
+
+navigation = (
+    NavContext(
+        name="Inventory",
+        groups=(
+            NavGrouping(
+                name="Devices",
+                weight=100,
+                items=(
+                    NavItem(
+                        name="Devices",
+                        link="dcim:device_list",
+                        weight=100,
+                        permissions=["dcim.view_device"],
+                    ),
+                    NavItem(
+                        name="Device Types",
+                        weight=200,
+                        link="dcim:devicetype_list",
+                        permissions=["dcim.view_devicetype"],
+                    ),
+                    NavItem(
+                        link="dcim:platform_list",
+                        name="Platforms",
+                        weight=300,
+                        permissions=[
+                            "dcim.view_platform",
+                        ],
+                    ),
+                    NavItem(
+                        link="dcim:manufacturer_list",
+                        name="Manufacturers",
+                        weight=400,
+                        permissions=[
+                            "dcim.view_manufacturer",
+                        ],
+                    ),
+                    NavItem(
+                        link="dcim:virtualchassis_list",
+                        name="Virtual Chassis",
+                        weight=500,
+                        permissions=[
+                            "dcim.view_virtualchassis",
+                        ],
+                    ),
+                    NavItem(
+                        name="Device Redundancy Groups",
+                        weight=600,
+                        link="dcim:deviceredundancygroup_list",
+                        permissions=["dcim.view_deviceredundancygroup"],
+                    ),
+                    NavGrouping(
+                        name="Connections",
+                        weight=700,
+                        items=(
+                            NavItem(
+                                name="Cables",
+                                weight=100,
+                                link="dcim:cable_list",
+                                permissions=["dcim.view_cable"],
+                            ),
+                            NavItem(
+                                name="Console Connections",
+                                weight=200,
+                                link="dcim:console_connections_list",
+                                permissions=[
+                                    "dcim.view_consoleport",
+                                    "dcim.view_consoleserverport",
+                                ],
+                            ),
+                            NavItem(
+                                name="Power Connections",
+                                weight=300,
+                                link="dcim:power_connections_list",
+                                permissions=[
+                                    "dcim.view_powerport",
+                                    "dcim.view_poweroutlet",
+                                ],
+                            ),
+                            NavItem(
+                                name="Interface Connections",
+                                weight=400,
+                                link="dcim:interface_connections_list",
+                                permissions=["dcim.view_interface"],
+                            ),
+                        ),
+                    ),
+                    NavGrouping(
+                        name="Components",
+                        weight=800,
+                        items=(
+                            NavItem(
+                                name="Interfaces",
+                                weight=100,
+                                link="dcim:interface_list",
+                                permissions=["dcim.view_interface"],
+                            ),
+                            NavItem(
+                                name="Front Ports",
+                                weight=200,
+                                link="dcim:frontport_list",
+                                permissions=["dcim.view_frontport"],
+                            ),
+                            NavItem(
+                                name="Rear Ports",
+                                weight=300,
+                                link="dcim:rearport_list",
+                                permissions=["dcim.view_rearport"],
+                            ),
+                            NavItem(
+                                name="Console Ports",
+                                weight=400,
+                                link="dcim:consoleport_list",
+                                permissions=["dcim.view_consoleport"],
+                            ),
+                            NavItem(
+                                name="Console Server Ports",
+                                weight=500,
+                                link="dcim:consoleserverport_list",
+                                permissions=["dcim.view_consoleserverport"],
+                            ),
+                            NavItem(
+                                name="Power Ports",
+                                weight=600,
+                                link="dcim:powerport_list",
+                                permissions=["dcim.view_powerport"],
+                            ),
+                            NavItem(
+                                name="Power Outlets",
+                                weight=700,
+                                link="dcim:poweroutlet_list",
+                                permissions=["dcim.view_poweroutlet"],
+                            ),
+                            NavItem(
+                                name="Device Bays",
+                                weight=800,
+                                link="dcim:devicebay_list",
+                                permissions=["dcim.view_devicebay"],
+                            ),
+                            NavItem(
+                                name="Inventory Items",
+                                weight=900,
+                                link="dcim:inventoryitem_list",
+                                permissions=["dcim.view_inventoryitem"],
+                            ),
+                        ),
+                    ),
+                    # space reserved for Dynamic Groups item with weight 900
+                    NavItem(
+                        name="Racks",
+                        weight=1000,
+                        link="dcim:rack_list",
+                        permissions=["dcim.view_rack"],
+                    ),
+                    NavItem(
+                        name="Rack Groups",
+                        weight=1100,
+                        link="dcim:rackgroup_list",
+                        permissions=["dcim.view_rackgroup"],
+                    ),
+                    NavItem(
+                        name="Rack Reservations",
+                        weight=1200,
+                        link="dcim:rackreservation_list",
+                        permissions=["dcim.view_rackreservation"],
+                    ),
+                    NavItem(
+                        name="Rack Elevations",
+                        weight=1300,
+                        link="dcim:rack_elevation_list",
+                        permissions=["dcim.view_rack"],
+                    ),
+                ),
+            ),
+            NavGrouping(
+                name="Organization",
+                weight=200,
+                items=(
+                    NavItem(
+                        link="dcim:location_list",
+                        name="Locations",
+                        weight=100,
+                        permissions=[
+                            "dcim.view_location",
+                        ],
+                    ),
+                    NavItem(
+                        link="dcim:locationtype_list",
+                        name="Location Types",
+                        weight=200,
+                        permissions=[
+                            "dcim.view_locationtype",
+                        ],
+                    ),
+                ),
+            ),
+            # space reserved for Tenants at weight 300
+            # space reserved for Circuits at weight 400
+            NavGrouping(
+                name="Power",
+                weight=500,
+                items=(
+                    NavItem(
+                        name="Power Feeds",
+                        link="dcim:powerfeed_list",
+                        weight=100,
+                        permissions=["dcim.view_powerfeed"],
+                    ),
+                    NavItem(
+                        name="Power Panels",
+                        weight=200,
+                        link="dcim:powerpanel_list",
+                        permissions=["dcim.view_powerpanel"],
+                    ),
+                ),
+            ),
+        ),
+    ),
+)
