@@ -1,14 +1,25 @@
 import django_tables2 as tables
 from django_tables2.utils import Accessor
 
+from nautobot.core.tables import (
+    BaseTable,
+    BooleanColumn,
+    ButtonsColumn,
+    ColoredLabelColumn,
+    LinkedCountColumn,
+    TagColumn,
+    ToggleColumn,
+)
 from nautobot.dcim.models import (
     ConsolePort,
     ConsoleServerPort,
     Device,
     DeviceBay,
-    DeviceRole,
+    DeviceRedundancyGroup,
     FrontPort,
     Interface,
+    InterfaceRedundancyGroup,
+    InterfaceRedundancyGroupAssociation,
     InventoryItem,
     Platform,
     PowerOutlet,
@@ -17,18 +28,8 @@ from nautobot.dcim.models import (
     VirtualChassis,
 )
 from nautobot.dcim.utils import cable_status_color_css
-from nautobot.extras.tables import StatusTableMixin
+from nautobot.extras.tables import RoleTableMixin, StatusTableMixin
 from nautobot.tenancy.tables import TenantColumn
-from nautobot.utilities.tables import (
-    BaseTable,
-    BooleanColumn,
-    ButtonsColumn,
-    ColorColumn,
-    ColoredLabelColumn,
-    LinkedCountColumn,
-    TagColumn,
-    ToggleColumn,
-)
 from .template_code import (
     CABLETERMINATION,
     CONSOLEPORT_BUTTONS,
@@ -39,7 +40,13 @@ from .template_code import (
     FRONTPORT_BUTTONS,
     INTERFACE_BUTTONS,
     INTERFACE_IPADDRESSES,
+    INTERFACE_REDUNDANCY_GROUP_INTERFACES,
+    INTERFACE_REDUNDANCY_GROUP_INTERFACES_IPADDRESSES,
+    INTERFACE_REDUNDANCY_GROUP_STATUS,
+    INTERFACE_REDUNDANCY_INTERFACE_PRIORITY,
+    INTERFACE_REDUNDANCY_INTERFACE_STATUS,
     INTERFACE_TAGGED_VLANS,
+    LINKED_RECORD_COUNT,
     PATHENDPOINT,
     POWEROUTLET_BUTTONS,
     POWERPORT_BUTTONS,
@@ -60,10 +67,12 @@ __all__ = (
     "DevicePowerPortTable",
     "DevicePowerOutletTable",
     "DeviceRearPortTable",
-    "DeviceRoleTable",
+    "DeviceRedundancyGroupTable",
     "DeviceTable",
     "FrontPortTable",
     "InterfaceTable",
+    "InterfaceRedundancyGroupTable",
+    "InterfaceRedundancyGroupAssociationTable",
     "InventoryItemTable",
     "PlatformTable",
     "PowerOutletTable",
@@ -73,49 +82,6 @@ __all__ = (
 )
 
 
-#
-# Device roles
-#
-
-
-class DeviceRoleTable(BaseTable):
-    pk = ToggleColumn()
-    name = tables.LinkColumn()
-    device_count = LinkedCountColumn(viewname="dcim:device_list", url_params={"role": "slug"}, verbose_name="Devices")
-    vm_count = LinkedCountColumn(
-        viewname="virtualization:virtualmachine_list",
-        url_params={"role": "slug"},
-        verbose_name="VMs",
-    )
-    color = ColorColumn()
-    vm_role = BooleanColumn()
-    actions = ButtonsColumn(DeviceRole, pk_field="slug")
-
-    class Meta(BaseTable.Meta):
-        model = DeviceRole
-        fields = (
-            "pk",
-            "name",
-            "device_count",
-            "vm_count",
-            "color",
-            "vm_role",
-            "description",
-            "slug",
-            "actions",
-        )
-        default_columns = (
-            "pk",
-            "name",
-            "device_count",
-            "vm_count",
-            "color",
-            "vm_role",
-            "description",
-            "actions",
-        )
-
-
 #
 # Platforms
 #
@@ -126,15 +92,15 @@ class PlatformTable(BaseTable):
     name = tables.LinkColumn()
     device_count = LinkedCountColumn(
         viewname="dcim:device_list",
-        url_params={"platform": "slug"},
+        url_params={"platform": "pk"},
         verbose_name="Devices",
     )
-    vm_count = LinkedCountColumn(
+    virtual_machine_count = LinkedCountColumn(
         viewname="virtualization:virtualmachine_list",
-        url_params={"platform": "slug"},
+        url_params={"platform": "pk"},
         verbose_name="VMs",
     )
-    actions = ButtonsColumn(Platform, pk_field="slug")
+    actions = ButtonsColumn(Platform)
 
     class Meta(BaseTable.Meta):
         model = Platform
@@ -143,10 +109,10 @@ class PlatformTable(BaseTable):
             "name",
             "manufacturer",
             "device_count",
-            "vm_count",
-            "slug",
+            "virtual_machine_count",
             "napalm_driver",
             "napalm_args",
+            "network_driver",
             "description",
             "actions",
         )
@@ -155,8 +121,9 @@ class PlatformTable(BaseTable):
             "name",
             "manufacturer",
             "device_count",
-            "vm_count",
+            "virtual_machine_count",
             "napalm_driver",
+            "network_driver",
             "description",
             "actions",
         )
@@ -167,14 +134,12 @@ class PlatformTable(BaseTable):
 #
 
 
-class DeviceTable(StatusTableMixin, BaseTable):
+class DeviceTable(StatusTableMixin, RoleTableMixin, BaseTable):
     pk = ToggleColumn()
     name = tables.TemplateColumn(order_by=("_name",), template_code=DEVICE_LINK)
     tenant = TenantColumn()
-    site = tables.Column(linkify=True)
     location = tables.Column(linkify=True)
     rack = tables.Column(linkify=True)
-    device_role = ColoredLabelColumn(verbose_name="Role")
     device_type = tables.LinkColumn(
         viewname="dcim:devicetype",
         args=[Accessor("device_type__pk")],
@@ -188,6 +153,10 @@ class DeviceTable(StatusTableMixin, BaseTable):
     virtual_chassis = tables.LinkColumn(viewname="dcim:virtualchassis", args=[Accessor("virtual_chassis__pk")])
     vc_position = tables.Column(verbose_name="VC Position")
     vc_priority = tables.Column(verbose_name="VC Priority")
+    device_redundancy_group = tables.Column(linkify=True)
+    device_redundancy_group_priority = tables.TemplateColumn(
+        template_code="""{% if record.device_redundancy_group %}<span class="badge badge-default">{{ record.device_redundancy_group_priority|default:'None' }}</span>{% else %}—{% endif %}"""
+    )
     secrets_group = tables.Column(linkify=True)
     tags = TagColumn(url_name="dcim:device_list")
 
@@ -198,12 +167,11 @@ class DeviceTable(StatusTableMixin, BaseTable):
             "name",
             "status",
             "tenant",
-            "device_role",
+            "role",
             "device_type",
             "platform",
             "serial",
             "asset_tag",
-            "site",
             "location",
             "rack",
             "position",
@@ -215,6 +183,8 @@ class DeviceTable(StatusTableMixin, BaseTable):
             "virtual_chassis",
             "vc_position",
             "vc_priority",
+            "device_redundancy_group",
+            "device_redundancy_group_priority",
             "secrets_group",
             "tags",
         )
@@ -223,10 +193,9 @@ class DeviceTable(StatusTableMixin, BaseTable):
             "name",
             "status",
             "tenant",
-            "site",
             "location",
             "rack",
-            "device_role",
+            "role",
             "device_type",
             "primary_ip",
         )
@@ -236,9 +205,9 @@ class DeviceImportTable(BaseTable):
     name = tables.TemplateColumn(template_code=DEVICE_LINK)
     status = ColoredLabelColumn()
     tenant = TenantColumn()
-    site = tables.Column(linkify=True)
+    location = tables.Column(linkify=True)
     rack = tables.Column(linkify=True)
-    device_role = tables.Column(verbose_name="Role")
+    role = tables.Column(verbose_name="Role")
     device_type = tables.Column(verbose_name="Type")
 
     class Meta(BaseTable.Meta):
@@ -247,10 +216,10 @@ class DeviceImportTable(BaseTable):
             "name",
             "status",
             "tenant",
-            "site",
+            "location",
             "rack",
             "position",
-            "device_role",
+            "role",
             "device_type",
         )
         empty_text = False
@@ -568,6 +537,7 @@ class BaseInterfaceTable(BaseTable):
         orderable=False,
         verbose_name="Tagged VLANs",
     )
+    vrf = tables.Column(linkify=True, verbose_name="VRF")
 
 
 class InterfaceTable(StatusTableMixin, DeviceComponentTable, BaseInterfaceTable, PathEndpointTable):
@@ -586,6 +556,7 @@ class InterfaceTable(StatusTableMixin, DeviceComponentTable, BaseInterfaceTable,
             "type",
             "mgmt_only",
             "mtu",
+            "vrf",
             "mode",
             "mac_address",
             "description",
@@ -636,6 +607,7 @@ class DeviceInterfaceTable(InterfaceTable):
             "lag",
             "mgmt_only",
             "mtu",
+            "vrf",
             "mode",
             "mac_address",
             "description",
@@ -658,6 +630,7 @@ class DeviceInterfaceTable(InterfaceTable):
             "parent_interface",
             "lag",
             "mtu",
+            "vrf",
             "mode",
             "description",
             "ip_addresses",
@@ -891,7 +864,7 @@ class InventoryItemTable(DeviceComponentTable):
 
 class DeviceInventoryItemTable(InventoryItemTable):
     name = tables.TemplateColumn(
-        template_code='<a href="{{ record.get_absolute_url }}" style="padding-left: {{ record.level }}0px">'
+        template_code='<a href="{{ record.get_absolute_url }}" style="padding-left: {{ record.tree_depth }}0px">'
         "{{ value }}</a>",
         attrs={"td": {"class": "text-nowrap"}},
     )
@@ -946,3 +919,106 @@ class VirtualChassisTable(BaseTable):
         model = VirtualChassis
         fields = ("pk", "name", "domain", "master", "member_count", "tags")
         default_columns = ("pk", "name", "domain", "master", "member_count")
+
+
+#
+# Device Redundancy Group
+#
+
+
+class DeviceRedundancyGroupTable(BaseTable):
+    pk = ToggleColumn()
+    name = tables.Column(linkify=True)
+    device_count = tables.TemplateColumn(
+        template_code=LINKED_RECORD_COUNT,
+        verbose_name="Devices",
+    )
+    secrets_group = tables.Column(linkify=True)
+    tags = TagColumn(url_name="dcim:deviceredundancygroup_list")
+
+    class Meta(BaseTable.Meta):
+        model = DeviceRedundancyGroup
+        fields = ("pk", "name", "status", "failover_strategy", "device_count", "secrets_group", "tags")
+        default_columns = ("pk", "name", "status", "failover_strategy", "device_count")
+
+
+#
+# Interface Redundancy Group
+#
+
+
+class InterfaceRedundancyGroupTable(BaseTable):
+    """Table for list view."""
+
+    pk = ToggleColumn()
+    name = tables.Column(linkify=True)
+    interfaces = tables.TemplateColumn(
+        template_code=INTERFACE_REDUNDANCY_GROUP_INTERFACES,
+        orderable=False,
+        verbose_name="Interfaces",
+    )
+    actions = ButtonsColumn(InterfaceRedundancyGroup)
+
+    class Meta(BaseTable.Meta):
+        """Meta attributes."""
+
+        model = InterfaceRedundancyGroup
+        fields = (
+            "pk",
+            "name",
+            "description",
+            "protocol",
+            "protocol_group_id",
+            "interfaces",
+        )
+
+
+class InterfaceRedundancyGroupAssociationTable(BaseTable):
+    """Table for list view."""
+
+    pk = ToggleColumn()
+    interface_redundancy_group = tables.Column(linkify=True, verbose_name="Group Name")
+    interface_redundancy_group__virtual_ip = tables.Column(linkify=True, verbose_name="Virtual IP")
+    interface_redundancy_group__protocol_group_id = tables.Column(verbose_name="Group ID")
+    priority = tables.TemplateColumn(template_code=INTERFACE_REDUNDANCY_INTERFACE_PRIORITY)
+    interface__device = tables.Column(linkify=True)
+    interface = tables.Column(linkify=True)
+    interface__status = tables.TemplateColumn(template_code=INTERFACE_REDUNDANCY_INTERFACE_STATUS)
+    interface_redundancy_group__status = tables.TemplateColumn(
+        template_code=INTERFACE_REDUNDANCY_GROUP_STATUS,
+        verbose_name="Group Status",
+    )
+    interface__ip_addresses = tables.TemplateColumn(
+        template_code=INTERFACE_REDUNDANCY_GROUP_INTERFACES_IPADDRESSES,
+        orderable=False,
+        verbose_name="IP Addresses",
+    )
+    actions = ButtonsColumn(
+        model=InterfaceRedundancyGroupAssociation,
+        buttons=("edit", "delete"),
+    )
+
+    class Meta(BaseTable.Meta):
+        """Meta attributes."""
+
+        model = InterfaceRedundancyGroupAssociation
+        fields = (
+            "pk",
+            "interface_redundancy_group",
+            "interface",
+            "priority",
+            "interface_redundancy_group__status",
+            "interface_redundancy_group__virtual_ip",
+            "interface_redundancy_group__protocol",
+            "interface_redundancy_group__protocol_group_id",
+            "interface__device",
+            "interface__name",
+            "interface__status",
+            "interface__label",
+            "interface__enabled",
+            "interface__type",
+            "interface__description",
+            "interface__ip_addresses",
+        )
+
+        default_columns = ("priority", "actions")
