@@ -4,29 +4,34 @@ from django.test import TestCase
 
 from nautobot.dcim.models import (
     Device,
-    DeviceRole,
+    DeviceRedundancyGroup,
     DeviceType,
+    Location,
+    LocationType,
     Manufacturer,
-    Site,
     VirtualChassis,
 )
+from nautobot.extras.models import Role, Status
 
 
 class VirtualChassisTest(TestCase):
     """Class to test signals for VirtualChassis."""
 
-    def setUp(self):
+    @classmethod
+    def setUpTestData(cls):
         """Setup Test Data for VirtualChassis Signal tests."""
-        site = Site.objects.first()
-        manufacturer = Manufacturer.objects.create(name="Manufacturer 1", slug="manufacturer-1")
-        devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type", slug="device-type")
-        devicerole = DeviceRole.objects.create(name="Device Role", slug="device-role", color="ff0000")
+        location = Location.objects.filter(location_type=LocationType.objects.get(name="Campus")).first()
+        manufacturer = Manufacturer.objects.first()
+        devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type")
+        devicerole = Role.objects.get_for_model(Device).first()
+        devicestatus = Status.objects.get_for_model(Device).first()
 
-        self.device = Device.objects.create(
+        cls.device = Device.objects.create(
             name="Device 1",
             device_type=devicetype,
-            device_role=devicerole,
-            site=site,
+            role=devicerole,
+            status=devicestatus,
+            location=location,
         )
 
     def test_master_device_vc_assignment(self):
@@ -65,3 +70,42 @@ class VirtualChassisTest(TestCase):
         self.device.refresh_from_db()
         self.assertEqual(self.device.vc_position, 1)
         self.assertEqual(self.device.virtual_chassis, virtualchassis)
+
+
+class DeviceRedundancyGroupTest(TestCase):
+    """Class to test signals for DeviceRedundancyGroup."""
+
+    @classmethod
+    def setUpTestData(cls):
+        """Setup Test Data for DeviceRedundancyGroup Signal tests."""
+        location = Location.objects.filter(location_type=LocationType.objects.get(name="Campus")).first()
+        manufacturer = Manufacturer.objects.first()
+        devicetype = DeviceType.objects.create(manufacturer=manufacturer, model="Device Type")
+        devicerole = Role.objects.get_for_model(Device).first()
+        devicestatus = Status.objects.get_for_model(Device).first()
+
+        cls.device = Device.objects.create(
+            name="Device 1",
+            device_type=devicetype,
+            role=devicerole,
+            status=devicestatus,
+            location=location,
+        )
+
+    def test_device_redundancy_group_priority_is_null(self):
+        """Test device with not null device_redundancy_group_priority is null after its device redundancy group is deleted.
+
+        This test is for https://github.com/nautobot/nautobot/issues/4718
+        """
+        deviceredundancygroup = DeviceRedundancyGroup.objects.first()
+
+        self.device.device_redundancy_group = deviceredundancygroup
+        self.device.device_redundancy_group_priority = 1
+        self.device.validated_save()
+
+        deviceredundancygroup.delete()
+
+        self.device.refresh_from_db()
+
+        self.assertIsNone(self.device.device_redundancy_group)
+        self.assertIsNone(self.device.device_redundancy_group_priority)
