@@ -10,7 +10,6 @@ from .models import (
     ConsoleServerPort,
     Device,
     DeviceBay,
-    DeviceRole,
     DeviceType,
     FrontPort,
     Interface,
@@ -26,10 +25,7 @@ from .models import (
     Rack,
     RackGroup,
     RackReservation,
-    RackRole,
     RearPort,
-    Region,
-    Site,
     VirtualChassis,
 )
 
@@ -37,79 +33,26 @@ app_name = "dcim"
 
 router = NautobotUIViewSetRouter()
 router.register("device-redundancy-groups", views.DeviceRedundancyGroupUIViewSet)
+router.register("interface-redundancy-groups", views.InterfaceRedundancyGroupUIViewSet)
+router.register("interface-redundancy-groups-associations", views.InterfaceRedundancyGroupAssociationUIViewSet)
 
 urlpatterns = [
-    # Regions
-    path("regions/", views.RegionListView.as_view(), name="region_list"),
-    path("regions/add/", views.RegionEditView.as_view(), name="region_add"),
-    path("regions/import/", views.RegionBulkImportView.as_view(), name="region_import"),
-    path(
-        "regions/delete/",
-        views.RegionBulkDeleteView.as_view(),
-        name="region_bulk_delete",
-    ),
-    path("regions/<uuid:pk>/", views.RegionView.as_view(), name="region"),
-    path("regions/<uuid:pk>/edit/", views.RegionEditView.as_view(), name="region_edit"),
-    path(
-        "regions/<uuid:pk>/delete/",
-        views.RegionDeleteView.as_view(),
-        name="region_delete",
-    ),
-    path(
-        "regions/<uuid:pk>/changelog/",
-        ObjectChangeLogView.as_view(),
-        name="region_changelog",
-        kwargs={"model": Region},
-    ),
-    path(
-        "regions/<uuid:pk>/notes/",
-        ObjectNotesView.as_view(),
-        name="region_notes",
-        kwargs={"model": Region},
-    ),
-    # Sites
-    path("sites/", views.SiteListView.as_view(), name="site_list"),
-    path("sites/add/", views.SiteEditView.as_view(), name="site_add"),
-    path("sites/import/", views.SiteBulkImportView.as_view(), name="site_import"),
-    path("sites/edit/", views.SiteBulkEditView.as_view(), name="site_bulk_edit"),
-    path("sites/delete/", views.SiteBulkDeleteView.as_view(), name="site_bulk_delete"),
-    path("sites/<slug:slug>/", views.SiteView.as_view(), name="site"),
-    path("sites/<slug:slug>/edit/", views.SiteEditView.as_view(), name="site_edit"),
-    path("sites/<slug:slug>/delete/", views.SiteDeleteView.as_view(), name="site_delete"),
-    path(
-        "sites/<slug:slug>/changelog/",
-        ObjectChangeLogView.as_view(),
-        name="site_changelog",
-        kwargs={"model": Site},
-    ),
-    path(
-        "sites/<slug:slug>/notes/",
-        ObjectNotesView.as_view(),
-        name="site_notes",
-        kwargs={"model": Site},
-    ),
-    path(
-        "sites/<uuid:object_id>/images/add/",
-        ImageAttachmentEditView.as_view(),
-        name="site_add_image",
-        kwargs={"model": Site},
-    ),
     # Location types
     path("location-types/", views.LocationTypeListView.as_view(), name="locationtype_list"),
     path("location-types/add/", views.LocationTypeEditView.as_view(), name="locationtype_add"),
     path("location-types/import/", views.LocationTypeBulkImportView.as_view(), name="locationtype_import"),
     path("location-types/delete/", views.LocationTypeBulkDeleteView.as_view(), name="locationtype_bulk_delete"),
-    path("location-types/<slug:slug>/", views.LocationTypeView.as_view(), name="locationtype"),
-    path("location-types/<slug:slug>/edit/", views.LocationTypeEditView.as_view(), name="locationtype_edit"),
-    path("location-types/<slug:slug>/delete/", views.LocationTypeDeleteView.as_view(), name="locationtype_delete"),
+    path("location-types/<uuid:pk>/", views.LocationTypeView.as_view(), name="locationtype"),
+    path("location-types/<uuid:pk>/edit/", views.LocationTypeEditView.as_view(), name="locationtype_edit"),
+    path("location-types/<uuid:pk>/delete/", views.LocationTypeDeleteView.as_view(), name="locationtype_delete"),
     path(
-        "location-types/<slug:slug>/changelog/",
+        "location-types/<uuid:pk>/changelog/",
         ObjectChangeLogView.as_view(),
         name="locationtype_changelog",
         kwargs={"model": LocationType},
     ),
     path(
-        "location-types/<slug:slug>/notes/",
+        "location-types/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
         name="locationtype_notes",
         kwargs={"model": LocationType},
@@ -120,23 +63,23 @@ urlpatterns = [
     path("locations/edit/", views.LocationBulkEditView.as_view(), name="location_bulk_edit"),
     path("locations/import/", views.LocationBulkImportView.as_view(), name="location_import"),
     path("locations/delete/", views.LocationBulkDeleteView.as_view(), name="location_bulk_delete"),
-    path("locations/<slug:slug>/", views.LocationView.as_view(), name="location"),
-    path("locations/<slug:slug>/edit/", views.LocationEditView.as_view(), name="location_edit"),
-    path("locations/<slug:slug>/delete/", views.LocationDeleteView.as_view(), name="location_delete"),
+    path("locations/<uuid:pk>/", views.LocationView.as_view(), name="location"),
+    path("locations/<uuid:pk>/edit/", views.LocationEditView.as_view(), name="location_edit"),
+    path("locations/<uuid:pk>/delete/", views.LocationDeleteView.as_view(), name="location_delete"),
     path(
-        "locations/<slug:slug>/changelog/",
+        "locations/<uuid:pk>/changelog/",
         ObjectChangeLogView.as_view(),
         name="location_changelog",
         kwargs={"model": Location},
     ),
     path(
-        "locations/<slug:slug>/notes/",
+        "locations/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
         name="location_notes",
         kwargs={"model": Location},
     ),
     path(
-        "locations/<slug:slug>/images/add/",
+        "locations/<uuid:object_id>/images/add/",
         ImageAttachmentEditView.as_view(),
         name="location_add_image",
         kwargs={"model": Location},
@@ -177,42 +120,6 @@ urlpatterns = [
         name="rackgroup_notes",
         kwargs={"model": RackGroup},
     ),
-    # Rack roles
-    path("rack-roles/", views.RackRoleListView.as_view(), name="rackrole_list"),
-    path("rack-roles/add/", views.RackRoleEditView.as_view(), name="rackrole_add"),
-    path(
-        "rack-roles/import/",
-        views.RackRoleBulkImportView.as_view(),
-        name="rackrole_import",
-    ),
-    path(
-        "rack-roles/delete/",
-        views.RackRoleBulkDeleteView.as_view(),
-        name="rackrole_bulk_delete",
-    ),
-    path("rack-roles/<uuid:pk>/", views.RackRoleView.as_view(), name="rackrole"),
-    path(
-        "rack-roles/<uuid:pk>/edit/",
-        views.RackRoleEditView.as_view(),
-        name="rackrole_edit",
-    ),
-    path(
-        "rack-roles/<uuid:pk>/delete/",
-        views.RackRoleDeleteView.as_view(),
-        name="rackrole_delete",
-    ),
-    path(
-        "rack-roles/<uuid:pk>/changelog/",
-        ObjectChangeLogView.as_view(),
-        name="rackrole_changelog",
-        kwargs={"model": RackRole},
-    ),
-    path(
-        "rack-roles/<uuid:pk>/notes/",
-        ObjectNotesView.as_view(),
-        name="rackrole_notes",
-        kwargs={"model": RackRole},
-    ),
     # Rack reservations
     path(
         "rack-reservations/",
@@ -322,28 +229,28 @@ urlpatterns = [
         name="manufacturer_bulk_delete",
     ),
     path(
-        "manufacturers/<slug:slug>/",
+        "manufacturers/<uuid:pk>/",
         views.ManufacturerView.as_view(),
         name="manufacturer",
     ),
     path(
-        "manufacturers/<slug:slug>/edit/",
+        "manufacturers/<uuid:pk>/edit/",
         views.ManufacturerEditView.as_view(),
         name="manufacturer_edit",
     ),
     path(
-        "manufacturers/<slug:slug>/delete/",
+        "manufacturers/<uuid:pk>/delete/",
         views.ManufacturerDeleteView.as_view(),
         name="manufacturer_delete",
     ),
     path(
-        "manufacturers/<slug:slug>/changelog/",
+        "manufacturers/<uuid:pk>/changelog/",
         ObjectChangeLogView.as_view(),
         name="manufacturer_changelog",
         kwargs={"model": Manufacturer},
     ),
     path(
-        "manufacturers/<slug:slug>/notes/",
+        "manufacturers/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
         name="manufacturer_notes",
         kwargs={"model": Manufacturer},
@@ -637,42 +544,6 @@ urlpatterns = [
         views.DeviceBayTemplateDeleteView.as_view(),
         name="devicebaytemplate_delete",
     ),
-    # Device roles
-    path("device-roles/", views.DeviceRoleListView.as_view(), name="devicerole_list"),
-    path("device-roles/add/", views.DeviceRoleEditView.as_view(), name="devicerole_add"),
-    path(
-        "device-roles/import/",
-        views.DeviceRoleBulkImportView.as_view(),
-        name="devicerole_import",
-    ),
-    path(
-        "device-roles/delete/",
-        views.DeviceRoleBulkDeleteView.as_view(),
-        name="devicerole_bulk_delete",
-    ),
-    path("device-roles/<slug:slug>/", views.DeviceRoleView.as_view(), name="devicerole"),
-    path(
-        "device-roles/<slug:slug>/edit/",
-        views.DeviceRoleEditView.as_view(),
-        name="devicerole_edit",
-    ),
-    path(
-        "device-roles/<slug:slug>/delete/",
-        views.DeviceRoleDeleteView.as_view(),
-        name="devicerole_delete",
-    ),
-    path(
-        "device-roles/<slug:slug>/changelog/",
-        ObjectChangeLogView.as_view(),
-        name="devicerole_changelog",
-        kwargs={"model": DeviceRole},
-    ),
-    path(
-        "device-roles/<slug:slug>/notes/",
-        ObjectNotesView.as_view(),
-        name="devicerole_notes",
-        kwargs={"model": DeviceRole},
-    ),
     # Platforms
     path("platforms/", views.PlatformListView.as_view(), name="platform_list"),
     path("platforms/add/", views.PlatformEditView.as_view(), name="platform_add"),
@@ -686,25 +557,25 @@ urlpatterns = [
         views.PlatformBulkDeleteView.as_view(),
         name="platform_bulk_delete",
     ),
-    path("platforms/<slug:slug>/", views.PlatformView.as_view(), name="platform"),
+    path("platforms/<uuid:pk>/", views.PlatformView.as_view(), name="platform"),
     path(
-        "platforms/<slug:slug>/edit/",
+        "platforms/<uuid:pk>/edit/",
         views.PlatformEditView.as_view(),
         name="platform_edit",
     ),
     path(
-        "platforms/<slug:slug>/delete/",
+        "platforms/<uuid:pk>/delete/",
         views.PlatformDeleteView.as_view(),
         name="platform_delete",
     ),
     path(
-        "platforms/<slug:slug>/changelog/",
+        "platforms/<uuid:pk>/changelog/",
         ObjectChangeLogView.as_view(),
         name="platform_changelog",
         kwargs={"model": Platform},
     ),
     path(
-        "platforms/<slug:slug>/notes/",
+        "platforms/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
         name="platform_notes",
         kwargs={"model": Platform},
@@ -713,11 +584,6 @@ urlpatterns = [
     path("devices/", views.DeviceListView.as_view(), name="device_list"),
     path("devices/add/", views.DeviceEditView.as_view(), name="device_add"),
     path("devices/import/", views.DeviceBulkImportView.as_view(), name="device_import"),
-    path(
-        "devices/import/child-devices/",
-        views.ChildDeviceBulkImportView.as_view(),
-        name="device_import_child",
-    ),
     path("devices/edit/", views.DeviceBulkEditView.as_view(), name="device_bulk_edit"),
     path(
         "devices/delete/",
