@@ -1,5 +1,8 @@
-from nautobot.utilities.choices import ChoiceSet
-from nautobot.utilities.deprecation import class_deprecated_in_favor_of
+from celery import states
+
+from nautobot.core.choices import ChoiceSet
+from nautobot.core.utils.deprecation import class_deprecated_in_favor_of
+
 
 #
 # Banners (currently plugin-specific)
@@ -28,7 +31,6 @@ class BannerClassChoices(ChoiceSet):
 
 
 class CustomFieldFilterLogicChoices(ChoiceSet):
-
     FILTER_DISABLED = "disabled"
     FILTER_LOOSE = "loose"
     FILTER_EXACT = "exact"
@@ -41,7 +43,6 @@ class CustomFieldFilterLogicChoices(ChoiceSet):
 
 
 class CustomFieldTypeChoices(ChoiceSet):
-
     TYPE_TEXT = "text"
     TYPE_INTEGER = "integer"
     TYPE_BOOLEAN = "boolean"
@@ -50,6 +51,7 @@ class CustomFieldTypeChoices(ChoiceSet):
     TYPE_SELECT = "select"
     TYPE_MULTISELECT = "multi-select"
     TYPE_JSON = "json"
+    TYPE_MARKDOWN = "markdown"
 
     CHOICES = (
         (TYPE_TEXT, "Text"),
@@ -60,6 +62,7 @@ class CustomFieldTypeChoices(ChoiceSet):
         (TYPE_SELECT, "Selection"),
         (TYPE_MULTISELECT, "Multiple selection"),
         (TYPE_JSON, "JSON"),
+        (TYPE_MARKDOWN, "Markdown"),
     )
 
     REGEX_TYPES = (
@@ -76,7 +79,6 @@ class CustomFieldTypeChoices(ChoiceSet):
 
 
 class ButtonClassChoices(ChoiceSet):
-
     CLASS_DEFAULT = "default"
     CLASS_PRIMARY = "primary"
     CLASS_SUCCESS = "success"
@@ -129,20 +131,20 @@ class DynamicGroupOperatorChoices(ChoiceSet):
 
 
 class JobSourceChoices(ChoiceSet):
-
     SOURCE_LOCAL = "local"
     SOURCE_GIT = "git"
     SOURCE_PLUGIN = "plugins"
+    SOURCE_SYSTEM = "system"
 
     CHOICES = (
         (SOURCE_LOCAL, "Installed in $JOBS_ROOT"),
         (SOURCE_GIT, "Provided by a Git repository"),
         (SOURCE_PLUGIN, "Part of a plugin"),
+        (SOURCE_SYSTEM, "Provided by Nautobot"),
     )
 
 
 class JobExecutionType(ChoiceSet):
-
     TYPE_IMMEDIATELY = "immediately"
     TYPE_FUTURE = "future"
     TYPE_HOURLY = "hourly"
@@ -187,26 +189,53 @@ class JobExecutionType(ChoiceSet):
 
 
 class JobResultStatusChoices(ChoiceSet):
+    """
+    These status choices are using the same taxonomy as within Celery core. A Nautobot Job status
+    is equivalent to a Celery task state.
+    """
 
-    STATUS_PENDING = "pending"
-    STATUS_RUNNING = "running"
-    STATUS_COMPLETED = "completed"
-    STATUS_ERRORED = "errored"
-    STATUS_FAILED = "failed"
+    STATUS_FAILURE = states.FAILURE
+    STATUS_PENDING = states.PENDING
+    STATUS_RECEIVED = states.RECEIVED
+    STATUS_RETRY = states.RETRY
+    STATUS_REVOKED = states.REVOKED
+    STATUS_STARTED = states.STARTED
+    STATUS_SUCCESS = states.SUCCESS
 
-    CHOICES = (
-        (STATUS_PENDING, "Pending"),
-        (STATUS_RUNNING, "Running"),
-        (STATUS_COMPLETED, "Completed"),
-        (STATUS_ERRORED, "Errored"),
-        (STATUS_FAILED, "Failed"),
-    )
+    CHOICES = sorted(zip(states.ALL_STATES, states.ALL_STATES))
 
-    TERMINAL_STATE_CHOICES = (
-        STATUS_COMPLETED,
-        STATUS_ERRORED,
-        STATUS_FAILED,
-    )
+    #: Set of all possible states.
+    ALL_STATES = states.ALL_STATES
+    #: Set of states meaning the task returned an exception.
+    EXCEPTION_STATES = states.EXCEPTION_STATES
+    #: State precedence.
+    #: None represents the precedence of an unknown state.
+    #: Lower index means higher precedence.
+    PRECEDENCE = states.PRECEDENCE
+    #: Set of exception states that should propagate exceptions to the user.
+    PROPAGATE_STATES = states.PROPAGATE_STATES
+    #: Set of states meaning the task result is ready (has been executed).
+    READY_STATES = states.READY_STATES
+    #: Set of states meaning the task result is not ready (hasn't been executed).
+    UNREADY_STATES = states.UNREADY_STATES
+
+    @staticmethod
+    def precedence(state):
+        """
+        Get the precedence for a state. Lower index means higher precedence.
+
+        Args:
+            state (str): One of the status choices.
+
+        Returns:
+            (int): Precedence value.
+
+        Examples:
+            >>> JobResultStatusChoices.precedence(JobResultStatusChoices.STATUS_SUCCESS)
+            0
+
+        """
+        return states.precedence(state)
 
 
 #
@@ -215,27 +244,26 @@ class JobResultStatusChoices(ChoiceSet):
 
 
 class LogLevelChoices(ChoiceSet):
-
-    LOG_DEFAULT = "default"
-    LOG_SUCCESS = "success"
+    LOG_DEBUG = "debug"
     LOG_INFO = "info"
     LOG_WARNING = "warning"
-    LOG_FAILURE = "failure"
+    LOG_ERROR = "error"
+    LOG_CRITICAL = "critical"
 
     CHOICES = (
-        (LOG_DEFAULT, "Default"),
-        (LOG_SUCCESS, "Success"),
+        (LOG_DEBUG, "Debug"),
         (LOG_INFO, "Info"),
         (LOG_WARNING, "Warning"),
-        (LOG_FAILURE, "Failure"),
+        (LOG_ERROR, "Error"),
+        (LOG_CRITICAL, "Critical"),
     )
 
     CSS_CLASSES = {
-        LOG_DEFAULT: "default",
-        LOG_SUCCESS: "success",
+        LOG_DEBUG: "debug",
         LOG_INFO: "info",
         LOG_WARNING: "warning",
-        LOG_FAILURE: "danger",
+        LOG_ERROR: "error",
+        LOG_CRITICAL: "critical",
     }
 
 
@@ -245,7 +273,6 @@ class LogLevelChoices(ChoiceSet):
 
 
 class ObjectChangeActionChoices(ChoiceSet):
-
     ACTION_CREATE = "create"
     ACTION_UPDATE = "update"
     ACTION_DELETE = "delete"
@@ -264,7 +291,6 @@ class ObjectChangeActionChoices(ChoiceSet):
 
 
 class ObjectChangeEventContextChoices(ChoiceSet):
-
     CONTEXT_WEB = "web"
     CONTEXT_JOB = "job"
     CONTEXT_JOB_HOOK = "job-hook"
@@ -298,7 +324,6 @@ class RelationshipRequiredSideChoices(ChoiceSet):
 
 
 class RelationshipSideChoices(ChoiceSet):
-
     SIDE_SOURCE = "source"
     SIDE_DESTINATION = "destination"
     SIDE_PEER = "peer"  # for symmetric / non-directional relationships
@@ -317,7 +342,6 @@ class RelationshipSideChoices(ChoiceSet):
 
 
 class RelationshipTypeChoices(ChoiceSet):
-
     TYPE_ONE_TO_ONE = "one-to-one"
     TYPE_ONE_TO_ONE_SYMMETRIC = "symmetric-one-to-one"
     TYPE_ONE_TO_MANY = "one-to-many"
@@ -339,7 +363,6 @@ class RelationshipTypeChoices(ChoiceSet):
 
 
 class SecretsGroupAccessTypeChoices(ChoiceSet):
-
     TYPE_GENERIC = "Generic"
 
     TYPE_CONSOLE = "Console"
@@ -365,7 +388,6 @@ class SecretsGroupAccessTypeChoices(ChoiceSet):
 
 
 class SecretsGroupSecretTypeChoices(ChoiceSet):
-
     TYPE_KEY = "key"
     TYPE_PASSWORD = "password"
     TYPE_SECRET = "secret"
@@ -387,7 +409,6 @@ class SecretsGroupSecretTypeChoices(ChoiceSet):
 
 
 class WebhookHttpMethodChoices(ChoiceSet):
-
     METHOD_GET = "GET"
     METHOD_POST = "POST"
     METHOD_PUT = "PUT"
