@@ -24,9 +24,8 @@ JOB_OVERRIDABLE_FIELDS = (
     "grouping",
     "name",
     "description",
-    "commit_default",
+    "dryrun_default",
     "hidden",
-    "read_only",
     "approval_required",
     "soft_time_limit",
     "time_limit",
@@ -37,9 +36,7 @@ JOB_OVERRIDABLE_FIELDS = (
 
 # Job field length limits
 JOB_MAX_NAME_LENGTH = 100  # TODO(Glenn): this should really be a more general-purpose constant, like NAME_MAX_LENGTH
-JOB_MAX_SLUG_LENGTH = 320  # 16 source, 100 GitRepository.name, 100 module_name, 100 job_class_name
 JOB_MAX_GROUPING_LENGTH = 255
-JOB_MAX_SOURCE_LENGTH = 16  # "git", "local", "plugins"
 
 
 # JobLogEntry Truncation Length
@@ -50,3 +47,11 @@ JOB_LOG_MAX_ABSOLUTE_URL_LENGTH = 255
 # ChangeLog Truncation Length
 CHANGELOG_MAX_CHANGE_CONTEXT_DETAIL = 400
 CHANGELOG_MAX_OBJECT_REPR = 200
+
+# JobResult custom Celery kwargs
+JOB_RESULT_CUSTOM_CELERY_KWARGS = (
+    "nautobot_job_profile",
+    "nautobot_job_job_model_id",
+    "nautobot_job_scheduled_job_id",
+    "nautobot_job_user_id",
+)
