@@ -3,7 +3,16 @@ from django.conf import settings
 from django.contrib.auth import get_user_model
 from django.contrib.contenttypes.models import ContentType
 
-from nautobot.dcim.models import DeviceRedundancyGroup, DeviceRole, DeviceType, Location, Platform, Region, Site
+from nautobot.core.filters import (
+    BaseFilterSet,
+    ContentTypeFilter,
+    ContentTypeMultipleChoiceFilter,
+    MultiValueUUIDFilter,
+    NaturalKeyOrPKMultipleChoiceFilter,
+    SearchFilter,
+)
+from nautobot.core.utils.deprecation import class_deprecated_in_favor_of
+from nautobot.dcim.models import DeviceRedundancyGroup, DeviceType, Location, Platform
 from nautobot.extras.choices import (
     JobResultStatusChoices,
     RelationshipTypeChoices,
@@ -23,11 +32,13 @@ from nautobot.extras.filters.customfields import (
     CustomFieldNumberFilter,
 )
 from nautobot.extras.filters.mixins import (
+    ConfigContextRoleFilter,
     CustomFieldModelFilterSetMixin,
     CreatedUpdatedModelFilterSetMixin,
     LocalContextModelFilterSetMixin,
     RelationshipFilter,
     RelationshipModelFilterSetMixin,
+    RoleModelFilterSetMixin,
     StatusFilter,
     StatusModelFilterSetMixin,
 )
@@ -53,6 +64,7 @@ from nautobot.extras.models import (
     ObjectChange,
     Relationship,
     RelationshipAssociation,
+    Role,
     ScheduledJob,
     Secret,
     SecretsGroup,
@@ -61,18 +73,8 @@ from nautobot.extras.models import (
     Tag,
     Webhook,
 )
-from nautobot.extras.utils import ChangeLoggedModelsQuery, FeatureQuery, TaggableClassesQuery
+from nautobot.extras.utils import ChangeLoggedModelsQuery, FeatureQuery, RoleModelsQuery, TaggableClassesQuery
 from nautobot.tenancy.models import Tenant, TenantGroup
-from nautobot.utilities.deprecation import class_deprecated_in_favor_of
-from nautobot.utilities.filters import (
-    BaseFilterSet,
-    ContentTypeFilter,
-    ContentTypeMultipleChoiceFilter,
-    MultiValueUUIDFilter,
-    NaturalKeyOrPKMultipleChoiceFilter,
-    SearchFilter,
-    TagFilter,
-)
 from nautobot.virtualization.models import Cluster, ClusterGroup
 
 
@@ -113,6 +115,8 @@ __all__ = (
     "RelationshipFilter",
     "RelationshipFilterSet",
     "RelationshipAssociationFilterSet",
+    "RoleFilterSet",
+    "RoleModelFilterSetMixin",
     "ScheduledJobFilterSet",
     "SecretFilterSet",
     "SecretsGroupFilterSet",
@@ -158,7 +162,7 @@ class ComputedFieldFilterSet(BaseFilterSet):
         model = ComputedField
         fields = (
             "content_type",
-            "slug",
+            "key",
             "template",
             "fallback_value",
             "weight",
@@ -180,30 +184,10 @@ class ConfigContextFilterSet(BaseFilterSet):
     )
     owner_content_type = ContentTypeFilter()
     schema = NaturalKeyOrPKMultipleChoiceFilter(
-        field_name="schema",
+        field_name="config_context_schema",
         queryset=ConfigContextSchema.objects.all(),
-        to_field_name="slug",
-        label="Schema (slug or PK)",
-    )
-    region_id = django_filters.ModelMultipleChoiceFilter(
-        field_name="regions",
-        queryset=Region.objects.all(),
-        label="Region (ID) - Deprecated (use region filter)",
-    )
-    region = NaturalKeyOrPKMultipleChoiceFilter(
-        field_name="regions",
-        queryset=Region.objects.all(),
-        label="Region (ID or slug)",
-    )
-    site_id = django_filters.ModelMultipleChoiceFilter(
-        field_name="sites",
-        queryset=Site.objects.all(),
-        label="Site (ID) - Deprecated (use site filter)",
-    )
-    site = NaturalKeyOrPKMultipleChoiceFilter(
-        field_name="sites",
-        queryset=Site.objects.all(),
-        label="Site (ID or slug)",
+        to_field_name="name",
+        label="Schema (name or PK)",
     )
     location_id = django_filters.ModelMultipleChoiceFilter(
         field_name="locations",
@@ -213,17 +197,8 @@ class ConfigContextFilterSet(BaseFilterSet):
     location = NaturalKeyOrPKMultipleChoiceFilter(
         field_name="locations",
         queryset=Location.objects.all(),
-        label="Location (ID or slug)",
-    )
-    role_id = django_filters.ModelMultipleChoiceFilter(
-        field_name="roles",
-        queryset=DeviceRole.objects.all(),
-        label="Role (ID) - Deprecated (use role filter)",
-    )
-    role = NaturalKeyOrPKMultipleChoiceFilter(
-        field_name="roles",
-        queryset=DeviceRole.objects.all(),
-        label="Role (ID or slug)",
+        to_field_name="name",
+        label="Location (name or ID)",
     )
     device_type_id = django_filters.ModelMultipleChoiceFilter(
         field_name="device_types",
@@ -233,7 +208,8 @@ class ConfigContextFilterSet(BaseFilterSet):
     device_type = NaturalKeyOrPKMultipleChoiceFilter(
         field_name="device_types",
         queryset=DeviceType.objects.all(),
-        label="Device Type (ID or slug)",
+        to_field_name="model",
+        label="Device Type (model or ID)",
     )
     platform_id = django_filters.ModelMultipleChoiceFilter(
         field_name="platforms",
@@ -243,7 +219,8 @@ class ConfigContextFilterSet(BaseFilterSet):
     platform = NaturalKeyOrPKMultipleChoiceFilter(
         field_name="platforms",
         queryset=Platform.objects.all(),
-        label="Platform (ID or slug)",
+        to_field_name="name",
+        label="Platform (ID or name)",
     )
     cluster_group_id = django_filters.ModelMultipleChoiceFilter(
         field_name="cluster_groups",
@@ -253,7 +230,8 @@ class ConfigContextFilterSet(BaseFilterSet):
     cluster_group = NaturalKeyOrPKMultipleChoiceFilter(
         field_name="cluster_groups",
         queryset=ClusterGroup.objects.all(),
-        label="Cluster group (ID or slug)",
+        to_field_name="name",
+        label="Cluster group (ID or name)",
     )
     cluster_id = django_filters.ModelMultipleChoiceFilter(
         field_name="clusters",
@@ -268,7 +246,8 @@ class ConfigContextFilterSet(BaseFilterSet):
     tenant_group = NaturalKeyOrPKMultipleChoiceFilter(
         field_name="tenant_groups",
         queryset=TenantGroup.objects.all(),
-        label="Tenant group (ID or slug)",
+        label="Tenant group (ID or name)",
+        to_field_name="name",
     )
     tenant_id = django_filters.ModelMultipleChoiceFilter(
         field_name="tenants",
@@ -278,20 +257,22 @@ class ConfigContextFilterSet(BaseFilterSet):
     tenant = NaturalKeyOrPKMultipleChoiceFilter(
         field_name="tenants",
         queryset=Tenant.objects.all(),
-        label="Tenant (ID or slug)",
+        label="Tenant (ID or name)",
+        to_field_name="name",
     )
     device_redundancy_group = NaturalKeyOrPKMultipleChoiceFilter(
         field_name="device_redundancy_groups",
         queryset=DeviceRedundancyGroup.objects.all(),
-        to_field_name="slug",
-        label="Device Redundancy Group (slug or PK)",
+        to_field_name="name",
+        label="Device Redundancy Group (name or PK)",
     )
     tag = django_filters.ModelMultipleChoiceFilter(
-        field_name="tags__slug",
+        field_name="tags",
         queryset=Tag.objects.all(),
-        to_field_name="slug",
-        label="Tag (slug)",
+        to_field_name="name",
+        label="Tag (name)",
     )
+    role = ConfigContextRoleFilter()
 
     # Conditional enablement of dynamic groups filtering
     def __init__(self, *args, **kwargs):
@@ -300,7 +281,8 @@ class ConfigContextFilterSet(BaseFilterSet):
         if settings.CONFIG_CONTEXT_DYNAMIC_GROUPS_ENABLED:
             self.filters["dynamic_groups"] = NaturalKeyOrPKMultipleChoiceFilter(
                 queryset=DynamicGroup.objects.all(),
-                label="Dynamic Groups (slug or ID)",
+                label="Dynamic Groups (name or ID)",
+                to_field_name="name",
             )
 
     class Meta:
@@ -359,7 +341,6 @@ class CustomFieldModelFilterSet(CustomFieldModelFilterSetMixin):
 class CustomFieldFilterSet(BaseFilterSet):
     q = SearchFilter(
         filter_predicates={
-            "name": "icontains",
             "label": "icontains",
             "description": "icontains",
         },
@@ -370,20 +351,15 @@ class CustomFieldFilterSet(BaseFilterSet):
 
     class Meta:
         model = CustomField
-        fields = ["id", "content_types", "name", "required", "filter_logic", "weight"]
+        fields = ["id", "content_types", "label", "required", "filter_logic", "weight"]
 
 
 class CustomFieldChoiceFilterSet(BaseFilterSet):
     q = SearchFilter(filter_predicates={"value": "icontains"})
-    field_id = django_filters.ModelMultipleChoiceFilter(
-        field_name="field",
+    custom_field = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=CustomField.objects.all(),
-        label="Field (ID) - Deprecated (use field filter)",
-    )
-    field = NaturalKeyOrPKMultipleChoiceFilter(
-        queryset=CustomField.objects.all(),
-        to_field_name="name",
-        label="Field (ID or name)",
+        to_field_name="key",
+        label="Field (ID or Key)",
     )
 
     class Meta:
@@ -449,7 +425,6 @@ class DynamicGroupFilterSet(NautobotFilterSet):
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "slug": "icontains",
             "description": "icontains",
             "content_type__app_label": "icontains",
             "content_type__model": "icontains",
@@ -459,7 +434,7 @@ class DynamicGroupFilterSet(NautobotFilterSet):
 
     class Meta:
         model = DynamicGroup
-        fields = ("id", "name", "slug", "description")
+        fields = ("id", "name", "description")
 
 
 class DynamicGroupMembershipFilterSet(NautobotFilterSet):
@@ -467,18 +442,18 @@ class DynamicGroupMembershipFilterSet(NautobotFilterSet):
         filter_predicates={
             "operator": "icontains",
             "group__name": "icontains",
-            "group__slug": "icontains",
             "parent_group__name": "icontains",
-            "parent_group__slug": "icontains",
         },
     )
     group = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=DynamicGroup.objects.all(),
-        label="Group (slug or ID)",
+        label="Group (name or ID)",
+        to_field_name="name",
     )
     parent_group = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=DynamicGroup.objects.all(),
-        label="Parent Group (slug or ID)",
+        label="Parent Group (name or ID)",
+        to_field_name="name",
     )
 
     class Meta:
@@ -529,13 +504,13 @@ class GitRepositoryFilterSet(NautobotFilterSet):
     )
     secrets_group = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=SecretsGroup.objects.all(),
-        label="Secrets group (ID or slug)",
+        label="Secrets group (ID or name)",
+        to_field_name="name",
     )
-    tag = TagFilter()
 
     class Meta:
         model = GitRepository
-        fields = ["id", "name", "slug", "remote_url", "branch", "provided_contents"]
+        fields = ["id", "branch", "name", "provided_contents", "remote_url", "slug", "tags"]
 
 
 #
@@ -547,14 +522,15 @@ class GraphQLQueryFilterSet(BaseFilterSet):
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "slug": "icontains",
             "query": "icontains",
         },
     )
 
     class Meta:
         model = GraphQLQuery
-        fields = ["name", "slug"]
+        fields = [
+            "name",
+        ]
 
 
 #
@@ -579,28 +555,24 @@ class JobFilterSet(BaseFilterSet, CustomFieldModelFilterSetMixin):
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "slug": "icontains",
             "grouping": "icontains",
             "description": "icontains",
         },
     )
-    tag = TagFilter()
 
     class Meta:
         model = Job
         fields = [
             "id",
-            "source",
             "module_name",
             "job_class_name",
-            "slug",
             "name",
             "grouping",
             "installed",
             "enabled",
             "has_sensitive_variables",
             "approval_required",
-            "commit_default",
+            "dryrun_default",
             "hidden",
             "read_only",
             "is_job_hook_receiver",
@@ -611,23 +583,24 @@ class JobFilterSet(BaseFilterSet, CustomFieldModelFilterSetMixin):
             "name_override",
             "approval_required_override",
             "description_override",
-            "commit_default_override",
+            "dryrun_default_override",
             "hidden_override",
-            "read_only_override",
             "soft_time_limit_override",
             "time_limit_override",
             "has_sensitive_variables_override",
+            "tags",
         ]
 
 
 class JobHookFilterSet(BaseFilterSet):
-    q = SearchFilter(filter_predicates={"name": "icontains", "slug": "icontains"})
+    q = SearchFilter(filter_predicates={"name": "icontains"})
     content_types = ContentTypeMultipleChoiceFilter(
         choices=ChangeLoggedModelsQuery().get_choices,
     )
     job = NaturalKeyOrPKMultipleChoiceFilter(
+        to_field_name="name",
         queryset=Job.objects.all(),
-        label="Job (slug or ID)",
+        label="Job (name or ID)",
     )
 
     class Meta:
@@ -637,7 +610,6 @@ class JobHookFilterSet(BaseFilterSet):
             "content_types",
             "enabled",
             "job",
-            "slug",
             "type_create",
             "type_update",
             "type_delete",
@@ -653,21 +625,19 @@ class JobResultFilterSet(BaseFilterSet, CustomFieldModelFilterSetMixin):
         },
     )
     job_model = NaturalKeyOrPKMultipleChoiceFilter(
+        to_field_name="name",
         queryset=Job.objects.all(),
-        label="Job (ID or slug)",
+        label="Job (name or ID)",
     )
     job_model_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Job.objects.all(),
         label="Job (ID) - Deprecated (use job_model filter)",
     )
-    obj_type = ContentTypeFilter()
-    created = django_filters.DateTimeFilter()
-    completed = django_filters.DateTimeFilter()
     status = django_filters.MultipleChoiceFilter(choices=JobResultStatusChoices, null_value=None)
 
     class Meta:
         model = JobResult
-        fields = ["id", "created", "completed", "status", "user", "obj_type", "name"]
+        fields = ["id", "date_created", "date_done", "name", "status", "user"]
 
 
 class JobLogEntryFilterSet(BaseFilterSet):
@@ -688,26 +658,23 @@ class ScheduledJobFilterSet(BaseFilterSet):
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "job_class": "icontains",
+            "task": "icontains",
             "description": "icontains",
         },
     )
     job_model = NaturalKeyOrPKMultipleChoiceFilter(
-        field_name="job_model__slug",
+        to_field_name="name",
         queryset=Job.objects.all(),
-        label="Job (ID or slug)",
+        label="Job (name or ID)",
     )
     job_model_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Job.objects.all(),
         label="Job (ID) - Deprecated (use job_model filter)",
     )
 
-    first_run = django_filters.DateTimeFilter()
-    last_run = django_filters.DateTimeFilter()
-
     class Meta:
         model = ScheduledJob
-        fields = ["id", "name", "total_run_count"]
+        fields = ["id", "name", "total_run_count", "start_time", "last_run_at"]
 
 
 #
@@ -724,7 +691,11 @@ class JobButtonFilterSet(BaseFilterSet):
         },
     )
     content_types = ContentTypeFilter()
-    job = NaturalKeyOrPKMultipleChoiceFilter(queryset=Job.objects.all(), label="Job (slug or ID)")
+    job = NaturalKeyOrPKMultipleChoiceFilter(
+        to_field_name="name",
+        queryset=Job.objects.all(),
+        label="Job (name or ID)",
+    )
 
     class Meta:
         model = JobButton
@@ -819,7 +790,7 @@ class ObjectChangeFilterSet(BaseFilterSet):
 class RelationshipFilterSet(BaseFilterSet):
     q = SearchFilter(
         filter_predicates={
-            "name": "icontains",
+            "label": "icontains",
             "description": "icontains",
         }
     )
@@ -831,16 +802,15 @@ class RelationshipFilterSet(BaseFilterSet):
 
     class Meta:
         model = Relationship
-        fields = ["id", "name", "slug", "type", "source_type", "destination_type"]
+        fields = ["id", "label", "key", "type", "source_type", "destination_type"]
 
 
 class RelationshipAssociationFilterSet(BaseFilterSet):
-
     relationship = django_filters.ModelMultipleChoiceFilter(
-        field_name="relationship__slug",
+        field_name="relationship__key",
         queryset=Relationship.objects.all(),
-        to_field_name="slug",
-        label="Relationship (slug)",
+        to_field_name="key",
+        label="Relationship (key)",
     )
     source_type = ContentTypeMultipleChoiceFilter(choices=FeatureQuery("relationships").get_choices, conjoined=False)
     destination_type = ContentTypeMultipleChoiceFilter(
@@ -880,7 +850,6 @@ class SecretFilterSet(
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "slug": "icontains",
         },
     )
     # TODO(Glenn): dynamic choices needed. The issue being that secrets providers are Python
@@ -889,7 +858,7 @@ class SecretFilterSet(
 
     class Meta:
         model = Secret
-        fields = ("id", "name", "slug", "provider", "created", "last_updated")
+        fields = ("id", "name", "provider", "created", "last_updated", "tags")
 
 
 class SecretsGroupFilterSet(
@@ -902,25 +871,21 @@ class SecretsGroupFilterSet(
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "slug": "icontains",
         },
     )
 
     class Meta:
         model = SecretsGroup
-        fields = ("id", "name", "slug", "created", "last_updated")
+        fields = ("id", "name", "created", "last_updated")
 
 
 class SecretsGroupAssociationFilterSet(BaseFilterSet):
     """Filterset for the SecretsGroupAssociation through model."""
 
-    group_id = django_filters.ModelMultipleChoiceFilter(
+    secrets_group = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=SecretsGroup.objects.all(),
-        label="Group (ID) - Deprecated (use group filter)",
-    )
-    group = NaturalKeyOrPKMultipleChoiceFilter(
-        queryset=SecretsGroup.objects.all(),
-        label="Group (ID or slug)",
+        label="Secrets Group (ID or name)",
+        to_field_name="name",
     )
     secret_id = django_filters.ModelMultipleChoiceFilter(
         queryset=Secret.objects.all(),
@@ -928,7 +893,8 @@ class SecretsGroupAssociationFilterSet(BaseFilterSet):
     )
     secret = NaturalKeyOrPKMultipleChoiceFilter(
         queryset=Secret.objects.all(),
-        label="Secret (ID or slug)",
+        label="Secret (ID or name)",
+        to_field_name="name",
     )
     access_type = django_filters.MultipleChoiceFilter(choices=SecretsGroupAccessTypeChoices)
     secret_type = django_filters.MultipleChoiceFilter(choices=SecretsGroupSecretTypeChoices)
@@ -949,7 +915,6 @@ class StatusFilterSet(NautobotFilterSet):
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "slug": "icontains",
             "content_types__model": "icontains",
         },
     )
@@ -964,7 +929,6 @@ class StatusFilterSet(NautobotFilterSet):
             "content_types",
             "color",
             "name",
-            "slug",
             "created",
             "last_updated",
         ]
@@ -979,7 +943,6 @@ class TagFilterSet(NautobotFilterSet):
     q = SearchFilter(
         filter_predicates={
             "name": "icontains",
-            "slug": "icontains",
             "content_types__model": "icontains",
         },
     )
@@ -989,7 +952,7 @@ class TagFilterSet(NautobotFilterSet):
 
     class Meta:
         model = Tag
-        fields = ["id", "name", "slug", "color", "content_types"]
+        fields = ["id", "name", "color", "content_types"]
 
 
 #
@@ -1021,3 +984,35 @@ class WebhookFilterSet(BaseFilterSet):
             "type_update",
             "type_delete",
         ]
+
+
+class RoleFilterSet(NautobotFilterSet):
+    """API filter for filtering custom role object fields."""
+
+    q = SearchFilter(
+        filter_predicates={
+            "name": "icontains",
+            "content_types__model": "icontains",
+        },
+    )
+    # TODO(timizuo): Add a feature to set conjoined to either True/False from query param in url;
+    #  this way only ConfigContext related query would set conjoined to True
+    content_types = ContentTypeMultipleChoiceFilter(
+        choices=RoleModelsQuery().get_choices,
+        # Set the 'conjoined' parameter to False to allow `ConfigContext`
+        # to filter the queryset by a combinations of content types,
+        # such as 'Device or VirtualMachine' but not 'Device and VirtualMachine'.
+        conjoined=False,
+    )
+
+    class Meta:
+        model = Role
+        fields = [
+            "id",
+            "content_types",
+            "color",
+            "name",
+            "weight",
+            "created",
+            "last_updated",
+        ]
