@@ -6,9 +6,9 @@ from nautobot.extras.models import GitRepository, JobResult, ObjectChange
 def get_job_results(request):
     """Callback function to collect job history for panel."""
     return (
-        JobResult.objects.filter(status__in=JobResultStatusChoices.TERMINAL_STATE_CHOICES)
-        .defer("data")
-        .order_by("-completed")[:10]
+        JobResult.objects.filter(status__in=JobResultStatusChoices.READY_STATES)
+        .defer("result")
+        .order_by("-date_done")[:10]
     )
 
 
