@@ -1,17 +1,15 @@
 from django.contrib.contenttypes.models import ContentType
 from django.db import models
-from django.urls import reverse
-from django.utils.text import slugify
-from taggit.models import TagBase, GenericUUIDTaggedItemBase
+from taggit.models import GenericUUIDTaggedItemBase
 
+from nautobot.core.choices import ColorChoices
+from nautobot.core.models import BaseManager, BaseModel
+from nautobot.core.models.fields import ColorField
+from nautobot.core.models.querysets import RestrictedQuerySet
 from nautobot.extras.models import ChangeLoggedModel, CustomFieldModel
 from nautobot.extras.models.mixins import NotesMixin
 from nautobot.extras.models.relationships import RelationshipModel
 from nautobot.extras.utils import extras_features, TaggableClassesQuery
-from nautobot.core.models import BaseModel
-from nautobot.utilities.choices import ColorChoices
-from nautobot.utilities.fields import ColorField
-from nautobot.utilities.querysets import RestrictedQuerySet
 
 
 #
@@ -26,16 +24,15 @@ class TagQuerySet(RestrictedQuerySet):
         """
         Return all `Tags` assigned to the given model.
         """
-        content_type = ContentType.objects.get_for_model(model._meta.concrete_model)
-        return self.filter(content_types=content_type)
+        return self.filter(content_types__model=model._meta.model_name, content_types__app_label=model._meta.app_label)
 
 
+# Tag *should* be a `NameColorContentTypesModel` but that way lies circular import purgatory. Sigh.
 @extras_features(
-    "custom_fields",
     "custom_validators",
-    "relationships",
 )
-class Tag(TagBase, BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel, NotesMixin):
+class Tag(BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipModel, NotesMixin):
+    name = models.CharField(max_length=100, unique=True)
     content_types = models.ManyToManyField(
         to=ContentType,
         related_name="tags",
@@ -47,26 +44,14 @@ class Tag(TagBase, BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipM
         blank=True,
     )
 
-    csv_headers = ["name", "slug", "color", "description"]
+    objects = BaseManager.from_queryset(TagQuerySet)()
 
-    objects = TagQuerySet.as_manager()
+    def __str__(self):
+        return self.name
 
     class Meta:
         ordering = ["name"]
 
-    def get_absolute_url(self):
-        return reverse("extras:tag", args=[self.slug])
-
-    def slugify(self, tag, i=None):
-        # Allow Unicode in Tag slugs (avoids empty slugs for Tags with all-Unicode names)
-        slug = slugify(tag, allow_unicode=True)
-        if i is not None:
-            slug += "_%d" % i
-        return slug
-
-    def to_csv(self):
-        return (self.name, self.slug, self.color, self.description)
-
     def validate_content_types_removal(self, content_types_id):
         """Validate content_types to be removed are not tagged to a model"""
         errors = {}
@@ -86,7 +71,11 @@ class Tag(TagBase, BaseModel, ChangeLoggedModel, CustomFieldModel, RelationshipM
 
 class TaggedItem(BaseModel, GenericUUIDTaggedItemBase):
     tag = models.ForeignKey(to=Tag, related_name="%(app_label)s_%(class)s_items", on_delete=models.CASCADE)
-    object_id = models.UUIDField()
+
+    documentation_static_path = "docs/user-guide/platform-functionality/tag.html"
+
+    natural_key_field_names = ["pk"]
 
     class Meta:
         index_together = ("content_type", "object_id")
+        unique_together = [["content_type", "object_id", "tag"]]
