@@ -1,4 +1,13 @@
-from nautobot.core.apps import NavMenuAddButton, NavMenuGroup, NavMenuItem, NavMenuImportButton, NavMenuTab
+from nautobot.core.apps import (
+    NavContext,
+    NavGrouping,
+    NavItem,
+    NavMenuAddButton,
+    NavMenuGroup,
+    NavMenuItem,
+    NavMenuImportButton,
+    NavMenuTab,
+)
 
 
 menu_items = (
@@ -62,6 +71,34 @@ menu_items = (
                     ),
                 ),
             ),
+            NavMenuGroup(
+                name="Roles",
+                weight=500,
+                items=(
+                    NavMenuItem(
+                        link="extras:role_list",
+                        name="Roles",
+                        weight=100,
+                        permissions=[
+                            "extras.view_status",
+                        ],
+                        buttons=(
+                            NavMenuAddButton(
+                                link="extras:role_add",
+                                permissions=[
+                                    "extras.add_role",
+                                ],
+                            ),
+                            NavMenuImportButton(
+                                link="extras:role_import",
+                                permissions=[
+                                    "extras.add_role",
+                                ],
+                            ),
+                        ),
+                    ),
+                ),
+            ),
             NavMenuGroup(
                 name="Dynamic Groups",
                 weight=500,
@@ -281,6 +318,14 @@ menu_items = (
                             ),
                         ),
                     ),
+                    NavMenuItem(
+                        link="extras:note_list",
+                        name="Notes",
+                        weight=300,
+                        permissions=[
+                            "extras.view_note",
+                        ],
+                    ),
                 ),
             ),
             NavMenuGroup(
@@ -429,3 +474,252 @@ menu_items = (
         ),
     ),
 )
+
+
+navigation = (
+    NavContext(
+        name="Inventory",
+        groups=(
+            NavGrouping(
+                name="Devices",
+                weight=100,
+                items=(
+                    NavItem(
+                        name="Dynamic Groups",
+                        weight=900,
+                        link="extras:dynamicgroup_list",
+                        permissions=["extras.view_dynamicgroup"],
+                    ),
+                ),
+            ),
+        ),
+    ),
+    NavContext(
+        name="Networks",
+        groups=(
+            NavGrouping(
+                name="Config Contexts",
+                weight=500,
+                items=(
+                    NavItem(
+                        name="Config Contexts",
+                        weight=100,
+                        link="extras:configcontext_list",
+                        permissions=["extras.view_configcontext"],
+                    ),
+                    NavItem(
+                        name="Config Context Schemas",
+                        weight=100,
+                        link="extras:configcontextschema_list",
+                        permissions=["extras.view_configcontextschema"],
+                    ),
+                ),
+            ),
+        ),
+    ),
+    NavContext(
+        name="Security",
+        groups=(
+            NavGrouping(
+                name="Secrets",
+                weight=500,
+                items=(
+                    NavItem(
+                        name="Secrets",
+                        weight=100,
+                        link="extras:secret_list",
+                        permissions=["extras.view_secret"],
+                    ),
+                    NavItem(
+                        name="Secret Groups",
+                        weight=200,
+                        link="extras:secretsgroup_list",
+                        permissions=["extras.view_secretsgroup"],
+                    ),
+                ),
+            ),
+        ),
+    ),
+    NavContext(
+        name="Automation",
+        groups=(
+            NavGrouping(
+                name="Jobs",
+                weight=100,
+                items=(
+                    NavItem(
+                        link="extras:job_list",
+                        name="Jobs",
+                        weight=100,
+                        permissions=[
+                            "extras.view_job",
+                        ],
+                    ),
+                    NavItem(
+                        link="extras:scheduledjob_approval_queue_list",
+                        name="Job Approval Queue",
+                        weight=200,
+                        permissions=[
+                            "extras.view_job",
+                        ],
+                    ),
+                    NavItem(
+                        link="extras:scheduledjob_list",
+                        name="Scheduled Jobs",
+                        weight=300,
+                        permissions=[
+                            "extras.view_job",
+                            "extras.view_scheduledjob",
+                        ],
+                    ),
+                    NavItem(
+                        link="extras:jobresult_list",
+                        name="Job Results",
+                        weight=400,
+                        permissions=[
+                            "extras.view_jobresult",
+                        ],
+                    ),
+                    NavItem(
+                        link="extras:jobhook_list",
+                        name="Job Hooks",
+                        weight=500,
+                        permissions=[
+                            "extras.view_jobhook",
+                        ],
+                    ),
+                    NavItem(
+                        link="extras:jobbutton_list",
+                        name="Job Buttons",
+                        weight=600,
+                        permissions=[
+                            "extras.view_jobbutton",
+                        ],
+                    ),
+                ),
+            ),
+            NavGrouping(
+                name="Extensibility",
+                weight=9999,  # always last
+                items=(
+                    NavItem(
+                        link="extras:webhook_list",
+                        name="Webhooks",
+                        weight=100,
+                        permissions=["extras.view_webhook"],
+                    ),
+                    NavItem(
+                        name="GraphQL Queries",
+                        weight=200,
+                        link="extras:graphqlquery_list",
+                        permissions=["extras.view_graphqlquery"],
+                    ),
+                    NavItem(
+                        name="Export Templates",
+                        weight=300,
+                        link="extras:exporttemplate_list",
+                        permissions=["extras.view_exporttemplate"],
+                    ),
+                ),
+            ),
+        ),
+    ),
+    NavContext(
+        name="Platform",
+        groups=(
+            NavGrouping(
+                name="Platform",
+                weight=100,
+                items=(
+                    NavItem(
+                        name="Installed Apps",
+                        weight=100,
+                        link="plugins:plugins_list",
+                        permissions=["is_staff"],
+                    ),
+                    NavItem(
+                        name="Git Repositories",
+                        weight=200,
+                        link="extras:gitrepository_list",
+                        permissions=["extras.view_gitrepository"],
+                    ),
+                ),
+            ),
+            NavGrouping(
+                name="Governance",
+                weight=200,
+                items=(
+                    NavItem(
+                        name="Change Log",
+                        weight=100,
+                        link="extras:objectchange_list",
+                        permissions=["extras.view_objectchange"],
+                    ),
+                ),
+            ),
+            NavGrouping(
+                name="Reference Data",
+                weight=300,
+                items=(
+                    NavItem(
+                        name="Tags",
+                        weight=100,
+                        link="extras:tag_list",
+                        permissions=["extras.view_tag"],
+                    ),
+                    NavItem(
+                        name="Statuses",
+                        weight=200,
+                        link="extras:status_list",
+                        permissions=["extras.view_status"],
+                    ),
+                    NavItem(
+                        name="Roles",
+                        weight=300,
+                        link="extras:role_list",
+                        permissions=["extras.view_role"],
+                    ),
+                ),
+            ),
+            NavGrouping(
+                name="Data Management",
+                weight=400,
+                items=(
+                    NavItem(
+                        name="Relationships",
+                        weight=100,
+                        link="extras:relationship_list",
+                        permissions=["extras.view_relationship"],
+                    ),
+                    NavItem(
+                        name="Computed Fields",
+                        weight=200,
+                        link="extras:computedfield_list",
+                        permissions=["extras.view_computedfield"],
+                    ),
+                    NavItem(
+                        name="Custom Fields",
+                        weight=300,
+                        link="extras:customfield_list",
+                        permissions=["extras.view_customfield"],
+                    ),
+                    NavItem(
+                        name="Custom Links",
+                        weight=400,
+                        link="extras:customlink_list",
+                        permissions=["extras.view_customlink"],
+                    ),
+                    NavItem(
+                        name="Notes",
+                        weight=500,
+                        # TODO figure out what link we should put here
+                        # This NavItem is required for the breadCrumb on NotesDetailView to render.
+                        # However `extras:note_list` is not a reverse url lookup that does not exist.
+                        link="extras:note_list",
+                        permissions=["extras.view_note"],
+                    ),
+                ),
+            ),
+        ),
+    ),
+)
