@@ -36,44 +36,41 @@ def update_custom_field_choice_data(field_id, old_value, new_value):
         # Loop through all field content types and search for values to update
         for ct in field.content_types.all():
             model = ct.model_class()
-            # 2.0 TODO: #824 field.slug rather than field.name
-            for obj in model.objects.filter(**{f"_custom_field_data__{field.name}": old_value}):
-                obj._custom_field_data[field.name] = new_value
+            for obj in model.objects.filter(**{f"_custom_field_data__{field.key}": old_value}):
+                obj._custom_field_data[field.key] = new_value
                 obj.save()
 
     elif field.type == CustomFieldTypeChoices.TYPE_MULTISELECT:
         # Loop through all field content types and search for values to update
         for ct in field.content_types.all():
             model = ct.model_class()
-            # 2.0 TODO: #824 field.slug rather than field.name
-            for obj in model.objects.filter(**{f"_custom_field_data__{field.name}__contains": old_value}):
-                old_list = obj._custom_field_data[field.name]
+            for obj in model.objects.filter(**{f"_custom_field_data__{field.key}__contains": old_value}):
+                old_list = obj._custom_field_data[field.key]
                 new_list = [new_value if e == old_value else e for e in old_list]
-                obj._custom_field_data[field.name] = new_list
+                obj._custom_field_data[field.key] = new_list
                 obj.save()
 
     else:
-        logger.error(f"Unknown field type, failing to act on choice data for this field {field.name}.")
+        logger.error(f"Unknown field type, failing to act on choice data for this field {field.key}.")
         return False
 
     return True
 
 
-# 2.0 TODO: #824 rename field_name to field_slug
 @nautobot_task
-def delete_custom_field_data(field_name, content_type_pk_set):
+def delete_custom_field_data(field_key, content_type_pk_set):
     """
     Delete the values for a custom field
 
     Args:
-        field_name (str): The name of the custom field which is being deleted
+        field_key (str): The key of the custom field which is being deleted
         content_type_pk_set (list): List of PKs for content types to act upon
     """
     with transaction.atomic():
         for ct in ContentType.objects.filter(pk__in=content_type_pk_set):
             model = ct.model_class()
-            for obj in model.objects.filter(**{f"_custom_field_data__{field_name}__isnull": False}):
-                del obj._custom_field_data[field_name]
+            for obj in model.objects.filter(**{f"_custom_field_data__{field_key}__isnull": False}):
+                del obj._custom_field_data[field_key]
                 obj.save()
 
 
@@ -98,8 +95,7 @@ def provision_field(field_id, content_type_pk_set):
         for ct in ContentType.objects.filter(pk__in=content_type_pk_set):
             model = ct.model_class()
             for obj in model.objects.all():
-                # 2.0 TODO: #824 field.slug rather than field.name
-                obj._custom_field_data.setdefault(field.name, field.default)
+                obj._custom_field_data.setdefault(field.key, field.default)
                 obj.save()
 
     return True
