@@ -1,7 +1,8 @@
 from django.urls import reverse
 
-from nautobot.dcim.models import Site
-from nautobot.utilities.testing.integration import SeleniumTestCase
+from nautobot.core.testing.integration import SeleniumTestCase
+from nautobot.dcim.models import Location, LocationType
+from nautobot.extras.models import Status
 
 
 class NoteTestCase(SeleniumTestCase):
@@ -23,14 +24,16 @@ class NoteTestCase(SeleniumTestCase):
         """
         Test initial add and then update of a new Note
         """
-        Site.objects.create(name="Site 1", slug="site-1")
+        location_type, _ = LocationType.objects.get_or_create(name="Campus")
+        location_status = Status.objects.get_for_model(Location).first()
+        location = Location.objects.create(name="Location 1", location_type=location_type, status=location_status)
 
-        # Navigate to the created site.
-        self.browser.visit(f'{self.live_server_url}{reverse("dcim:site", kwargs={"slug": "site-1"})}')
+        # Navigate to the created location.
+        self.browser.visit(f'{self.live_server_url}{reverse("dcim:location", kwargs={"pk": location.pk})}')
 
         # Verify notes tab shows up and click it.
-        self.assertTrue(self.browser.links.find_by_partial_href("/dcim/sites/site-1/notes/"))
-        self.browser.links.find_by_partial_href("/dcim/sites/site-1/notes/").click()
+        self.assertTrue(self.browser.links.find_by_partial_href(f"/dcim/locations/{location.pk}/notes/"))
+        self.browser.links.find_by_partial_href(f"/dcim/locations/{location.pk}/notes/").click()
 
         # Fill out the form.
         self.browser.fill("note", "This is a maintenance notice.")
