@@ -1,5 +1,4 @@
 from django.urls import path
-from django.views.generic import RedirectView
 
 from nautobot.core.views.routers import NautobotUIViewSetRouter
 from nautobot.extras import views
@@ -24,14 +23,16 @@ from nautobot.extras.models import (
     Webhook,
 )
 
+app_name = "extras"
+
 router = NautobotUIViewSetRouter()
 router.register("job-buttons", views.JobButtonUIViewSet)
+router.register("roles", views.RoleUIViewSet)
 
-app_name = "extras"
 urlpatterns = [
     # Change logging
-    path("changelog/", views.ObjectChangeListView.as_view(), name="objectchange_list"),
-    path("changelog/<uuid:pk>/", views.ObjectChangeView.as_view(), name="objectchange"),
+    path("object-changes/", views.ObjectChangeListView.as_view(), name="objectchange_list"),
+    path("object-changes/<uuid:pk>/", views.ObjectChangeView.as_view(), name="objectchange"),
     # Computed Fields
     path("computed-fields/", views.ComputedFieldListView.as_view(), name="computedfield_list"),
     path("computed-fields/add/", views.ComputedFieldEditView.as_view(), name="computedfield_add"),
@@ -40,25 +41,25 @@ urlpatterns = [
         views.ComputedFieldBulkDeleteView.as_view(),
         name="computedfield_bulk_delete",
     ),
-    path("computed-fields/<slug:slug>/", views.ComputedFieldView.as_view(), name="computedfield"),
+    path("computed-fields/<uuid:pk>/", views.ComputedFieldView.as_view(), name="computedfield"),
     path(
-        "computed-fields/<slug:slug>/edit/",
+        "computed-fields/<uuid:pk>/edit/",
         views.ComputedFieldEditView.as_view(),
         name="computedfield_edit",
     ),
     path(
-        "computed-fields/<slug:slug>/delete/",
+        "computed-fields/<uuid:pk>/delete/",
         views.ComputedFieldDeleteView.as_view(),
         name="computedfield_delete",
     ),
     path(
-        "computed-fields/<slug:slug>/changelog/",
+        "computed-fields/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="computedfield_changelog",
         kwargs={"model": ComputedField},
     ),
     path(
-        "computed-fields/<slug:slug>/notes/",
+        "computed-fields/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="computedfield_notes",
         kwargs={"model": ComputedField},
@@ -133,33 +134,33 @@ urlpatterns = [
         name="configcontextschema_bulk_delete",
     ),
     path(
-        "config-context-schemas/<slug:slug>/",
+        "config-context-schemas/<uuid:pk>/",
         views.ConfigContextSchemaView.as_view(),
         name="configcontextschema",
     ),
     path(
-        "config-context-schemas/<slug:slug>/validation/",
+        "config-context-schemas/<uuid:pk>/validation/",
         views.ConfigContextSchemaObjectValidationView.as_view(),
         name="configcontextschema_object_validation",
     ),
     path(
-        "config-context-schemas/<slug:slug>/edit/",
+        "config-context-schemas/<uuid:pk>/edit/",
         views.ConfigContextSchemaEditView.as_view(),
         name="configcontextschema_edit",
     ),
     path(
-        "config-context-schemas/<slug:slug>/delete/",
+        "config-context-schemas/<uuid:pk>/delete/",
         views.ConfigContextSchemaDeleteView.as_view(),
         name="configcontextschema_delete",
     ),
     path(
-        "config-context-schemas/<slug:slug>/changelog/",
+        "config-context-schemas/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="configcontextschema_changelog",
         kwargs={"model": ConfigContextSchema},
     ),
     path(
-        "config-context-schemas/<slug:slug>/notes/",
+        "config-context-schemas/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="configcontextschema_notes",
         kwargs={"model": ConfigContextSchema},
@@ -172,25 +173,25 @@ urlpatterns = [
         views.CustomFieldBulkDeleteView.as_view(),
         name="customfield_bulk_delete",
     ),
-    path("custom-fields/<slug:slug>/", views.CustomFieldView.as_view(), name="customfield"),
+    path("custom-fields/<uuid:pk>/", views.CustomFieldView.as_view(), name="customfield"),
     path(
-        "custom-fields/<slug:slug>/edit/",
+        "custom-fields/<uuid:pk>/edit/",
         views.CustomFieldEditView.as_view(),
         name="customfield_edit",
     ),
     path(
-        "custom-fields/<slug:slug>/delete/",
+        "custom-fields/<uuid:pk>/delete/",
         views.CustomFieldDeleteView.as_view(),
         name="customfield_delete",
     ),
     path(
-        "custom-fields/<slug:slug>/changelog/",
+        "custom-fields/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="customfield_changelog",
         kwargs={"model": CustomField},
     ),
     path(
-        "custom-fields/<str:name>/notes/",
+        "custom-fields/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="customfield_notes",
         kwargs={"model": CustomField},
@@ -234,17 +235,17 @@ urlpatterns = [
         views.DynamicGroupBulkDeleteView.as_view(),
         name="dynamicgroup_bulk_delete",
     ),
-    path("dynamic-groups/<str:slug>/", views.DynamicGroupView.as_view(), name="dynamicgroup"),
-    path("dynamic-groups/<str:slug>/edit/", views.DynamicGroupEditView.as_view(), name="dynamicgroup_edit"),
-    path("dynamic-groups/<str:slug>/delete/", views.DynamicGroupDeleteView.as_view(), name="dynamicgroup_delete"),
+    path("dynamic-groups/<uuid:pk>/", views.DynamicGroupView.as_view(), name="dynamicgroup"),
+    path("dynamic-groups/<uuid:pk>/edit/", views.DynamicGroupEditView.as_view(), name="dynamicgroup_edit"),
+    path("dynamic-groups/<uuid:pk>/delete/", views.DynamicGroupDeleteView.as_view(), name="dynamicgroup_delete"),
     path(
-        "dynamic-groups/<str:slug>/changelog/",
+        "dynamic-groups/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="dynamicgroup_changelog",
         kwargs={"model": DynamicGroup},
     ),
     path(
-        "dynamic-groups/<str:slug>/notes/",
+        "dynamic-groups/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="dynamicgroup_notes",
         kwargs={"model": DynamicGroup},
@@ -319,44 +320,44 @@ urlpatterns = [
         name="gitrepository_import",
     ),
     path(
-        "git-repositories/<str:slug>/",
+        "git-repositories/<uuid:pk>/",
         views.GitRepositoryView.as_view(),
         name="gitrepository",
     ),
     path(
-        "git-repositories/<str:slug>/edit/",
+        "git-repositories/<uuid:pk>/edit/",
         views.GitRepositoryEditView.as_view(),
         name="gitrepository_edit",
     ),
     path(
-        "git-repositories/<str:slug>/delete/",
+        "git-repositories/<uuid:pk>/delete/",
         views.GitRepositoryDeleteView.as_view(),
         name="gitrepository_delete",
     ),
     path(
-        "git-repositories/<str:slug>/changelog/",
+        "git-repositories/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="gitrepository_changelog",
         kwargs={"model": GitRepository},
     ),
     path(
-        "git-repositories/<str:slug>/notes/",
+        "git-repositories/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="gitrepository_notes",
         kwargs={"model": GitRepository},
     ),
     path(
-        "git-repositories/<str:slug>/result/",
+        "git-repositories/<uuid:pk>/result/",
         views.GitRepositoryResultView.as_view(),
         name="gitrepository_result",
     ),
     path(
-        "git-repositories/<str:slug>/sync/",
+        "git-repositories/<uuid:pk>/sync/",
         views.GitRepositorySyncView.as_view(),
         name="gitrepository_sync",
     ),
     path(
-        "git-repositories/<str:slug>/dry-run/",
+        "git-repositories/<uuid:pk>/dry-run/",
         views.GitRepositoryDryRunView.as_view(),
         name="gitrepository_dryrun",
     ),
@@ -368,14 +369,14 @@ urlpatterns = [
         views.GraphQLQueryBulkDeleteView.as_view(),
         name="GraphQLQuery_bulk_delete",
     ),
-    path("graphql-queries/<str:slug>/", views.GraphQLQueryView.as_view(), name="graphqlquery"),
+    path("graphql-queries/<uuid:pk>/", views.GraphQLQueryView.as_view(), name="graphqlquery"),
     path(
-        "graphql-queries/<str:slug>/edit/",
+        "graphql-queries/<uuid:pk>/edit/",
         views.GraphQLQueryEditView.as_view(),
         name="graphqlquery_edit",
     ),
     path(
-        "graphql-queries/<str:slug>/delete/",
+        "graphql-queries/<uuid:pk>/delete/",
         views.GraphQLQueryDeleteView.as_view(),
         name="graphqlquery_delete",
     ),
@@ -404,11 +405,6 @@ urlpatterns = [
     ),
     # Jobs
     path("jobs/", views.JobListView.as_view(), name="job_list"),
-    path(
-        "jobs/results/<uuid:pk>/",
-        RedirectView.as_view(pattern_name="extras:jobresult"),
-        name="job_jobresult",
-    ),  # 2.0 TODO(jathan): Remove this, no existing code references `job_jobresult` but plugins and others may.
     path("jobs/scheduled-jobs/", views.ScheduledJobListView.as_view(), name="scheduledjob_list"),
     path("jobs/scheduled-jobs/<uuid:pk>/", views.ScheduledJobView.as_view(), name="scheduledjob"),
     path("jobs/scheduled-jobs/<uuid:pk>/delete/", views.ScheduledJobDeleteView.as_view(), name="scheduledjob_delete"),
@@ -427,29 +423,27 @@ urlpatterns = [
         views.JobApprovalRequestView.as_view(),
         name="scheduledjob_approval_request_view",
     ),
-    # 2.0 TODO: JobDetailView should be just JobView, but see below
     path(
-        "jobs/<slug:slug>/",
-        views.JobDetailView.as_view(),
-        name="job_detail",  # 2.0 TODO: name="job",
+        "jobs/<uuid:pk>/",
+        views.JobView.as_view(),
+        name="job",
     ),
-    path("jobs/<slug:slug>/edit/", views.JobEditView.as_view(), name="job_edit"),
-    path("jobs/<slug:slug>/delete/", views.JobDeleteView.as_view(), name="job_delete"),
+    path("jobs/<uuid:pk>/edit/", views.JobEditView.as_view(), name="job_edit"),
+    path("jobs/<uuid:pk>/delete/", views.JobDeleteView.as_view(), name="job_delete"),
     path(
-        "jobs/<slug:slug>/changelog/",
+        "jobs/<uuid:pk>/changelog/",
         views.JobObjectChangeLogView.as_view(),
         name="job_changelog",
         kwargs={"model": Job},
     ),
     path(
-        "jobs/<slug:slug>/notes/",
+        "jobs/<uuid:pk>/notes/",
         views.JobObjectNotesView.as_view(),
         name="job_notes",
         kwargs={"model": Job},
     ),
-    # 2.0 TODO: JobView should actually be JobRunView, but keeping it as-is for backwards compatibility
-    path("jobs/<slug:slug>/run/", views.JobView.as_view(), name="job_run"),
-    path("jobs/<path:class_path>/", views.JobView.as_view(), name="job"),
+    path("jobs/<uuid:pk>/run/", views.JobRunView.as_view(), name="job_run"),
+    path("jobs/<str:class_path>/run/", views.JobRunView.as_view(), name="job_run_by_class_path"),
     # Job hooks
     path("job-hooks/", views.JobHookListView.as_view(), name="jobhook_list"),
     path("job-hooks/add/", views.JobHookEditView.as_view(), name="jobhook_add"),
@@ -458,15 +452,15 @@ urlpatterns = [
         views.JobHookBulkDeleteView.as_view(),
         name="jobhook_bulk_delete",
     ),
-    path("job-hooks/<str:slug>/", views.JobHookView.as_view(), name="jobhook"),
-    path("job-hooks/<str:slug>/edit/", views.JobHookEditView.as_view(), name="jobhook_edit"),
+    path("job-hooks/<uuid:pk>/", views.JobHookView.as_view(), name="jobhook"),
+    path("job-hooks/<uuid:pk>/edit/", views.JobHookEditView.as_view(), name="jobhook_edit"),
     path(
-        "job-hooks/<str:slug>/delete/",
+        "job-hooks/<uuid:pk>/delete/",
         views.JobHookDeleteView.as_view(),
         name="jobhook_delete",
     ),
     path(
-        "job-hooks/<str:slug>/changelog/",
+        "job-hooks/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="jobhook_changelog",
         kwargs={"model": JobHook},
@@ -488,16 +482,17 @@ urlpatterns = [
     # Job Button Run
     path("job-button/<uuid:pk>/run/", views.JobButtonRunView.as_view(), name="jobbutton_run"),
     # Notes
+    path("notes/", views.NoteListView.as_view(), name="note_list"),
     path("notes/add/", views.NoteEditView.as_view(), name="note_add"),
-    path("notes/<slug:slug>/", views.NoteView.as_view(), name="note"),
+    path("notes/<uuid:pk>/", views.NoteView.as_view(), name="note"),
     path(
-        "notes/<slug:slug>/changelog/",
+        "notes/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="note_changelog",
         kwargs={"model": Note},
     ),
-    path("notes/<slug:slug>/edit/", views.NoteEditView.as_view(), name="note_edit"),
-    path("notes/<slug:slug>/delete/", views.NoteDeleteView.as_view(), name="note_delete"),
+    path("notes/<uuid:pk>/edit/", views.NoteEditView.as_view(), name="note_edit"),
+    path("notes/<uuid:pk>/delete/", views.NoteDeleteView.as_view(), name="note_delete"),
     # Custom relationships
     path("relationships/", views.RelationshipListView.as_view(), name="relationship_list"),
     path(
@@ -511,28 +506,28 @@ urlpatterns = [
         name="relationship_bulk_delete",
     ),
     path(
-        "relationships/<slug:slug>/",
+        "relationships/<uuid:pk>/",
         views.RelationshipView.as_view(),
         name="relationship",
     ),
     path(
-        "relationships/<slug:slug>/edit/",
+        "relationships/<uuid:pk>/edit/",
         views.RelationshipEditView.as_view(),
         name="relationship_edit",
     ),
     path(
-        "relationships/<slug:slug>/delete/",
+        "relationships/<uuid:pk>/delete/",
         views.RelationshipDeleteView.as_view(),
         name="relationship_delete",
     ),
     path(
-        "relationships/<slug:slug>/changelog/",
+        "relationships/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="relationship_changelog",
         kwargs={"model": Relationship},
     ),
     path(
-        "relationships/<slug:slug>/notes/",
+        "relationships/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="relationship_notes",
         kwargs={"model": Relationship},
@@ -562,17 +557,17 @@ urlpatterns = [
         views.SecretProviderParametersFormView.as_view(),
         name="secret_provider_parameters_form",
     ),
-    path("secrets/<str:slug>/", views.SecretView.as_view(), name="secret"),
-    path("secrets/<str:slug>/edit/", views.SecretEditView.as_view(), name="secret_edit"),
-    path("secrets/<str:slug>/delete/", views.SecretDeleteView.as_view(), name="secret_delete"),
+    path("secrets/<uuid:pk>/", views.SecretView.as_view(), name="secret"),
+    path("secrets/<uuid:pk>/edit/", views.SecretEditView.as_view(), name="secret_edit"),
+    path("secrets/<uuid:pk>/delete/", views.SecretDeleteView.as_view(), name="secret_delete"),
     path(
-        "secrets/<str:slug>/changelog/",
+        "secrets/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="secret_changelog",
         kwargs={"model": Secret},
     ),
     path(
-        "secrets/<str:slug>/notes/",
+        "secrets/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="secret_notes",
         kwargs={"model": Secret},
@@ -580,17 +575,17 @@ urlpatterns = [
     path("secrets-groups/", views.SecretsGroupListView.as_view(), name="secretsgroup_list"),
     path("secrets-groups/add/", views.SecretsGroupEditView.as_view(), name="secretsgroup_add"),
     path("secrets-groups/delete/", views.SecretsGroupBulkDeleteView.as_view(), name="secretsgroup_bulk_delete"),
-    path("secrets-groups/<str:slug>/", views.SecretsGroupView.as_view(), name="secretsgroup"),
-    path("secrets-groups/<str:slug>/edit/", views.SecretsGroupEditView.as_view(), name="secretsgroup_edit"),
-    path("secrets-groups/<str:slug>/delete/", views.SecretsGroupDeleteView.as_view(), name="secretsgroup_delete"),
+    path("secrets-groups/<uuid:pk>/", views.SecretsGroupView.as_view(), name="secretsgroup"),
+    path("secrets-groups/<uuid:pk>/edit/", views.SecretsGroupEditView.as_view(), name="secretsgroup_edit"),
+    path("secrets-groups/<uuid:pk>/delete/", views.SecretsGroupDeleteView.as_view(), name="secretsgroup_delete"),
     path(
-        "secrets-groups/<str:slug>/changelog/",
+        "secrets-groups/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="secretsgroup_changelog",
         kwargs={"model": SecretsGroup},
     ),
     path(
-        "secrets-groups/<str:slug>/notes/",
+        "secrets-groups/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="secretsgroup_notes",
         kwargs={"model": SecretsGroup},
@@ -605,21 +600,21 @@ urlpatterns = [
         name="status_bulk_delete",
     ),
     path("statuses/import/", views.StatusBulkImportView.as_view(), name="status_import"),
-    path("statuses/<str:slug>/", views.StatusView.as_view(), name="status"),
-    path("statuses/<str:slug>/edit/", views.StatusEditView.as_view(), name="status_edit"),
+    path("statuses/<uuid:pk>/", views.StatusView.as_view(), name="status"),
+    path("statuses/<uuid:pk>/edit/", views.StatusEditView.as_view(), name="status_edit"),
     path(
-        "statuses/<str:slug>/delete/",
+        "statuses/<uuid:pk>/delete/",
         views.StatusDeleteView.as_view(),
         name="status_delete",
     ),
     path(
-        "statuses/<str:slug>/changelog/",
+        "statuses/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="status_changelog",
         kwargs={"model": Status},
     ),
     path(
-        "statuses/<str:slug>/notes/",
+        "statuses/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="status_notes",
         kwargs={"model": Status},
@@ -630,17 +625,17 @@ urlpatterns = [
     path("tags/import/", views.TagBulkImportView.as_view(), name="tag_import"),
     path("tags/edit/", views.TagBulkEditView.as_view(), name="tag_bulk_edit"),
     path("tags/delete/", views.TagBulkDeleteView.as_view(), name="tag_bulk_delete"),
-    path("tags/<str:slug>/", views.TagView.as_view(), name="tag"),
-    path("tags/<str:slug>/edit/", views.TagEditView.as_view(), name="tag_edit"),
-    path("tags/<str:slug>/delete/", views.TagDeleteView.as_view(), name="tag_delete"),
+    path("tags/<uuid:pk>/", views.TagView.as_view(), name="tag"),
+    path("tags/<uuid:pk>/edit/", views.TagEditView.as_view(), name="tag_edit"),
+    path("tags/<uuid:pk>/delete/", views.TagDeleteView.as_view(), name="tag_delete"),
     path(
-        "tags/<str:slug>/changelog/",
+        "tags/<uuid:pk>/changelog/",
         views.ObjectChangeLogView.as_view(),
         name="tag_changelog",
         kwargs={"model": Tag},
     ),
     path(
-        "tags/<str:slug>/notes/",
+        "tags/<uuid:pk>/notes/",
         views.ObjectNotesView.as_view(),
         name="tag_notes",
         kwargs={"model": Tag},
@@ -673,4 +668,5 @@ urlpatterns = [
         kwargs={"model": Webhook},
     ),
 ]
+
 urlpatterns += router.urls
