@@ -2,34 +2,38 @@ import django_tables2 as tables
 from django.utils.safestring import mark_safe
 from django_tables2.utils import Accessor
 
-from nautobot.dcim.models import Interface
-from nautobot.extras.tables import StatusTableMixin
-from nautobot.tenancy.tables import TenantColumn
-from nautobot.utilities.tables import (
+from nautobot.core.tables import (
     BaseTable,
     BooleanColumn,
     ButtonsColumn,
-    ChoiceFieldColumn,
     LinkedCountColumn,
     TagColumn,
     ToggleColumn,
 )
-from nautobot.utilities.templatetags.helpers import render_boolean
+from nautobot.core.templatetags.helpers import render_boolean
+from nautobot.dcim.models import Interface
+from nautobot.dcim.tables import InterfaceTable
+from nautobot.dcim.tables.devices import DeviceComponentTable
+from nautobot.dcim.utils import cable_status_color_css
+from nautobot.extras.tables import RoleTableMixin, StatusTableMixin
+from nautobot.tenancy.tables import TenantColumn
 from nautobot.virtualization.models import VMInterface
 from .models import (
-    Aggregate,
     IPAddress,
+    IPAddressToInterface,
+    Namespace,
     Prefix,
     RIR,
-    Role,
     RouteTarget,
     Service,
     VLAN,
     VLANGroup,
     VRF,
+    VRFDeviceAssignment,
+    VRFPrefixAssignment,
 )
 
-AVAILABLE_LABEL = mark_safe('<span class="label label-success">Available</span>')
+AVAILABLE_LABEL = mark_safe('<span class="label label-success">Available</span>')  # noqa: S308
 
 UTILIZATION_GRAPH = """
 {% load helpers %}
@@ -38,26 +42,46 @@ UTILIZATION_GRAPH = """
 
 PREFIX_LINK = """
 {% load helpers %}
-{% for i in record.parents|as_range %}
+{% for i in record.ancestors.count|as_range %}
     <i class="mdi mdi-circle-small"></i>
 {% endfor %}
-<a href="{% if record.present_in_database %}{% url 'ipam:prefix' pk=record.pk %}{% else %}{% url 'ipam:prefix_add' %}?prefix={{ record }}{% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}{% if object.site %}&site={{ object.site.pk }}{% endif %}{% if object.tenant %}&tenant_group={{ object.tenant.group.pk }}&tenant={{ object.tenant.pk }}{% endif %}{% endif %}">{{ record.prefix }}</a>
+    <a href="\
+{% if record.present_in_database %}\
+{% url 'ipam:prefix' pk=record.pk %}\
+{% else %}\
+{% url 'ipam:prefix_add' %}\
+?prefix={{ record }}&namespace={{ object.namespace.pk }}\
+{% if object.location %}&location={{ object.location.pk }}{% endif %}\
+{% if object.tenant %}&tenant_group={{ object.tenant.tenant_group.pk }}&tenant={{ object.tenant.pk }}{% endif %}\
+{% endif %}\
+">{{ record.prefix }}</a>
 """
 
 PREFIX_COPY_LINK = """
 {% load helpers %}
-{% for i in record.parents|as_range %}
+{% for i in record.ancestors.count|as_range %}
     <i class="mdi mdi-circle-small"></i>
 {% endfor %}
-<span class="hover_copy"><a href="{% if record.present_in_database %}{% url 'ipam:prefix' pk=record.pk %}{% else %}{% url 'ipam:prefix_add' %}?prefix={{ record }}{% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}{% if object.site %}&site={{ object.site.pk }}{% endif %}{% if object.tenant %}&tenant_group={{ object.tenant.group.pk }}&tenant={{ object.tenant.pk }}{% endif %}{% endif %}" id="copy_{{record.id}}">{{ record.prefix }}</a><button type="button" class="btn btn-inline btn-default hover_copy_button" data-clipboard-target="#copy_{{record.id}}">
-                                <span class="mdi mdi-content-copy"></span>
-                            </button>
-                        </span>
+<span class="hover_copy">
+  <a href="\
+{% if record.present_in_database %}\
+{% url 'ipam:prefix' pk=record.pk %}\
+{% else %}\
+{% url 'ipam:prefix_add' %}\
+?prefix={{ record }}&namespace={{ object.namespace.pk }}\
+{% if object.location %}&location={{ object.location.pk }}{% endif %}\
+{% if object.tenant %}&tenant_group={{ object.tenant.tenant_group.pk }}&tenant={{ object.tenant.pk }}{% endif %}\
+{% endif %}\
+" id="copy_{{record.id}}">{{ record.prefix }}</a>
+  <button type="button" class="btn btn-inline btn-default hover_copy_button" data-clipboard-target="#copy_{{record.id}}">
+    <span class="mdi mdi-content-copy"></span>
+  </button>
+</span>
 """
 
 PREFIX_ROLE_LINK = """
 {% if record.role %}
-    <a href="{% url 'ipam:prefix_list' %}?role={{ record.role.slug }}">{{ record.role }}</a>
+    <a href="{% url 'ipam:prefix_list' %}?role={{ record.role.name }}">{{ record.role }}</a>
 {% else %}
     &mdash;
 {% endif %}
@@ -67,7 +91,13 @@ IPADDRESS_LINK = """
 {% if record.present_in_database %}
     <a href="{{ record.get_absolute_url }}">{{ record.address }}</a>
 {% elif perms.ipam.add_ipaddress %}
-    <a href="{% url 'ipam:ipaddress_add' %}?address={{ record.1 }}{% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}{% if object.tenant %}&tenant={{ object.tenant.pk }}{% endif %}" class="btn btn-xs btn-success">{% if record.0 <= 65536 %}{{ record.0 }}{% else %}Many{% endif %} IP{{ record.0|pluralize }} available</a>
+    <a href="\
+{% url 'ipam:ipaddress_add' %}\
+?address={{ record.1 }}\
+{% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}\
+{% if object.tenant %}&tenant={{ object.tenant.pk }}{% endif %}\
+" class="btn btn-xs btn-success">\
+{% if record.0 <= 65536 %}{{ record.0 }}{% else %}Many{% endif %} IP{{ record.0|pluralize }} available</a>
 {% else %}
     {% if record.0 <= 65536 %}{{ record.0 }}{% else %}Many{% endif %} IP{{ record.0|pluralize }} available
 {% endif %}
@@ -75,26 +105,50 @@ IPADDRESS_LINK = """
 
 IPADDRESS_COPY_LINK = """
 {% if record.present_in_database %}
-    <span class="hover_copy"><a href="{{ record.get_absolute_url }}" id="copy_{{record.id}}">{{ record.address }}</a><button type="button" class="btn btn-inline btn-default hover_copy_button" data-clipboard-target="#copy_{{record.id}}">
-                                <span class="mdi mdi-content-copy"></span>
-                            </button>
-                        </span>
+    <span class="hover_copy">
+        <a href="{{ record.get_absolute_url }}" id="copy_{{record.id}}">
+            {{ record.address }}</a>
+        <button type="button" class="btn btn-inline btn-default hover_copy_button" data-clipboard-target="#copy_{{record.id}}">
+            <span class="mdi mdi-content-copy"></span>
+        </button>
+    </span>
 {% elif perms.ipam.add_ipaddress %}
-    <a href="{% url 'ipam:ipaddress_add' %}?address={{ record.1 }}{% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}{% if object.tenant %}&tenant={{ object.tenant.pk }}{% endif %}" class="btn btn-xs btn-success">{% if record.0 <= 65536 %}{{ record.0 }}{% else %}Many{% endif %} IP{{ record.0|pluralize }} available</a>
+    <a href="\
+{% url 'ipam:ipaddress_add' %}\
+?address={{ record.1 }}\
+{% if object.vrf %}&vrf={{ object.vrf.pk }}{% endif %}\
+{% if object.tenant %}&tenant={{ object.tenant.pk }}{% endif %}\
+" class="btn btn-xs btn-success">\
+{% if record.0 <= 65536 %}{{ record.0 }}{% else %}Many{% endif %} IP{{ record.0|pluralize }} available</a>
 {% else %}
     {% if record.0 <= 65536 %}{{ record.0 }}{% else %}Many{% endif %} IP{{ record.0|pluralize }} available
 {% endif %}
 """
 
 IPADDRESS_ASSIGN_LINK = """
-<a href="{% url 'ipam:ipaddress_edit' pk=record.pk %}?{% if request.GET.interface %}interface={{ request.GET.interface }}{% elif request.GET.vminterface %}vminterface={{ request.GET.vminterface }}{% endif %}&return_url={{ request.GET.return_url }}">{{ record }}</a>
+<a href="\
+{% url 'ipam:ipaddress_edit' pk=record.pk %}\
+?{% if request.GET.interface %}interface={{ request.GET.interface }}\
+{% elif request.GET.vminterface %}\
+vminterface={{ request.GET.vminterface }}{% endif %}\
+&return_url={{ request.GET.return_url }}">{{ record }}</a>
 """
 
 IPADDRESS_ASSIGN_COPY_LINK = """
-<span class="hover_copy"><a href="{% url 'ipam:ipaddress_edit' pk=record.pk %}?{% if request.GET.interface %}interface={{ request.GET.interface }}{% elif request.GET.vminterface %}vminterface={{ request.GET.vminterface }}{% endif %}&return_url={{ request.GET.return_url }}" id="copy_{{record.pk}}">{{ record }}</a><button type="button" class="btn btn-inline btn-default hover_copy_button" data-clipboard-target="#copy_{{record.pk}}">
-                                <span class="mdi mdi-content-copy"></span>
-                            </button>
-                        </span>
+<span class="hover_copy">
+<a href="\
+{% url 'ipam:ipaddress_edit' pk=record.pk %}\
+?{% if request.GET.interface %}\
+interface={{ request.GET.interface }}\
+{% elif request.GET.vminterface %}\
+vminterface={{ request.GET.vminterface }}\
+{% endif %}\
+&return_url={{ request.GET.return_url }}" id="copy_{{record.pk}}">\
+{{ record }}\
+</a><button type="button" class="btn btn-inline btn-default hover_copy_button" data-clipboard-target="#copy_{{record.pk}}">
+    <span class="mdi mdi-content-copy"></span>
+</button>
+</span>
 """
 
 VRF_LINK = """
@@ -119,7 +173,12 @@ VLAN_LINK = """
 {% if record.present_in_database %}
     <a href="{{ record.get_absolute_url }}">{{ record.vid }}</a>
 {% elif perms.ipam.add_vlan %}
-    <a href="{% url 'ipam:vlan_add' %}?vid={{ record.vid }}&group={{ vlan_group.pk }}{% if vlan_group.site %}&site={{ vlan_group.site.pk }}{% endif %}{% if vlan_group.location %}&location={{ vlan_group.location.pk }}{% endif %}" class="btn btn-xs btn-success">{{ record.available }} VLAN{{ record.available|pluralize }} available</a>
+    <a href="\
+{% url 'ipam:vlan_add' %}\
+?vid={{ record.vid }}&vlan_group={{ vlan_group.pk }}\
+{% if vlan_group.location %}&location={{ vlan_group.location.pk }}{% endif %}\
+{% if vlan_group.location %}&location={{ vlan_group.location.pk }}{% endif %}\
+" class="btn btn-xs btn-success">{{ record.available }} VLAN{{ record.available|pluralize }} available</a>\
 {% else %}
     {{ record.available }} VLAN{{ record.available|pluralize }} available
 {% endif %}
@@ -135,7 +194,7 @@ VLAN_PREFIXES = """
 
 VLAN_ROLE_LINK = """
 {% if record.role %}
-    <a href="{% url 'ipam:vlan_list' %}?role={{ record.role.slug }}">{{ record.role }}</a>
+    <a href="{% url 'ipam:vlan_list' %}?role={{ record.role.name }}">{{ record.role }}</a>
 {% else %}
     &mdash;
 {% endif %}
@@ -144,13 +203,32 @@ VLAN_ROLE_LINK = """
 VLANGROUP_ADD_VLAN = """
 {% with next_vid=record.get_next_available_vid %}
     {% if next_vid and perms.ipam.add_vlan %}
-        <a href="{% url 'ipam:vlan_add' %}?site={{ record.site_id }}{% if record.location %}&location={{ record.location_id }}{% endif %}&group={{ record.pk }}&vid={{ next_vid }}" title="Add VLAN" class="btn btn-xs btn-success">
-            <i class="mdi mdi-plus-thick" aria-hidden="true"></i>
-        </a>
+        <a href="\
+{% url 'ipam:vlan_add' %}\
+?location={{ record.location_id }}\
+{% if record.location %}&location={{ record.location_id }}{% endif %}\
+&vlan_group={{ record.pk }}&vid={{ next_vid }}\
+" title="Add VLAN" class="btn btn-xs btn-success"><i class="mdi mdi-plus-thick" aria-hidden="true"></i></a>
     {% endif %}
 {% endwith %}
 """
 
+
+#
+# Namespaces
+#
+
+
+class NamespaceTable(BaseTable):
+    pk = ToggleColumn()
+    name = tables.LinkColumn()
+    tags = TagColumn(url_name="ipam:namespace_list")
+
+    class Meta(BaseTable.Meta):
+        model = Namespace
+        fields = ("pk", "name", "description", "location")
+
+
 #
 # VRFs
 #
@@ -159,9 +237,8 @@ VLANGROUP_ADD_VLAN = """
 class VRFTable(BaseTable):
     pk = ToggleColumn()
     name = tables.LinkColumn()
-    rd = tables.Column(verbose_name="RD")
+    # rd = tables.Column(verbose_name="RD")
     tenant = TenantColumn()
-    enforce_unique = BooleanColumn(verbose_name="Unique")
     import_targets = tables.TemplateColumn(template_code=VRF_TARGETS, orderable=False)
     export_targets = tables.TemplateColumn(template_code=VRF_TARGETS, orderable=False)
     tags = TagColumn(url_name="ipam:vrf_list")
@@ -171,19 +248,59 @@ class VRFTable(BaseTable):
         fields = (
             "pk",
             "name",
-            "rd",
+            # "rd",
+            "namespace",
             "tenant",
-            "enforce_unique",
             "description",
             "import_targets",
             "export_targets",
             "tags",
         )
-        default_columns = ("pk", "name", "rd", "tenant", "description")
+        # default_columns = ("pk", "name", "rd", "namespace", "tenant", "description")
+        default_columns = ("pk", "name", "namespace", "tenant", "description")
+
+
+class VRFDeviceAssignmentTable(BaseTable):
+    """Table for displaying VRF Device Assignments with RD."""
+
+    vrf = tables.Column(verbose_name="VRF", linkify=lambda record: record.vrf.get_absolute_url(), accessor="vrf.name")
+    namespace = tables.Column(
+        verbose_name="Namespace",
+        linkify=lambda record: record.vrf.namespace.get_absolute_url(),
+        accessor="vrf.namespace.name",
+    )
+    device = tables.Column(
+        linkify=lambda record: record.device.get_absolute_url(), accessor="device.name", verbose_name="Device"
+    )
+    virtual_machine = tables.Column(
+        linkify=lambda record: record.virtual_machine.get_absolute_url(),
+        accessor="virtual_machine.name",
+        verbose_name="Virtual Machine",
+    )
+    rd = tables.Column(verbose_name="VRF RD")
+    tenant = TenantColumn(accessor="vrf.tenant")
+
+    class Meta(BaseTable.Meta):
+        model = VRFDeviceAssignment
+        orderable = False
+        fields = ("vrf", "namespace", "device", "virtual_machine", "rd", "tenant")
+
+
+class VRFPrefixAssignmentTable(BaseTable):
+    """Table for displaying VRF Prefix Assignments."""
+
+    vrf = tables.Column(verbose_name="VRF", linkify=lambda record: record.vrf.get_absolute_url(), accessor="vrf.name")
+    prefix = tables.Column(linkify=True)
+    rd = tables.Column(accessor="vrf.rd", verbose_name="RD")
+    tenant = TenantColumn(accessor="vrf.tenant")
+
+    class Meta(BaseTable.Meta):
+        model = VRFPrefixAssignment
+        fields = ("vrf", "prefix", "rd", "tenant")
 
 
 #
-# Route targets
+#
 #
 
 
@@ -208,21 +325,20 @@ class RIRTable(BaseTable):
     pk = ToggleColumn()
     name = tables.LinkColumn()
     is_private = BooleanColumn(verbose_name="Private")
-    aggregate_count = LinkedCountColumn(
-        viewname="ipam:aggregate_list",
-        url_params={"rir": "slug"},
-        verbose_name="Aggregates",
+    assigned_prefix_count = LinkedCountColumn(
+        viewname="ipam:prefix_list",
+        url_params={"rir": "name"},
+        verbose_name="Assigned Prefixes",
     )
-    actions = ButtonsColumn(RIR, pk_field="slug")
+    actions = ButtonsColumn(RIR)
 
     class Meta(BaseTable.Meta):
         model = RIR
         fields = (
             "pk",
             "name",
-            "slug",
             "is_private",
-            "aggregate_count",
+            "assigned_prefix_count",
             "description",
             "actions",
         )
@@ -230,91 +346,7 @@ class RIRTable(BaseTable):
             "pk",
             "name",
             "is_private",
-            "aggregate_count",
-            "description",
-            "actions",
-        )
-
-
-#
-# Aggregates
-#
-
-
-class AggregateTable(BaseTable):
-    pk = ToggleColumn()
-    prefix = tables.LinkColumn(verbose_name="Aggregate", order_by=("network", "prefix_length"))
-    rir = tables.Column(linkify=True)
-    tenant = TenantColumn()
-    date_added = tables.DateColumn(format="Y-m-d", verbose_name="Added")
-
-    class Meta(BaseTable.Meta):
-        model = Aggregate
-        fields = ("pk", "prefix", "rir", "tenant", "date_added", "description")
-
-
-class AggregateDetailTable(AggregateTable):
-    child_count = tables.Column(verbose_name="Prefixes")
-    utilization = tables.TemplateColumn(template_code=UTILIZATION_GRAPH, orderable=False)
-    tags = TagColumn(url_name="ipam:aggregate_list")
-
-    class Meta(AggregateTable.Meta):
-        fields = (
-            "pk",
-            "prefix",
-            "rir",
-            "tenant",
-            "child_count",
-            "utilization",
-            "date_added",
-            "description",
-            "tags",
-        )
-        default_columns = (
-            "pk",
-            "prefix",
-            "rir",
-            "tenant",
-            "child_count",
-            "utilization",
-            "date_added",
-            "description",
-        )
-
-
-#
-# Roles
-#
-
-
-class RoleTable(BaseTable):
-    pk = ToggleColumn()
-    name = tables.LinkColumn()
-    prefix_count = LinkedCountColumn(
-        viewname="ipam:prefix_list",
-        url_params={"role": "slug"},
-        verbose_name="Prefixes",
-    )
-    vlan_count = LinkedCountColumn(viewname="ipam:vlan_list", url_params={"role": "slug"}, verbose_name="VLANs")
-    actions = ButtonsColumn(Role, pk_field="slug")
-
-    class Meta(BaseTable.Meta):
-        model = Role
-        fields = (
-            "pk",
-            "name",
-            "slug",
-            "prefix_count",
-            "vlan_count",
-            "description",
-            "weight",
-            "actions",
-        )
-        default_columns = (
-            "pk",
-            "name",
-            "prefix_count",
-            "vlan_count",
+            "assigned_prefix_count",
             "description",
             "actions",
         )
@@ -325,42 +357,47 @@ class RoleTable(BaseTable):
 #
 
 
-class PrefixTable(StatusTableMixin, BaseTable):
+class PrefixTable(StatusTableMixin, RoleTableMixin, BaseTable):
     pk = ToggleColumn()
     prefix = tables.TemplateColumn(
         template_code=PREFIX_COPY_LINK, attrs={"td": {"class": "text-nowrap"}}, order_by=("network", "prefix_length")
     )
-    vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
+    # vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
     tenant = TenantColumn()
-    site = tables.Column(linkify=True)
     location = tables.Column(linkify=True)
+    namespace = tables.Column(linkify=True)
     vlan = tables.Column(linkify=True, verbose_name="VLAN")
-    role = tables.TemplateColumn(template_code=PREFIX_ROLE_LINK)
-    is_pool = BooleanColumn(verbose_name="Pool")
+    rir = tables.Column(linkify=True)
+    children = tables.Column(accessor="descendants_count")
+    date_allocated = tables.DateTimeColumn()
 
     class Meta(BaseTable.Meta):
         model = Prefix
+        orderable = False
         fields = (
             "pk",
             "prefix",
+            "type",
             "status",
             "children",
-            "vrf",
+            # "vrf",
+            "namespace",
             "tenant",
-            "site",
             "location",
             "vlan",
             "role",
-            "is_pool",
+            "rir",
+            "date_allocated",
             "description",
         )
         default_columns = (
             "pk",
             "prefix",
+            "type",
             "status",
-            "vrf",
+            # "vrf",
+            "namespace",
             "tenant",
-            "site",
             "location",
             "vlan",
             "role",
@@ -380,28 +417,29 @@ class PrefixDetailTable(PrefixTable):
         fields = (
             "pk",
             "prefix",
+            "namespace",
+            "type",
             "status",
             "children",
-            "vrf",
+            # "vrf",
             "utilization",
             "tenant",
-            "site",
             "location",
             "vlan",
             "role",
-            "is_pool",
             "description",
             "tags",
         )
         default_columns = (
             "pk",
             "prefix",
+            "namespace",
+            "type",
             "status",
             "children",
-            "vrf",
+            # "vrf",
             "utilization",
             "tenant",
-            "site",
             "location",
             "vlan",
             "role",
@@ -414,35 +452,36 @@ class PrefixDetailTable(PrefixTable):
 #
 
 
-class IPAddressTable(StatusTableMixin, BaseTable):
+class IPAddressTable(StatusTableMixin, RoleTableMixin, BaseTable):
     pk = ToggleColumn()
     address = tables.TemplateColumn(
-        template_code=IPADDRESS_COPY_LINK, verbose_name="IP Address", order_by=("host", "prefix_length")
+        template_code=IPADDRESS_COPY_LINK, verbose_name="IP Address", order_by=("host", "mask_length")
     )
-    vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
-    role = ChoiceFieldColumn()
     tenant = TenantColumn()
-    assigned_object = tables.Column(linkify=True, orderable=False, verbose_name="Interface")
-    assigned_object_parent = tables.Column(
-        accessor="assigned_object__parent",
-        linkify=True,
-        orderable=False,
-        verbose_name="Interface Parent",
+    parent__namespace = tables.Column(linkify=True)
+    interface_count = tables.Column(verbose_name="Interfaces")
+    interface_parent_count = tables.Column(verbose_name="Devices")
+    vm_interface_count = LinkedCountColumn(
+        viewname="virtualization:vminterface_list", url_params={"ip_addresses": "pk"}, verbose_name="VM Interfaces"
     )
+    vm_interface_parent_count = tables.Column(verbose_name="Virtual Machines")
 
     class Meta(BaseTable.Meta):
         model = IPAddress
         fields = (
             "pk",
             "address",
-            "vrf",
+            "type",
             "status",
             "role",
             "tenant",
-            "assigned_object",
-            "assigned_object_parent",
             "dns_name",
             "description",
+            "parent__namespace",
+            "interface_count",
+            "interface_parent_count",
+            "vm_interface_count",
+            "vm_interface_parent_count",
         )
         row_attrs = {
             "class": lambda record: "success" if not isinstance(record, IPAddress) else "",
@@ -452,14 +491,18 @@ class IPAddressTable(StatusTableMixin, BaseTable):
 class IPAddressDetailTable(IPAddressTable):
     nat_inside = tables.Column(linkify=True, orderable=False, verbose_name="NAT (Inside)")
     tenant = TenantColumn()
-    assigned = BooleanColumn(accessor="assigned_object_id", verbose_name="Assigned")
     tags = TagColumn(url_name="ipam:ipaddress_list")
+    assigned = BooleanColumn(accessor="assigned_count")
+
+    def render_assigned(self, column, value):
+        return column.render(value > 0)
 
     class Meta(IPAddressTable.Meta):
         fields = (
             "pk",
             "address",
-            "vrf",
+            "parent__namespace",
+            "type",
             "status",
             "role",
             "tenant",
@@ -472,7 +515,8 @@ class IPAddressDetailTable(IPAddressTable):
         default_columns = (
             "pk",
             "address",
-            "vrf",
+            "parent__namespace",
+            "type",
             "status",
             "role",
             "tenant",
@@ -483,19 +527,21 @@ class IPAddressDetailTable(IPAddressTable):
 
 
 class IPAddressAssignTable(StatusTableMixin, BaseTable):
+    pk = ToggleColumn(visible=True)
     address = tables.TemplateColumn(template_code=IPADDRESS_ASSIGN_COPY_LINK, verbose_name="IP Address")
-    assigned_object = tables.Column(orderable=False)
+    # TODO: add interface M2M
 
     class Meta(BaseTable.Meta):
         model = IPAddress
         fields = (
+            "pk",
             "address",
+            "parent__namespace",
             "dns_name",
-            "vrf",
+            "type",
             "status",
             "role",
             "tenant",
-            "assigned_object",
             "description",
         )
         orderable = False
@@ -506,13 +552,106 @@ class InterfaceIPAddressTable(StatusTableMixin, BaseTable):
     List IP addresses assigned to a specific Interface.
     """
 
-    address = tables.TemplateColumn(template_code=IPADDRESS_ASSIGN_COPY_LINK, verbose_name="IP Address")
-    vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
+    address = tables.TemplateColumn(template_code=IPADDRESS_COPY_LINK, verbose_name="IP Address")
+    # vrf = tables.TemplateColumn(template_code=VRF_LINK, verbose_name="VRF")
     tenant = TenantColumn()
 
     class Meta(BaseTable.Meta):
         model = IPAddress
-        fields = ("address", "vrf", "status", "role", "tenant", "description")
+        fields = ("address", "type", "status", "role", "tenant", "description")
+
+
+class IPAddressInterfaceTable(InterfaceTable):
+    name = tables.TemplateColumn(
+        template_code='<i class="mdi mdi-{% if iface.mgmt_only %}wrench{% elif iface.is_lag %}drag-horizontal-variant'
+        "{% elif iface.is_virtual %}circle{% elif iface.is_wireless %}wifi{% else %}ethernet"
+        '{% endif %}"></i> <a href="{{ record.get_absolute_url }}">{{ value }}</a>',
+        attrs={"td": {"class": "text-nowrap"}},
+    )
+    parent_interface = tables.Column(linkify=True, verbose_name="Parent")
+    bridge = tables.Column(linkify=True)
+    lag = tables.Column(linkify=True, verbose_name="LAG")
+
+    class Meta(DeviceComponentTable.Meta):
+        model = Interface
+        fields = (
+            "pk",
+            "name",
+            "device",
+            "type",
+            "status",
+            "label",
+            "enabled",
+            "type",
+            "parent_interface",
+            "bridge",
+            "lag",
+            "mgmt_only",
+            "mtu",
+            "mode",
+            "mac_address",
+            "description",
+            "cable",
+            "cable_peer",
+            "connection",
+            "tags",
+            "ip_addresses",
+            "untagged_vlan",
+            "tagged_vlans",
+        )
+        default_columns = [
+            "pk",
+            "device",
+            "name",
+            "status",
+            "label",
+            "enabled",
+            "type",
+            "parent_interface",
+            "lag",
+            "mtu",
+            "mode",
+            "description",
+            "ip_addresses",
+            "cable",
+            "connection",
+        ]
+        row_attrs = {
+            "style": cable_status_color_css,
+            "data-name": lambda record: record.name,
+        }
+
+
+#
+# IPAddress to Interface
+#
+
+
+class IPAddressToInterfaceTable(BaseTable):
+    pk = ToggleColumn()
+    ip_address = tables.Column(linkify=True, verbose_name="IP Address")
+    # TODO(jathan): Probably should crib from something like the CABLETERMINATION column template so
+    # that these columns show something like device1 > interface1 instead of just interface1 for
+    # usability?
+    interface = tables.Column(linkify=True)
+    vm_interface = tables.Column(linkify=True, verbose_name="VM Interface")
+
+    class Meta(BaseTable.Meta):
+        model = IPAddressToInterface
+        fields = (
+            "pk",
+            "ip_address",
+            "interface",
+            "vm_interface",
+            "is_source",
+            "is_destination",
+            "is_default",
+            "is_preferred",
+            "is_primary",
+            "is_secondary",
+            "is_standby",
+        )
+        default_columns = ("pk", "ip_address", "interface", "vm_interface")
 
 
 #
@@ -523,15 +662,14 @@ class InterfaceIPAddressTable(StatusTableMixin, BaseTable):
 class VLANGroupTable(BaseTable):
     pk = ToggleColumn()
     name = tables.Column(linkify=True)
-    site = tables.Column(linkify=True)
     location = tables.Column(linkify=True)
-    vlan_count = LinkedCountColumn(viewname="ipam:vlan_list", url_params={"group": "slug"}, verbose_name="VLANs")
+    vlan_count = LinkedCountColumn(viewname="ipam:vlan_list", url_params={"vlan_group": "name"}, verbose_name="VLANs")
     actions = ButtonsColumn(model=VLANGroup, prepend_template=VLANGROUP_ADD_VLAN)
 
     class Meta(BaseTable.Meta):
         model = VLANGroup
-        fields = ("pk", "name", "site", "location", "vlan_count", "slug", "description", "actions")
-        default_columns = ("pk", "name", "site", "location", "vlan_count", "description", "actions")
+        fields = ("pk", "name", "location", "vlan_count", "description", "actions")
+        default_columns = ("pk", "name", "location", "vlan_count", "description", "actions")
 
 
 #
@@ -539,23 +677,20 @@ class VLANGroupTable(BaseTable):
 #
 
 
-class VLANTable(StatusTableMixin, BaseTable):
+class VLANTable(StatusTableMixin, RoleTableMixin, BaseTable):
     pk = ToggleColumn()
     vid = tables.TemplateColumn(template_code=VLAN_LINK, verbose_name="ID")
-    site = tables.Column(linkify=True)
     location = tables.Column(linkify=True)
-    group = tables.Column(linkify=True)
+    vlan_group = tables.Column(linkify=True)
     tenant = TenantColumn()
-    role = tables.TemplateColumn(template_code=VLAN_ROLE_LINK)
 
     class Meta(BaseTable.Meta):
         model = VLAN
         fields = (
             "pk",
             "vid",
-            "site",
             "location",
-            "group",
+            "vlan_group",
             "name",
             "tenant",
             "status",
@@ -576,9 +711,8 @@ class VLANDetailTable(VLANTable):
         fields = (
             "pk",
             "vid",
-            "site",
             "location",
-            "group",
+            "vlan_group",
             "name",
             "prefixes",
             "tenant",
@@ -590,9 +724,8 @@ class VLANDetailTable(VLANTable):
         default_columns = (
             "pk",
             "vid",
-            "site",
             "location",
-            "group",
+            "vlan_group",
             "name",
             "prefixes",
             "tenant",
@@ -639,9 +772,8 @@ class InterfaceVLANTable(StatusTableMixin, BaseTable):
 
     vid = tables.LinkColumn(viewname="ipam:vlan", args=[Accessor("pk")], verbose_name="ID")
     tagged = BooleanColumn()
-    site = tables.Column(linkify=True)
     location = tables.Column(linkify=True)
-    group = tables.Column(accessor=Accessor("group__name"), verbose_name="Group")
+    vlan_group = tables.Column(accessor=Accessor("vlan_group__name"), verbose_name="Group")
     tenant = TenantColumn()
     role = tables.TemplateColumn(template_code=VLAN_ROLE_LINK)
 
@@ -650,9 +782,8 @@ class InterfaceVLANTable(StatusTableMixin, BaseTable):
         fields = (
             "vid",
             "tagged",
-            "site",
             "location",
-            "group",
+            "vlan_group",
             "name",
             "tenant",
             "status",
@@ -685,7 +816,7 @@ class ServiceTable(BaseTable):
             "parent",
             "protocol",
             "ports",
-            "ipaddresses",
+            "ip_addresses",
             "description",
             "tags",
         )
