@@ -4,14 +4,16 @@
 
 // Slugify
 function slugify(s, num_chars) {
-    s = s.replace(/[^\-\.\w\s]/g, '');          // Remove unneeded chars
+    s = s.replace(/[^_A-Za-z0-9\-\s\.]/g, '');  // Remove non-ascii chars
     s = s.replace(/^[\s\.]+|[\s\.]+$/g, '');    // Trim leading/trailing spaces
     s = s.replace(/[\-\.\s]+/g, '-');           // Convert spaces and decimals to hyphens
     s = s.toLowerCase();                        // Convert to lowercase
-    // Declare `const slugify_prefer_underscores = true` globally if you want to use underscores instead of hyphens
-    if (typeof slugify_prefer_underscores !== "undefined") {
-        s = s.replace(/-/g, '_');
+    s = s.replace(/-/g, '_');
+
+    if (/^[^_A-Za-z]/.test(s)) {  // Slug must start with a letter or underscore only
+        s = 'a' + s;
     }
+
     return s.substring(0, num_chars);           // Trim to first num_chars chars
 }
 
@@ -87,6 +89,11 @@ function initializeCheckboxes(context){
 function initializeSlugField(context){
     this_context = $(context);
     var slug_field = this_context.find('#id_slug');
+    // If id_slug field is not to be found
+    // check if it is rename to key field like what we did for CustomField and Relationship
+    if (slug_field.length == 0) {
+        slug_field = this_context.find('#id_key');
+    }
     if (slug_field.length != 0) {
         var slug_source_arr = slug_field.attr('slug-source').split(" ");
         var slug_length = slug_field.attr('maxlength');
@@ -197,8 +204,8 @@ function initializeDynamicChoiceSelection(context, dropdownParent=null){
                     }
 
 
-                    // Allow for controlling the brief setting from within APISelect
-                    parameters.brief = ( $(element).is('[data-full]') ? undefined : true );
+                    // Allow for controlling the depth setting from within APISelect
+                    parameters.depth = parseInt($(element).attr('data-depth'))
 
                     // Attach any extra query parameters
                     $.each(element.attributes, function(index, attr){
@@ -257,11 +264,6 @@ function initializeDynamicChoiceSelection(context, dropdownParent=null){
 
                     results = results.reduce((results,record,idx) => {
                         record.text = record[element.getAttribute('display-field')] || record.name;
-                        if (record._depth) {
-                            // Annotate hierarchical depth for MPTT objects
-                            record.text = '--'.repeat(record._depth) + ' ' + record.text;
-                        }
-
                         record.id = record[element.getAttribute('value-field')] || record.id;
                         if(element.getAttribute('disabled-indicator') && record[element.getAttribute('disabled-indicator')]) {
                             // The disabled-indicator equated to true, so we disable this option
@@ -287,6 +289,12 @@ function initializeDynamicChoiceSelection(context, dropdownParent=null){
                         else {
                             results[idx] = record;
                         }
+                        // DynamicGroupSerializer has a `children` field which fits an inappropriate if condition
+                        // in select2.min.js, which will result in the incorrect rendering of DynamicGroup DynamicChoiceField.
+                        // So we nullify the field here since we do not need this field.
+                        if (record?.url ? record.url.includes("dynamic-groups") : false){
+                            record.children = undefined;
+                        }
 
                         return results;
                     },Object.create(null));
@@ -372,7 +380,6 @@ function initializeTags(context, dropdownParent=null){
                 var offset = (params.page - 1) * 50 || 0;
                 var parameters = {
                     q: params.term,
-                    brief: 1,
                     limit: 50,
                     offset: offset,
                 };
@@ -468,6 +475,61 @@ function initializeMultiValueChar(context, dropdownParent=null){
 
 function initializeDynamicFilterForm(context){
     this_context = $(context);
+
+    function initializeDynamicFilterSelect(element) {
+        // On change of a select field in default filter form
+        // Replicate that change into dynamic filter form and vice-versa
+        $(element).on("change", function (e){
+            let field_name = $(this).attr("name");
+            let field_values = $(this).select2('data');
+            let form_id = $(this).parents("form").attr("id");
+
+            let default_filters_field_dom = $(`#default-filter form select[name=${field_name}]`);
+            let advanced_filters_field_dom = $(`#advanced-filter #filterform-table tbody tr td select[name=${field_name}]`);
+
+            // Only apply logic if fields with same name attr are on both advanced and default filter form
+            if(default_filters_field_dom.length && advanced_filters_field_dom.length){
+                let default_filters_field_ids = default_filters_field_dom.select2('data').map(data => data["id"]);
+                let advanced_filters_field_ids = advanced_filters_field_dom.select2('data').map(data => data["id"]);
+
+                // Only change field value if both fields do not have equal values
+                if (JSON.stringify(advanced_filters_field_ids) !== JSON.stringify(default_filters_field_ids)){
+                    if(form_id === "dynamic-filter-form"){
+                        changeSelect2FieldValue(default_filters_field_dom, field_values);
+                    }
+                    else {
+                        changeSelect2FieldValue(advanced_filters_field_dom, field_values);
+                    }
+                }
+            }
+        });
+    }
+
+    function initializeDynamicFilterInput(element) {
+        // On change of input field in default filter form
+        // Replicate that change into dynamic filter form and vice-versa
+        $(element).on("change", function (e){
+            let field_name = $(this).attr("name");
+            let field_value = $(this).val();
+            let form_id = $(this).parents("form").attr("id");
+            let default_filters_field_dom = $(`#default-filter form input[name=${field_name}]`);
+            let advanced_filters_field_dom = $(`#advanced-filter #filterform-table tbody tr td input[name=${field_name}]`);
+
+            // Only apply logic if fields with same name attr are on both advanced and default filter form
+            if(default_filters_field_dom.length && advanced_filters_field_dom.length){
+                // Only change field value if both fields do not have equal values
+                if (default_filters_field_dom.val() !== advanced_filters_field_dom.val()){
+                    if(form_id === "dynamic-filter-form"){
+                        default_filters_field_dom.val(field_value);
+                    }
+                    else {
+                        advanced_filters_field_dom.val(field_value);
+                    }
+                }
+            }
+        })
+    }
+
     // Dynamic filter form
     this_context.find(".lookup_type-select").bind("change", function(){
         let parent_element = $(this).parents("tr")
@@ -478,15 +540,19 @@ function initializeDynamicFilterForm(context){
 
         if(lookup_type_val){
             $.ajax({
-                url: `/api/core/filterset-fields/lookup-value-dom-element/?field_name=${lookup_type_val}&content_type=${contenttype}`,
+                url: `/api/ui/core/filterset-fields/lookup-value-dom-element/?field_name=${lookup_type_val}&content_type=${contenttype}`,
                 async: true,
-                contentType: 'application/json',
-                dataType: 'json',
+                headers: {'Accept': '*/*'},
                 type: 'GET',
             }).done(function (response) {
-                newEl = $(response.dom_element)
+                newEl = $(response)
                 newEl.addClass("lookup_value-input")
                 replaceEl(lookup_value_element, newEl)
+                if (newEl.prop("tagName") == "SELECT") {
+                    initializeDynamicFilterSelect(newEl);
+                } else {
+                    initializeDynamicFilterInput(newEl);
+                }
             }).fail(function (xhr, status, error) {
                 // Default to Input:text field if error occurs
                 createInput(lookup_value_element)
@@ -508,6 +574,91 @@ function initializeDynamicFilterForm(context){
         lookup_value_element.val(null).trigger('change')
 
     })
+
+    // By default on lookup_value field names are form-\d-lookup_value, thats why
+    // on page load we change all `lookup_value` name to its relevant `lookup_type` value
+    this_context.find(".dynamic-filterform").each(function(){
+        lookup_type_value = $(this).find(".lookup_type-select").val();
+        lookup_value = $(this).find(".lookup_value-input");
+        lookup_value.attr("name", lookup_type_value);
+    })
+
+    // Remove applied filters
+    this_context.find(".remove-filter-param").on("click", function(){
+        let query_params = location.search;
+        let type = $(this).attr("data-field-type");
+        let field_value = $(this).attr("data-field-value");
+        let query_string = location.search.substr(1).split("&");
+
+        if (type === "parent") {
+            query_string = query_string.filter(item => item.search(field_value) < 0);
+        } else {
+            let parent = $(this).attr("data-field-parent");
+            query_string = query_string.filter(item => item.search(parent + "=" + field_value) < 0)
+        }
+        location.replace("?" + query_string.join("&"))
+    })
+
+    // On submit of filter form
+    this_context.find("#dynamic-filter-form, #default-filter form").on("submit", function(e){
+        e.preventDefault()
+        let dynamic_form = $("#dynamic-filter-form");
+        dynamic_form.find(`input[name*="form-"], select[name*="form-"]`).removeAttr("name")
+        // Append q form field to dynamic filter form via hidden input
+        let q_field = $('#id_q')
+        let q_field_phantom = $('<input type="hidden" name="q" />')
+        q_field_phantom.val(q_field.val())
+        dynamic_form.append(q_field_phantom);
+
+        // Get the serialize data from the forms and filter out query_params which values are empty e.g ?sam=&dan=2 becomes dan=2
+        let dynamic_filter_form_query = $("#dynamic-filter-form").serialize().split("&").filter(params => params.split("=")[1]?.length || 0 )
+        let default_filter_form_query = $("#default-filter form").serialize().split("&").filter(params => params.split("=")[1]?.length || 0 )
+        // Union Operation
+        let search_query = [...new Set([...default_filter_form_query, ...dynamic_filter_form_query])].join("&")
+        location.replace("?" + search_query)
+    })
+
+    // On submit of filter search form
+    this_context.find("#search-form").on("submit", function(e){
+        // Since the Dynamic Filter Form will already grab my q field, just have it do a majority of the work.
+        e.preventDefault()
+        $("#dynamic-filter-form").submit()
+    })
+
+    // Clear new row values upon creation
+    this_context.find(".dynamic-filterform-add .add-row").click(function(){
+        let new_fields_parent_element = $(".dynamic-filterform").last()
+        let lookup_field_classes = [".lookup_field-select", ".lookup_type-select", ".lookup_value-input"];
+        lookup_field_classes.forEach(field_class => {
+            let element = new_fields_parent_element.find(field_class);
+            element.val(null).trigger('change')
+        })
+        // reinitialize jsify_form
+        initializeDynamicFilterForm($(document));
+    })
+
+    function changeSelect2FieldValue(dom_element, values){
+        dom_element.val(null)
+        values.forEach(function (value){
+            // Does an element already exist?
+            if (!dom_element.find("option[value='" + value.id + "']").length) {
+                let new_option = new Option(value.text, value.id, true, true);
+                dom_element.append(new_option);
+            }
+        })
+        dom_element.val(values.map(data => data["id"]));
+        dom_element.trigger('change');
+    }
+
+    this_context.find("#default-filter form select, #advanced-filter select").each(function() {
+        initializeDynamicFilterSelect(this);
+    });
+
+    // On change of input field in default filter form
+    // Replicate that change into dynamic filter form and vice-versa
+    this_context.find("#default-filter form input, #advanced-filter input").each(function() {
+        initializeDynamicFilterInput(this);
+    });
 }
 
 function initializeSortableList(context){
@@ -675,4 +826,4 @@ function headerOffsetScroll() {
 
 // Account for the header height when hash-scrolling
 window.addEventListener('load', headerOffsetScroll);
-window.addEventListener('hashchange', headerOffsetScroll);
\ No newline at end of file
+window.addEventListener('hashchange', headerOffsetScroll);
