@@ -1,17 +1,18 @@
 var url = nautobot_api_path + "extras/job-results/";
 var timeout = 1000;
-var terminal_statuses = ['completed', 'failed', 'errored'];
+var terminal_statuses = ['FAILURE', 'REVOKED', 'SUCCESS'];
+var session_key = "ajax_table_current_page";
 
 function updatePendingStatusLabel(status) {
     // Updates "Status" label in "Summary of Results" table in JobResult detail view.
     var labelClass;
-    if (status.value === 'failed' || status.value === 'errored') {
+    if (status.value === 'FAILURE') {
         labelClass = 'danger';
     }
-    else if (status.value === 'running') {
+    else if (status.value === 'STARTED') {
         labelClass = 'warning';
     }
-    else if (status.value === 'completed') {
+    else if (status.value === 'SUCCESS') {
         labelClass = 'success';
     }
     else {
@@ -24,7 +25,9 @@ function updatePendingStatusLabel(status) {
 
 function updateLogTable(result_id) {
     // Calls `update_log_table` to refresh the jobs table from the `/log-table/` endpoint
-    update_log_table('', '/extras/job-results/' + result_id + '/log-table/');
+    // Grab the query string from session storage and pass it through to the log table.
+    let qs = window.sessionStorage.getItem(session_key) || "";
+    update_log_table(qs, '/extras/job-results/' + result_id + '/log-table/');
 }
 
 $(document).ready(function(){
@@ -44,9 +47,13 @@ $(document).ready(function(){
                     // Update the job logs table
                     updateLogTable(pending_result_id);
 
-                    // If there is a terminal status, refresh the page.
+                    /// Should reload the page yet?
+                    let reload_page = window.sessionStorage.getItem(session_key) == null;
+
+                    // If there is a terminal status, refresh the page and clear session storage.
                     if (terminal_statuses.includes(data.status.value)) {
-                        window.location.reload();
+                        window.sessionStorage.removeItem(session_key);
+                        reload_page && window.location.reload();
                     }
                     // Otherwise call myself again after `timeout`.
                     else {
