@@ -130,4 +130,9 @@ function setSystemThemeActive() {
     tdSystemTheme.classList.add("active-theme");
     tdDarkTheme.classList.remove("active-theme");
     tdLightTheme.classList.remove("active-theme");
+}
+
+function viewNewUI() {
+    document.cookie = "newui=1; path=/";
+    document.location.reload(true);
 }
\ No newline at end of file
