@@ -1,8 +1,13 @@
 import factory
 
-from nautobot.core.factory import OrganizationalModelFactory, PrimaryModelFactory
+from nautobot.core.factory import (
+    NautobotBoolIterator,
+    OrganizationalModelFactory,
+    PrimaryModelFactory,
+    UniqueFaker,
+    random_instance,
+)
 from nautobot.tenancy.models import TenantGroup, Tenant
-from nautobot.utilities.factory import random_instance, UniqueFaker
 
 
 class TenantGroupFactory(OrganizationalModelFactory):
@@ -15,10 +20,10 @@ class TenantGroupFactory(OrganizationalModelFactory):
 
     name = UniqueFaker("company")
 
-    has_description = factory.Faker("pybool")
+    has_description = NautobotBoolIterator()
     description = factory.Maybe("has_description", factory.Faker("text", max_nb_chars=200), "")
 
-    has_parent = factory.Faker("pybool")
+    has_parent = NautobotBoolIterator()
     parent = factory.Maybe("has_parent", random_instance(TenantGroup), None)
 
 
@@ -28,16 +33,16 @@ class TenantFactory(PrimaryModelFactory):
         exclude = (
             "has_comments",
             "has_description",
-            "has_group",
+            "has_tenant_group",
         )
 
     name = UniqueFaker("company")
 
-    has_comments = factory.Faker("pybool")
+    has_comments = NautobotBoolIterator()
     comments = factory.Maybe("has_comments", factory.Faker("paragraph"), "")
 
-    has_description = factory.Faker("pybool")
+    has_description = NautobotBoolIterator()
     description = factory.Maybe("has_description", factory.Faker("text", max_nb_chars=200), "")
 
-    has_group = factory.Faker("pybool")
-    group = factory.Maybe("has_group", random_instance(TenantGroup), None)
+    has_tenant_group = NautobotBoolIterator()
+    tenant_group = factory.Maybe("has_tenant_group", random_instance(TenantGroup), None)
