@@ -2,7 +2,7 @@ import binascii
 import os
 
 from django.conf import settings
-from django.contrib.auth.models import AbstractUser, Group, UserManager
+from django.contrib.auth.models import AbstractUser, Group, UserManager as UserManager_
 from django.contrib.contenttypes.models import ContentType
 from django.core.serializers.json import DjangoJSONEncoder
 from django.core.validators import MinLengthValidator
@@ -10,10 +10,9 @@ from django.db import models
 from django.db.models import Q
 from django.utils import timezone
 
-from nautobot.core.models import BaseModel
-from nautobot.utilities.fields import JSONArrayField
-from nautobot.utilities.querysets import RestrictedQuerySet
-from nautobot.utilities.utils import flatten_dict
+from nautobot.core.models import BaseManager, BaseModel, CompositeKeyQuerySetMixin
+from nautobot.core.models.fields import JSONArrayField
+from nautobot.core.utils.data import flatten_dict
 
 
 __all__ = (
@@ -29,6 +28,23 @@ __all__ = (
 #
 
 
+class UserQuerySet(CompositeKeyQuerySetMixin, models.QuerySet):
+    """
+    Add support for composite-keys to the User queryset.
+
+    Note that this is *NOT* based around RestrictedQuerySet.
+    """
+
+
+class UserManager(BaseManager, UserManager_):
+    """
+    Natural-key subclass of Django's UserManager.
+    """
+
+    def get_queryset(self):
+        return UserQuerySet(self.model, using=self._db)
+
+
 class User(BaseModel, AbstractUser):
     """
     Nautobot implements its own User model to suport several specific use cases.
@@ -38,11 +54,13 @@ class User(BaseModel, AbstractUser):
 
     config_data = models.JSONField(encoder=DjangoJSONEncoder, default=dict, blank=True)
 
-    # We must use the stock UserManager instead of RestrictedQuerySet from BaseModel
+    # TODO: we don't currently have a general "Users" guide.
+    documentation_static_path = "docs/development/core/user-preferences.html"
     objects = UserManager()
 
     class Meta:
         db_table = "auth_user"
+        ordering = ["username"]
 
     def get_config(self, path, default=None):
         """
@@ -58,7 +76,7 @@ class User(BaseModel, AbstractUser):
 
         # Iterate down the hierarchy, returning the default value if any invalid key is encountered
         for key in keys:
-            if type(d) is dict and key in d:
+            if isinstance(d, dict) and key in d:
                 d = d.get(key)
             else:
                 return default
@@ -92,7 +110,7 @@ class User(BaseModel, AbstractUser):
         # Iterate through the hierarchy to find the key we're setting. Raise TypeError if we encounter any
         # interim leaf nodes (keys which do not contain dictionaries).
         for i, key in enumerate(keys[:-1]):
-            if key in d and type(d[key]) is dict:
+            if key in d and isinstance(d[key], dict):
                 d = d[key]
             elif key in d:
                 err_path = ".".join(path.split(".")[: i + 1])
@@ -102,7 +120,7 @@ class User(BaseModel, AbstractUser):
 
         # Set a key based on the last item in the path. Raise TypeError if attempting to overwrite a non-leaf node.
         key = keys[-1]
-        if key in d and type(d[key]) is dict:
+        if key in d and isinstance(d[key], dict):
             raise TypeError(f"Key '{path}' has child keys; cannot assign a value")
         else:
             d[key] = value
@@ -128,7 +146,7 @@ class User(BaseModel, AbstractUser):
         for key in keys[:-1]:
             if key not in d:
                 break
-            if type(d[key]) is dict:
+            if isinstance(d[key], dict):
                 d = d[key]
 
         key = keys[-1]
@@ -171,19 +189,23 @@ class Token(BaseModel):
     write_enabled = models.BooleanField(default=True, help_text="Permit create/update/delete operations using this key")
     description = models.CharField(max_length=200, blank=True)
 
+    documentation_static_path = "docs/user-guide/platform-functionality/users/token.html"
+    natural_key_field_names = ["pk"]  # default would be `["key"]`, which is obviously not ideal!
+
     class Meta:
-        pass
+        ordering = ["created"]
 
     def __str__(self):
         # Only display the last 24 bits of the token to avoid accidental exposure.
-        return "{} ({})".format(self.key[-6:], self.user)
+        return f"{self.key[-6:]} ({self.user})"
 
     def save(self, *args, **kwargs):
         if not self.key:
             self.key = self.generate_key()
         return super().save(*args, **kwargs)
 
-    def generate_key(self):
+    @staticmethod
+    def generate_key():
         # Generate a random 160-bit key expressed in hexadecimal.
         return binascii.hexlify(os.urandom(20)).decode()
 
@@ -205,9 +227,11 @@ class ObjectPermission(BaseModel):
     identified by ORM query parameters.
     """
 
-    name = models.CharField(max_length=100)
+    name = models.CharField(max_length=100, unique=True)
     description = models.CharField(max_length=200, blank=True)
     enabled = models.BooleanField(default=True)
+    # TODO: Remove pylint disable after issue is resolved (see: https://github.com/PyCQA/pylint/issues/7381)
+    # pylint: disable=unsupported-binary-operation
     object_types = models.ManyToManyField(
         to=ContentType,
         limit_choices_to=Q(
@@ -226,6 +250,7 @@ class ObjectPermission(BaseModel):
         ),
         related_name="object_permissions",
     )
+    # pylint: enable=unsupported-binary-operation
     groups = models.ManyToManyField(to=Group, blank=True, related_name="object_permissions")
     users = models.ManyToManyField(to=settings.AUTH_USER_MODEL, blank=True, related_name="object_permissions")
     actions = JSONArrayField(
@@ -239,7 +264,7 @@ class ObjectPermission(BaseModel):
         help_text="Queryset filter matching the applicable objects of the selected type(s)",
     )
 
-    objects = RestrictedQuerySet.as_manager()
+    documentation_static_path = "docs/user-guide/platform-functionality/users/objectpermission.html"
 
     class Meta:
         ordering = ["name"]
@@ -252,6 +277,6 @@ class ObjectPermission(BaseModel):
         """
         Return all constraint sets as a list (even if only a single set is defined).
         """
-        if type(self.constraints) is not list:
+        if not isinstance(self.constraints, list):
             return [self.constraints]
         return self.constraints
