@@ -1,15 +1,16 @@
 import base64
+from unittest import skip
 from django.contrib.auth import get_user_model
 from django.contrib.auth.models import Group
 from django.contrib.contenttypes.models import ContentType
 from django.urls import reverse
+from django.utils.timezone import now
+from rest_framework import HTTP_HEADER_ENCODING, status
 
+from nautobot.core.testing import APIViewTestCases, APITestCase, get_deletable_objects
+from nautobot.core.utils.data import deepmerge
 from nautobot.users.filters import GroupFilterSet
 from nautobot.users.models import ObjectPermission, Token
-from nautobot.utilities.testing import APIViewTestCases, APITestCase
-from nautobot.utilities.utils import deepmerge
-
-from rest_framework import HTTP_HEADER_ENCODING
 
 
 # Use the proper swappable User model
@@ -18,7 +19,6 @@ User = get_user_model()
 
 class AppTest(APITestCase):
     def test_root(self):
-
         url = reverse("users-api:api-root")
         response = self.client.get(f"{url}?format=api", **self.header)
 
@@ -27,35 +27,107 @@ class AppTest(APITestCase):
 
 class UserTest(APIViewTestCases.APIViewTestCase):
     model = User
-    brief_fields = ["display", "id", "url", "username"]
     validation_excluded_fields = ["password"]
     create_data = [
         {
-            "username": "User_4",
+            "username": "user_4",
             "password": "password4",
+            "is_superuser": True,
+            "is_staff": True,
+            "is_active": False,
+            "first_name": "Fourth",
+            "last_name": "User",
+            "email": "fourth.user@example.com",
+            "config_data": {
+                "tables": {
+                    "CircuitTable": {
+                        "columns": ["cid", "provider", "status", "tags"],
+                    },
+                },
+            },
+            "date_joined": now(),
         },
         {
-            "username": "User_5",
+            "username": "user_5",
             "password": "password5",
         },
         {
-            "username": "User_6",
-            "password": "password6",
+            "username": "user_6",
         },
     ]
 
     @classmethod
     def setUpTestData(cls):
+        user1 = User.objects.create(username="User_1")
+        user1.set_password(None)
+        user1.save()
+        user2 = User.objects.create(username="User_2")
+        user2.set_password(None)
+        user2.save()
+        user3 = User.objects.create(username="User_3")
+        user3.set_password(None)
+        user3.save()
+        group = Group.objects.create(name="Group 22")
+        cls.update_data = {
+            "username": "user_22",
+            "password": "password22",
+            "is_staff": False,
+            "is_superuser": False,
+            "is_active": True,
+            "groups": [group.pk],
+            "config_data": {
+                "tables": {
+                    "ProviderTable": {
+                        "columns": ["name", "asn", "tags"],
+                    },
+                },
+            },
+        }
 
-        User.objects.create(username="User_1")
-        User.objects.create(username="User_2")
-        User.objects.create(username="User_3")
+    def get_deletable_object(self):
+        """Get an instance that can be deleted, being sure not to delete the test user!"""
+        return User.objects.create(username="User_100")
+
+    def test_create_object(self):
+        """Add validation of the password on the created users."""
+        self.maxDiff = None
+        super().test_create_object()
+        for entry in self.create_data:
+            user = User.objects.get(username=entry["username"])
+            if "password" in entry:
+                self.assertTrue(user.check_password(entry["password"]))
+            else:
+                self.assertFalse(user.has_usable_password())
+
+    def test_recreate_object_csv(self):
+        """Add validation that the recreated user has no password."""
+        super().test_recreate_object_csv()
+        user = User.objects.get(username="User_100")
+        self.assertFalse(user.has_usable_password())
+
+    def test_update_object(self):
+        """Add validation that a partial update can change the password if requested."""
+        user = self._get_queryset().first()
+        super().test_update_object()
+        user.refresh_from_db()
+        self.assertTrue(user.check_password(self.update_data["password"]))
+        # Make sure the password *isn't* changed if we make a PATCH without a specified password
+        response = self.client.patch(self._get_detail_url(user), {}, format="json", **self.header)
+        self.assertHttpStatus(response, status.HTTP_200_OK)
+        self.assertNotIn("password", response.json())
+        user.refresh_from_db()
+        self.assertTrue(user.check_password(self.update_data["password"]))
+
+    def test_get_put_round_trip(self):
+        """Add validation that the password is cleared by a PUT with no specified password."""
+        super().test_get_put_round_trip()
+        user = self._get_queryset().first()
+        self.assertFalse(user.has_usable_password())
 
 
 class GroupTest(APIViewTestCases.APIViewTestCase):
     model = Group
     filterset = GroupFilterSet
-    brief_fields = ["display", "id", "name", "url"]
     create_data = [
         {
             "name": "Group 4",
@@ -78,7 +150,6 @@ class GroupTest(APIViewTestCases.APIViewTestCase):
 
     @classmethod
     def setUpTestData(cls):
-
         Group.objects.create(name="Group 1")
         Group.objects.create(name="Group 2")
         Group.objects.create(name="Group 3")
@@ -86,7 +157,6 @@ class GroupTest(APIViewTestCases.APIViewTestCase):
 
 class TokenTest(APIViewTestCases.APIViewTestCase):
     model = Token
-    brief_fields = ["display", "id", "key", "url", "write_enabled"]
     bulk_update_data = {
         "description": "New description",
     }
@@ -107,7 +177,7 @@ class TokenTest(APIViewTestCases.APIViewTestCase):
             username="basicusergranted", password=self.basic_auth_user_password
         )
 
-        obj_perm = ObjectPermission(name="Test permission", actions=["add", "change", "view", "delete"])
+        obj_perm = ObjectPermission(name="Token Test Permission", actions=["add", "change", "view", "delete"])
         obj_perm.save()
         obj_perm.users.add(self.basic_auth_user_granted)
         obj_perm.object_types.add(ContentType.objects.get_for_model(self.model))
@@ -128,9 +198,16 @@ class TokenTest(APIViewTestCases.APIViewTestCase):
             },
         ]
 
+    def get_deletable_object(self):
+        """Get an instance that can be deleted, being sure not to delete the test token!"""
+        instance = get_deletable_objects(self.model, self._get_queryset().exclude(pk=self.token.pk)).first()
+        if instance is None:
+            self.fail("Couldn't find a single deletable object!")
+        return instance
+
     def _create_basic_authentication_header(self, username, password):
         """
-        Given username, password create a valid Basic authenticaition header string.
+        Given username, password create a valid Basic authentication header string.
 
         Same procedure used to test DRF.
         """
@@ -270,21 +347,9 @@ class TokenTest(APIViewTestCases.APIViewTestCase):
 
 class ObjectPermissionTest(APIViewTestCases.APIViewTestCase):
     model = ObjectPermission
-    brief_fields = [
-        "actions",
-        "display",
-        "enabled",
-        "groups",
-        "id",
-        "name",
-        "object_types",
-        "url",
-        "users",
-    ]
 
     @classmethod
     def setUpTestData(cls):
-
         groups = (
             Group.objects.create(name="Group 1"),
             Group.objects.create(name="Group 2"),
@@ -312,7 +377,7 @@ class ObjectPermissionTest(APIViewTestCases.APIViewTestCase):
         cls.create_data = [
             {
                 "name": "Permission 4",
-                "object_types": ["dcim.site"],
+                "object_types": ["dcim.location"],
                 "groups": [groups[0].pk],
                 "users": [users[0].pk],
                 "actions": ["view", "add", "change", "delete"],
@@ -320,7 +385,7 @@ class ObjectPermissionTest(APIViewTestCases.APIViewTestCase):
             },
             {
                 "name": "Permission 5",
-                "object_types": ["dcim.site"],
+                "object_types": ["dcim.location"],
                 "groups": [groups[1].pk],
                 "users": [users[1].pk],
                 "actions": ["view", "add", "change", "delete"],
@@ -328,7 +393,7 @@ class ObjectPermissionTest(APIViewTestCases.APIViewTestCase):
             },
             {
                 "name": "Permission 6",
-                "object_types": ["dcim.site"],
+                "object_types": ["dcim.location"],
                 "groups": [groups[2].pk],
                 "users": [users[2].pk],
                 "actions": ["view", "add", "change", "delete"],
@@ -340,6 +405,22 @@ class ObjectPermissionTest(APIViewTestCases.APIViewTestCase):
             "description": "New description",
         }
 
+    def get_deletable_object(self):
+        return ObjectPermission.objects.create(
+            name="Permission 100",
+            actions=["view", "add", "change", "delete"],
+            constraints={"name": "TEST100"},
+        )
+
+    # TODO: Unskip after resolving #2908, #2909
+    @skip("DRF's built-in OrderingFilter triggering natural key attribute error in our base")
+    def test_list_objects_ascending_ordered(self):
+        pass
+
+    @skip("DRF's built-in OrderingFilter triggering natural key attribute error in our base")
+    def test_list_objects_descending_ordered(self):
+        pass
+
 
 class UserConfigTest(APITestCase):
     def test_get(self):
