@@ -1,6 +1,6 @@
 from django.urls import path
 
-from nautobot.extras.views import ObjectChangeLogView, ObjectNotesView
+from nautobot.extras.views import ObjectChangeLogView, ObjectDynamicGroupsView, ObjectNotesView
 from nautobot.ipam.views import ServiceEditView
 from . import views
 from .models import Cluster, ClusterGroup, ClusterType, VirtualMachine, VMInterface
@@ -25,28 +25,28 @@ urlpatterns = [
         name="clustertype_bulk_delete",
     ),
     path(
-        "cluster-types/<slug:slug>/",
+        "cluster-types/<uuid:pk>/",
         views.ClusterTypeView.as_view(),
         name="clustertype",
     ),
     path(
-        "cluster-types/<slug:slug>/edit/",
+        "cluster-types/<uuid:pk>/edit/",
         views.ClusterTypeEditView.as_view(),
         name="clustertype_edit",
     ),
     path(
-        "cluster-types/<slug:slug>/delete/",
+        "cluster-types/<uuid:pk>/delete/",
         views.ClusterTypeDeleteView.as_view(),
         name="clustertype_delete",
     ),
     path(
-        "cluster-types/<slug:slug>/changelog/",
+        "cluster-types/<uuid:pk>/changelog/",
         ObjectChangeLogView.as_view(),
         name="clustertype_changelog",
         kwargs={"model": ClusterType},
     ),
     path(
-        "cluster-types/<slug:slug>/notes/",
+        "cluster-types/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
         name="clustertype_notes",
         kwargs={"model": ClusterType},
@@ -73,28 +73,28 @@ urlpatterns = [
         name="clustergroup_bulk_delete",
     ),
     path(
-        "cluster-groups/<slug:slug>/",
+        "cluster-groups/<uuid:pk>/",
         views.ClusterGroupView.as_view(),
         name="clustergroup",
     ),
     path(
-        "cluster-groups/<slug:slug>/edit/",
+        "cluster-groups/<uuid:pk>/edit/",
         views.ClusterGroupEditView.as_view(),
         name="clustergroup_edit",
     ),
     path(
-        "cluster-groups/<slug:slug>/delete/",
+        "cluster-groups/<uuid:pk>/delete/",
         views.ClusterGroupDeleteView.as_view(),
         name="clustergroup_delete",
     ),
     path(
-        "cluster-groups/<slug:slug>/changelog/",
+        "cluster-groups/<uuid:pk>/changelog/",
         ObjectChangeLogView.as_view(),
         name="clustergroup_changelog",
         kwargs={"model": ClusterGroup},
     ),
     path(
-        "cluster-groups/<slug:slug>/notes/",
+        "cluster-groups/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
         name="clustergroup_notes",
         kwargs={"model": ClusterGroup},
@@ -122,6 +122,12 @@ urlpatterns = [
         name="cluster_changelog",
         kwargs={"model": Cluster},
     ),
+    path(
+        "clusters/<uuid:pk>/dynamic-groups/",
+        ObjectDynamicGroupsView.as_view(),
+        name="cluster_dynamicgroups",
+        kwargs={"model": Cluster},
+    ),
     path(
         "clusters/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
@@ -190,6 +196,12 @@ urlpatterns = [
         name="virtualmachine_changelog",
         kwargs={"model": VirtualMachine},
     ),
+    path(
+        "virtual-machines/<uuid:pk>/dynamic-groups/",
+        ObjectDynamicGroupsView.as_view(),
+        name="virtualmachine_dynamicgroups",
+        kwargs={"model": VirtualMachine},
+    ),
     path(
         "virtual-machines/<uuid:pk>/notes/",
         ObjectNotesView.as_view(),
